//    Copyright (C) 2003 Chad Brassil
//    This file is part of NFP.
//
//    NFP is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Foobar is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


/******************************************************************************
 *
 * Copyright (c) 1996-2000 Palm, Inc. or its subsidiaries.
 * All rights reserved.
 *
 * File: DateMonth.c
 *
 * Release: Palm OS SDK 4.0 (63220)
 *
 * Description:
 *	  This is the Datebook application's month view module.
 *
 * History:
 *		April 8, 1996	Created by Art Lamb
 *
 *****************************************************************************/

#include <PalmOS.h>
#include <PalmUtils.h>
#include <SonyTypes.h>
#include <SonySystemPublic.h>
#include <SonyHRLib.h>

#include "NFP.h"
#include "NFPRsc.h"
#include "DateMonth.h" //originally was include Datebook.h


//extern void PrintApptRecord (DmOpenRef dbP, UInt16 index);

//I moved these here
UInt16				StartDayOfWeek;
//Boolean				TimeDisplayed;
//UInt32				TimeDisplayTick;
RGBColorType	 	colorLine;
Boolean				EventInCurrentView;
//DateType			Date;
TimeFormatType		TimeFormat;
static UInt8 phaseStor[42];


void MonthDrawTodayEffect (const RectangleType *rP);

/***********************************************************************
 *
 *	Internal Constants
 *
 ***********************************************************************/
#define linesInMonthPlusTitle			7
#define maxWeeksInMonth					6

// Day number's margins
#define textLeftMargin					3
#define textTopMargin					1


// Event indicators
#define lunchStartTime					0x0b1e		// 11:30 am
#define lunchMidTime						0x0c0f		// 12:15 pm
#define lunchEndTime						0xd00			//  1:00 pm

#define timeIndicatorWidth				2
#define timeIndicatorTopMargin		3
#define timeIndicatorRightMargin		2

#define untimeIndicatorTopMargin		13
#define untimeIndicatorRightMargin	5

#define repeatIndicatorBottomMargin	2


#define morningHeight					4
#define lunchHeight						3
#define afternoonHeight					4

#define drawDaySelected					true
#define dontDrawDaySelected			false

#define TodayIsVisible(monthP,today) ((monthP->month == today.month) && (monthP->year == today.year))

/***********************************************************************
 *
 *	Internal Structutes
 *
 ***********************************************************************/
typedef struct {
	RectangleType			bounds;
	Int16						month;			// month displayed
	Int16						year;				// year displayed
	DateTimeType			selected;
} MonthType;

typedef MonthType * MonthPtr;


/***********************************************************************
 *
 * FUNCTION:    FirstDayOfMonth
 *
 * DESCRIPTION: This routine returns the day-of-the-week, adjusted by the 
 *              preference setting that specifies the first day-of-
 *              the-week.  If the date passed is a Tuesday and the 
 *              start day of week is Monday, this routine will return
 *              a value of one.
 *
 * PARAMETERS:  monthP     - pointer to MontnType structure
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/8/96	Initial Revision
 *
 ***********************************************************************/
static UInt16 SECT4 FirstDayOfMonth (MonthPtr monthP)
	{
	return (DayOfWeek (monthP->month, 1, monthP->year) - 
			StartDayOfWeek + daysInWeek) % daysInWeek;
	}


/***********************************************************************
 *
 * FUNCTION:    MonthDrawInversionEffect
 *
 * DESCRIPTION: This routine does an inversion effect by swapping colors
 *					 this is NOT undoable by calling it a second time, rather
 *					 it just applies a selected look on top of already
 *					 rendered data.  (It's kind of a hack.)
 *
 * PARAMETERS:	 rP - pointer to a rectangle to 'invert'
 *					 cornerDiam - corner diameter
 *					 
 * RETURNED:	 nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			bob	10/28/99	Initial revision
 *			jmp	11/14/99	Modified for use with Datebook's MonthView (i.e., backcolor
 *								changed to UIObjectFill).
 *
 ***********************************************************************/
//if select == true then select, if false then deselect
//this is needed in release mode so colors don't get swapped in a wierd way
void MonthDrawInversionEffect (const RectangleType *rP, UInt16 cornerDiam, UInt8 phase, Boolean select) 
{
	RGBColorType	color;
	
	if(colorEnabled)
	{
		WinPushDrawState();
		WinSetDrawMode(winSwap);
		WinSetPatternType (blackPattern);

		if (!select && UIColorGetTableEntryIndex(UIObjectSelectedFill) != UIColorGetTableEntryIndex(UIObjectForeground))
		{

			WinSetBackColor(UIColorGetTableEntryIndex(UIObjectForeground));
			WinSetForeColor(UIColorGetTableEntryIndex(UIObjectSelectedForeground));
			WinPaintRectangle(rP, cornerDiam);
		}		

//		WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
		color = PhaseColor (phase);
		WinSetBackColor(WinRGBToIndex(&color));
		WinSetForeColor(UIColorGetTableEntryIndex(UIObjectSelectedFill));
		WinPaintRectangle(rP, cornerDiam);
	
		if (select && UIColorGetTableEntryIndex(UIObjectSelectedFill) != UIColorGetTableEntryIndex(UIObjectForeground))
		{

			WinSetBackColor(UIColorGetTableEntryIndex(UIObjectForeground));
			WinSetForeColor(UIColorGetTableEntryIndex(UIObjectSelectedForeground));
			WinPaintRectangle(rP, cornerDiam);
		}	
		
		WinPopDrawState();
	}
	else
		WinInvertRectangle(rP, cornerDiam);

}

void MonthDrawTodayEffect (const RectangleType *rP) 
{
	RGBColorType	color;
	
	if(colorEnabled)
	{
		WinPushDrawState();
		WinSetDrawMode(winSwap);
		WinSetPatternType (blackPattern);

		color.r = 175;
		color.g = 175;
		color.b = 255;
		WinSetBackColor(WinRGBToIndex(&color));
		WinSetForeColor(UIColorGetTableEntryIndex(UIObjectFill));
		WinPaintRectangle(rP, 0);
		
		WinPopDrawState();
	}
	else
		WinInvertRectangle(rP, 0);
}


void DrawDayColor (RectanglePtr dayR, UInt8 phase) 
{
	RGBColorType 	color;
	
	if(colorEnabled)
	{
		WinPushDrawState();
		
		WinSetDrawMode(winSwap);
		WinSetPatternType (blackPattern);
		WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
		//WinSetForeColor(UIColorGetTableEntryIndex(UIObjectSelectedFill));
		
		//this should only be done if device is explicitly color, otherwise may get
		//bad greyscale results.  Figure out switch.
		color = PhaseColor (phase);
		WinSetForeColor(WinRGBToIndex(&color));
		WinPaintRectangle(dayR, 0);
		
		WinPopDrawState();
	}

}

/***********************************************************************
 *
 * FUNCTION:    GetDayBounds
 *
 * DESCRIPTION: This routine returns the bounds of the specified day
 *
 * PARAMETERS:  monthP  - pointer to month object
 *              day     - day of month
 *              r       - returned: bounds of the day     
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/10/96	Initial Revision
 *
 ***********************************************************************/
static void SECT4 GetDayBounds (MonthPtr monthP, UInt16 day, RectanglePtr r)
	{
	UInt16 cell;
	Int16	cellWidth;
	Int16 cellHeight;

	cell = day - 1 + FirstDayOfMonth (monthP);
	cellWidth = monthP->bounds.extent.x / daysInWeek;
	cellHeight = monthP->bounds.extent.y / linesInMonthPlusTitle;
	r->topLeft.x = monthP->bounds.topLeft.x + (cell % daysInWeek) * cellWidth;
	r->topLeft.y = monthP->bounds.topLeft.y + ((cell / daysInWeek) + 1) * cellHeight;
	r->extent.x = cellWidth;
	r->extent.y = cellHeight;
	}
	

/***********************************************************************
 *
 * FUNCTION:    GetDaySelectionBounds
 *
 * DESCRIPTION: This routine returns the selection bounds of the specified day
 *
 * PARAMETERS:  monthP  - pointer to month object
 *              day     - day of month
 *              r       - returned: bounds of the day selection 
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			jmp	11/14/99	Initial Revision
 *			jmp	11/30/99	Grrr... swapped meaning of "cell" and "day" and
 *								broke selection of Sunday vs. Monday start-of-week
 *								selection.  Fixed that!
 *
 ***********************************************************************/
static void GetDaySelectionBounds(MonthPtr monthP, UInt16 day, RectanglePtr r)
{
	UInt16 cell;
	Int16	cellWidth;
	Int16 cellHeight;
	Int16 charWidth;
	
	cell = day - 1 + FirstDayOfMonth (monthP);
	cellWidth = monthP->bounds.extent.x / daysInWeek;
	cellHeight = monthP->bounds.extent.y / linesInMonthPlusTitle;
	charWidth = FntCharWidth('0');
	r->topLeft.x = monthP->bounds.topLeft.x + (cell % daysInWeek) * cellWidth;
	r->topLeft.y = monthP->bounds.topLeft.y + ((cell / daysInWeek) + 1) * cellHeight;
	r->extent.x = ((day < 10) ? charWidth : charWidth * 2) + (textLeftMargin * 2) - 1;//Chad added - 1 so doesn't interfer with my codes.
	r->extent.y = FntLineHeight () + (textTopMargin * 2);
	RctInsetRectangle (r, 1);
}

/***********************************************************************
 *
 * FUNCTION:    DrawAppointment
 *
 * DESCRIPTION: This routine draw an appointment indicator.
 *
 * PARAMETERS:  dayR  - bounds of the day's drawing region
 *              start - appointment's start time
 *              end   - appointment's end time
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/8/96	Initial Revision
 *
 ***********************************************************************/

static void SECT4 DrawAppointment (RectanglePtr dayR, Int8 numberOfData, UInt8 phase, UInt8 sex)
{
	Int16 x, y;
	FontID oldFont;
 	Char *phaseLabels;
 	Char *label;
 	BitmapType *closedHeart;
 	BitmapType *openHeart;
 	MemHandle h1;
 	MemHandle h2;
//	RectangleType r;
	UInt16 refNum;
//	Err error;

	// Draw the indicator for Data Collection
		x = dayR->topLeft.x + untimeIndicatorRightMargin;
		y = dayR->topLeft.y + untimeIndicatorTopMargin + 1;
	
		//will cascade through all lower numbers
		//0 means some data collected, but not predictive
		switch(numberOfData)
		{
			case 13:
			case  3:
				WinDrawLine (x - 2, y + 2, x + 2 , y + 2);			

			case 12:
			case  2:
				WinDrawLine (x + 2, y + 2, x, y - 2);			

			case 11:
			case  1:
				WinDrawLine (x - 2, y + 2, x, y - 2);

			case -1:
			
			default:
				break;
		}
		
		switch(numberOfData/10)
		{
			case 1:
				WinDrawLine (x, y, x, y + 1);

			default:
				break;
		}
		
	// Draw the indicator for an timed appointment.
	oldFont = FntSetFont (stdFont);
	
	h1 = DmGetResource (strRsc, PhaseLabelsString);
	phaseLabels = (Char*)MemHandleLock (h1);
	
	x = dayR->topLeft.x + dayR->extent.x - 5 - 
		timeIndicatorRightMargin;
	y = dayR->topLeft.y + timeIndicatorTopMargin + 4;


	switch(phase)
	{
		case 0:
			break;
		case 1:
			label = &phaseLabels[0];
			WinDrawChars(label,1,x-1,y);
			break;
		case 2:
			label = &phaseLabels[1];
			WinDrawChars(label,1,x,y);
			break;
		case 3:
			label = &phaseLabels[2];
			WinDrawChars(label,1,x,y);
			break;
		case 4:
			label = &phaseLabels[3];
			WinDrawChars(label,1,x,y);
			break;
		case 5:
			label = &phaseLabels[4];
			WinDrawChars(label,1,x,y);
			break;
		case 6:
			label = &phaseLabels[5];
			WinDrawChars(label,1,x,y);
			break;						
		default:
			break;
	}
	
	MemPtrUnlock(phaseLabels);
	DmReleaseResource(h1);
	FntSetFont(oldFont);
	
	//Draw Sex indicator
	if(sex)
	{
		x = dayR->topLeft.x + dayR->extent.x - 4 - 
			timeIndicatorRightMargin;
		y = dayR->topLeft.y + timeIndicatorTopMargin - 1;
	
		if(hrEnabled)
		{
			SysLibFind(sonySysLibNameHR, &refNum);
			//I failed to access these using Bitmap Calls, I had to make another family
			h1 = DmGetResource (bitmapRsc, HROpenHeartBitmapFamily);
			h2 = DmGetResource (bitmapRsc, HRClosedHeartBitmapFamily);
		}
		else
		{
			h1 = DmGetResource (bitmapRsc, OpenHeartBitmapFamily);
			h2 = DmGetResource (bitmapRsc, ClosedHeartBitmapFamily);
		}
		
		//error = DmGetLastErr();
		//ErrAlert(error);
		
		//if(!error)
		//{	
		openHeart = (BitmapType*)MemHandleLock (h1);
		closedHeart = (BitmapType*)MemHandleLock (h2);
		//}
	
		switch(sex)
		{
			case 0:
				break;

			case 1:
				if(hrEnabled)
					HRWinDrawBitmap(refNum,closedHeart,2*x,2*y);
				else
					WinDrawBitmap (closedHeart,x,y);
				break;

			case 2:
				if(hrEnabled)
					HRWinDrawBitmap (refNum,openHeart,2*x,2*y);
				else
					WinDrawBitmap (openHeart,x,y);
				break;
	
			default:
				break;
		}
	
		MemPtrUnlock(openHeart);
		MemPtrUnlock(closedHeart);
		DmReleaseResource(h1);
		DmReleaseResource(h2);
	}
	
	return;

/*	// Draw indicator for a morning appointment.
	if (TimeToInt(start) < lunchStartTime)
		{
		r.topLeft.x = x;
		r.topLeft.y = y;
		r.extent.x = timeIndicatorWidth;
		r.extent.y = morningHeight;
		WinDrawRectangle (&r, 0);
		}

	// Draw indicator for a lunch appointment.
	if ((TimeToInt(start) >= lunchStartTime && TimeToInt(start) < lunchEndTime) ||
		 (TimeToInt(start) < lunchStartTime && TimeToInt(end) > lunchMidTime))
		{
		r.topLeft.x = x;
		r.topLeft.y = y + morningHeight  + 1;
		r.extent.x = timeIndicatorWidth;
		r.extent.y = lunchHeight;
		WinDrawRectangle (&r, 0);
		}

	// Draw indicator for a afternoon appointment.
	if (TimeToInt(start) >= lunchEndTime ||
		(TimeToInt(start) < lunchEndTime && TimeToInt(end) > lunchEndTime))
		{
		r.topLeft.x = x;
		r.topLeft.y = y + morningHeight + lunchHeight + 2;
		r.extent.x = timeIndicatorWidth;
		r.extent.y = afternoonHeight;
		WinDrawRectangle (&r, 0);
		}
*/
}


/***********************************************************************
 *
 * FUNCTION:    DrawTimedAppointments
 *
 * DESCRIPTION: This routine draws appointment indicators for all the 
 *              appointment in a month.
 *
 * PARAMETERS:  monthP  - month object
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/10/96	Initial Revision
 *
 ***********************************************************************/

static void SECT4 DrawRecords (MonthPtr monthP)
{
	DateType date;
//	Boolean sameMonth;
	RectangleType dayR;
	UInt16 index;
	Boolean isOldRecord;
	NFPDBRecord* item;
	Int8 numberOfData;
//	DateType focalDate;
	UInt8 phase;
	UInt8 sexresult;
	Boolean dataExists;
	Boolean beyondMaxIndex;
	Int16 totalDays;


	date.day = 1;
	date.month = monthP->month;
	date.year = monthP->year - firstYear;
	totalDays = DaysInMonth(date.month, date.year);	
	
	// Find the first non-repeating appointment of the day.
	index = s_currentRecordNumber; //used to be 0
	isOldRecord = FindRecordFromDate(date, &index, &dataExists, &beyondMaxIndex);
	item = DataQueryData(index);  //will be null if beyond max index

	//cycle through all days of the month
	for(Int16 i=1; i<=totalDays; i++)
	{
		date.day = i;
		GetDayBounds (monthP, date.day, &dayR);
		if( item && !DateTypeCmp(item->recDate, date) )//first check to see if item is NULL
		{
			//calculate number of data
			numberOfData = 0;
			if(item->basalTemp != 0 || item->time !=4) numberOfData++;
			if(item->cervix != 0) numberOfData++;
			if(item->fluidSensation != 0 || item->fluidColor !=0) numberOfData++;
			if(item->LHTest !=0 && numberOfData < 3) numberOfData++;
			if(item->alcohol !=0 || item->other !=0 || item->note != NULL) numberOfData += 10;
			sexresult = item->sex;
			DataReleaseData();
			
			phase = DeterminePhase(date, index, true, true, false, NULL, true, NULL);
						
			//get next data record
			index++;
			item = DataQueryData(index);
		}
		
		else
		{
			numberOfData = -1;
			
			DataReleaseData();  //Have to release data so that DeterminePhase can use it
			phase = DeterminePhase(date, index, false, dataExists, !item, NULL, true, NULL);
			sexresult = 0;
			item = DataQueryData(index); //Then pick it back up
		}
	
	
		DrawAppointment (&dayR, numberOfData, phase, sexresult);
		DrawDayColor (&dayR, phase);
		phaseStor[date.day] = phase;
	
	}

	DataReleaseData();
	
	//so that the you don't have to start from the begining, record index into global variable
	if(index > 0)
		s_currentRecordNumber = index - 1;  //subtract one in case went past actual number of records
	
	return;

	
	
/*	
	while (true)
		{	
		item = DataQueryData(index);
		if (! item) break;


		// Check if the appointment is on the same month
		sameMonth = (item->recDate.month == monthP->month) && 
						(item->recDate.year == monthP->year - firstYear); //Double Check this firstYear thing
		DataReleaseData();

		if (sameMonth)
			{
			DataQueryData(index);
			GetDayBounds (monthP, item->recDate.day, &dayR);
			numberOfData = 0;
			if(item->basalTemp != 0) numberOfData++;
			if(item->cervix != 0) numberOfData++;
			if(item->fluidSensation != 0) numberOfData++;
			focalDate = item->recDate;
			DataReleaseData();
			
			//in this case, from a real record, so fill in Booleans
			phase = DeterminePhase(focalDate, index, true, true, false);
			
			DrawAppointment (&dayR, numberOfData, phase);
			}
		
		if (! sameMonth) break;

		// Get the next record.
		index++;
		}
*/
}

/***********************************************************************
 *
 * FUNCTION: 	 DrawMonth
 *
 * DESCRIPTION: Draw the month object.
 *
 * PARAMETERS:  monthP - pointer to month object to draw
 *					 selectDay - if true, draw "today" selected if possible
 *
 * RETURNED:	 nothing
 *
 *	HISTORY:
 *		04/08/96	rsf	Created by Roger Flores.
 *		08/26/99	kwk	Use DateTemplateToAscii to get the DOW first letter.
 *		10/18/99	gap	Get DOW labels from daysOfWeekInitialsStrID resource and
 *							use StartDayOfWeek to determine which letter is start day.
 *		10/31/99	jmp	Eliminate WinInvertRectangle(); draw day as selected instead.
 *							Also, always call DrawAppointmentsInMonth(), as DrawMonth()
 *							was always followed by that call -- helps consolidate various
 *							drawing problems.
 *		11/14/99	jmp	Updated the drawing such that non-selectable areas of the MonthView
 *							are in standard Form colors while selectable areas use control-
 *							style colors.  Also, added selectDay argument to allow today's
 *							date to either by drawn selected or not.
 *
 ***********************************************************************/
static void SECT4 DrawMonth (MonthPtr monthP, Boolean selectDay)
	{
	Int16				drawX, drawY;
	Int16				cellWidth, cellHeight;
	Int16				charWidth;
	Int16				lastDay;
//	Char				dowTemplate[] = "^1s";
	Char				dayInAscii[3];
	UInt16			dow;
	Int16			i; //originally was UInt16, changed to satisfy prc-tools
	Int16				x, y;
	DateTimeType	today;
	RectangleType	r;
	UInt8 			dayOfWeek;
 	Char *			dayLabels;
	UInt16			labelLength;
	Char*				label;
	FontID			oldFont;
	FormType* 		frmP;
	
	if(colorEnabled)
	{
		WinPushDrawState();
		WinSetBackColor(UIColorGetTableEntryIndex(UIFormFill));
		WinSetForeColor(UIColorGetTableEntryIndex(UIObjectForeground));
		WinSetTextColor(UIColorGetTableEntryIndex(UIObjectForeground));
		
	}
	oldFont = FntSetFont (boldFont);
	
	// Make sure the "unselectable" parts of the MonthView are
	// drawn in the proper colors first.
	WinEraseRectangle (&monthP->bounds, 0);
	
	cellWidth = monthP->bounds.extent.x / daysInWeek;
	cellHeight = monthP->bounds.extent.y / linesInMonthPlusTitle;

	// Get the resource that contains the first letter of each day.
 	dayLabels = (Char*)MemHandleLock (DmGetResource (strRsc, DaysofweekinitialsString));

	// Calculate length of one item in string */
	labelLength = StrLen (dayLabels) / daysInWeek;

	// Draw the days of the week labels right justified to the number columns	
	// Be sure to draw the labels with respect to current setting of StartDayOfWeek. Somes locales
	// consider Monday the first day while others use Sunday.  There is also a user preference 
	// selection in Prefs app/Formats panelthat will allow the user to change first day of week.
	drawY = 	monthP->bounds.topLeft.y + (cellHeight - FntLineHeight()) / 2;
	for (i = 0; i <= daysInWeek; i++)
		{
		dayOfWeek = (i + StartDayOfWeek) % daysInWeek;
		label = &dayLabels[labelLength * dayOfWeek];
		drawX = monthP->bounds.topLeft.x + (cellWidth * i) + ((cellWidth - FntCharsWidth (label, labelLength)) / 2);
		WinDrawChars (label, labelLength, drawX, drawY);
		}

	// Unlock the day of week label resource now that we are done with it.
	MemPtrUnlock (dayLabels);
	
	// Set the background color behind the grid in control-style colors since
	// the grid is selectable.
	if(colorEnabled)
		WinSetBackColor(UIColorGetTableEntryIndex(UIObjectFill));
	RctCopyRectangle(&monthP->bounds, &r);
	r.topLeft.y += cellHeight;
	r.extent.y -= cellHeight;
	WinEraseRectangle (&r, 0);
	
	// Draw the grid.  Change the foreground color temporarily to get the right effect,
	// and put it back when we're done.
	if(colorEnabled)
		WinSetForeColor(WinRGBToIndex(&colorLine));
	x = monthP->bounds.topLeft.x;
	y = monthP->bounds.topLeft.y + cellHeight;
	for (i = 0; i < daysInWeek + 1; i++)
		{
		WinDrawLine (x, y, x, y + (cellHeight * maxWeeksInMonth));
		x += cellWidth;
		}

	x = monthP->bounds.topLeft.x;
	for (i = 0; i < maxWeeksInMonth + 1; i++)
		{
		WinDrawLine (x, y, x + (cellWidth * daysInWeek) , y);
		y += cellHeight;
		}
	if(colorEnabled)
		WinSetForeColor(UIColorGetTableEntryIndex(UIObjectForeground));

	// Draw the days of the month.
	FntSetFont (stdFont);
	charWidth = FntCharWidth('0');
	
	dow = FirstDayOfMonth (monthP);
	drawX = monthP->bounds.topLeft.x + (dow * cellWidth) + textLeftMargin;
	drawY = monthP->bounds.topLeft.y + cellHeight + textTopMargin;
	lastDay = DaysInMonth(monthP->month, monthP->year);
	for (i=1; i <= lastDay; i++, dow++)
		{
		if (dow == daysInWeek)
			{
			drawX = monthP->bounds.topLeft.x + textLeftMargin;
			drawY += cellHeight;
			dow = 0;
			}
		StrIToA (dayInAscii, i);
		WinDrawChars (dayInAscii, 
						  (i < 10) ? 1 : 2, 
//						  (i < 10) ? drawX + charWidth : drawX,
						  drawX,
						  drawY);
		drawX += cellWidth;
		}

	// Display a rectangle around today's day if it's visible.
	TimSecondsToDateTime(TimGetSeconds(), &today);
	if (TodayIsVisible(monthP, today) && selectDay)
		{
		GetDaySelectionBounds(monthP, today.day, &r);
		//MonthDrawInversionEffect (&r, 0, 0, true); //phase is 0 because have yet to color squares
		// set to purple instead, so that inversion works without having to draw off screen.
		MonthDrawTodayEffect(&r);
		
		
		}

//	DrawAppointmentsInMonth (monthP);
	DrawRecords(monthP);
	
	//return to old font manually for version prior to 3.5
	FntSetFont (oldFont);
	if(colorEnabled)
		WinPopDrawState ();
	
	//check to see if Pregnant Form needs to be displayed
	//has to be done here at end instead of in DeterminePhase to aviod double drawing month
	//resulting in odd colors
	if(s_displayPregnancyMessage)
	{
		frmP = FrmGetFormPtr (PregnantForm);
		s_displayedPregnancyMessage = true;
		s_displayPregnancyMessage = false;
		FrmDoDialog (frmP);
		FrmDeleteForm (frmP);
	}
	
	}


/***********************************************************************
 *
 * FUNCTION: 	 MapPointToItem
 *
 * DESCRIPTION: Return return the item at x, y.
 *
 * PARAMETERS:  x, y - coordinate
 *              r - the bounds of the item area (not the MTWTFSS area)
 *
 * RETURNED:	 item number (doesn't check for invalid bounds!)
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			roger	11/15/94	Initial Revision
 *
 ***********************************************************************/
static Int16 SECT4 MapPointToItem (Int16 x, Int16 y, RectanglePtr r)
	{
	Int16 itemNumber;

	itemNumber = daysInWeek * (((y - r->topLeft.y) /
		(r->extent.y / linesInMonthPlusTitle)) - 1);
	itemNumber += ((x - r->topLeft.x) / (r->extent.x / daysInWeek));
	return itemNumber;
	}


/***********************************************************************
 *
 * FUNCTION:    MonthViewGetMonthPtr
 *
 * DESCRIPTION: This routine returns a pointer to the month object.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:	 pointer to a MonthType structure
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/10/96	Initial Revision
 *			jmp	10/6/99	Replace FrmGetActiveForm() with FrmGetFormPtr();
 *
 ***********************************************************************/
static MonthPtr SECT4 MonthViewGetMonthPtr (void)
	{
	FormPtr frm;
	
	frm = FrmGetFormPtr (MonthViewForm);
	return MonthPtr (FrmGetGadgetData (frm, FrmGetObjectIndex (frm, MonthViewDaySelectorGadget)));
	}


/***********************************************************************
 *
 * FUNCTION:    MonthViewSetTitle
 *
 * DESCRIPTION: This routine set the title of the Month View
 *
 * PARAMETERS:  monthP  - pointer to month object (MonthType)
 *
 * RETURNED:	 nothing
 *
 * HISTORY:
 *		04/10/96	art	Created by Art Lamb.
 *		09/08/99	kwk	Use DateTemplateToAscii & load format from string resource.
 *		10/6/99	jmp	Replace FrmGetActiveForm() with FrmGetFormPtr();
 *		11/29/02	Chad completely change this to change the label
 *
 ***********************************************************************/
static void SECT4 MonthViewSetTitle (MonthPtr monthP)
	{
	Char* pString;
	FormType* frmP;
	UInt16 objectIndex;
	Err err;
	
	frmP = FrmGetActiveForm();
	objectIndex = FrmGetObjectIndex(frmP,MonthViewMonthLabel);
	FrmHideObject(frmP,objectIndex);
	
	pString = (Char*)MemPtrNew(longDateStrLength);
	DateToAscii(monthP->month, 1, monthP->year, dfDMYLongNoDay, pString);//leave this as dfDMYLongNoDay, not preferences
	FrmCopyLabel(frmP,MonthViewMonthLabel,pString);
	err = MemPtrFree(pString);

	FrmShowObject(frmP,objectIndex);
	
	}

/***********************************************************************
 *
 * FUNCTION:    MonthSelectDayRect
 *
 * DESCRIPTION: This routine selects or unselects day in the month
 *					 object.
 *
 * PARAMETERS:	 monthP   - pointer to control object (ControlType)
 *              r        - pointer to day's rectangle
 *					 selected - says whether to draw the rectangle selected
 *									or unselected
 *
 * RETURNED:	nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			jmp	10/31/99	Initial Revision.
 *			jmp	11/06/99	Oops, was just doing a WinResetClip() instead of
 *								save/restore WinSetClip().
 *			jmp	11/14/99	Added support for drawing the current day's selection
 *								unselected to get the right selection effect on
 *								"today."
 *			jmp	11/19/99	Use WinScreenLock()/WinScreenUnlock() to eliminate all of
 *								the ugly redrawing effects.
 *       jmp   12/02/99 Fix bug #24377.  Don't call WinScreenUnlock() if WinScreenLock()
 *                      fails.
 *                          
 ***********************************************************************/
/*
static void MonthSelectDayRect (MonthPtr monthP, RectangleType *r, Boolean selected)
{
	RectangleType savedClip;
	UInt8 * lockedWinP;
	UInt32			romVersion;
	UInt32			minROM;
	
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	minROM = sysMakeROMVersion(3,5,0,sysROMStageRelease,0);
	
	// Draw everything offscreen to begin with -- we have a lot to redraw, and
	// it's somewhat unsightly to see it all happen onscreen.  But still restrict
	// the clipping rectangle to get the effect we want.
	//
	if(romVersion >= minROM) lockedWinP = WinScreenLock(winLockCopy);
	WinGetClip(&savedClip);
	WinSetClip(r);
	
	// Always redraw unselected first.
	//
	DrawMonth(monthP, !selected);
	
	// If selection is desired, do that now.
	//
	if (selected)
	{
		RectangleType todayRect;
		DateTimeType today;

		// First, draw the entire rectangle selected.
		//
		MonthDrawInversionEffect(r, 0);
		
		// If we're drawing "today," then draw the
		// day selection itself unselected.
		//
		TimSecondsToDateTime(TimGetSeconds(), &today);
		GetDaySelectionBounds(monthP, today.day, &todayRect);
		
		if (TodayIsVisible(monthP, today) && RctPtInRectangle(r->topLeft.x, r->topLeft.y, &todayRect))
		{
			WinSetClip(&todayRect);
			DrawMonth(monthP, dontDrawDaySelected);
		}
	}

	// Now that we're done drawing, reset the clipping rectangle, and put everything we've done back
	// onscreen.
	//
	WinSetClip(&savedClip);
	if(romVersion >= minROM && lockedWinP)
		WinScreenUnlock();
}
*/

/***********************************************************************
 *
 * FUNCTION:    MonthSelectDay
 *
 * DESCRIPTION: This routine selects a day in the month object.
 *
 * PARAMETERS:	 monthP  - pointer to control object (ControlType)
 *              eventP  - pointer to an EventType structure.
 *              dateP   - returned: pointer to the date selected
 *
 * RETURNED:	true if the event was handle or false if it was not.
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/9/96	Initial Revision
 *			jmp	10/31/99	Eliminate WinInvertRectangle() by calling
 *								MonthSelectDayRect() instead.
 *
 ***********************************************************************/
static Boolean SECT4 MonthSelectDay (MonthPtr monthP, EventType* eventP, DatePtr dateP)
	{
	Int16				x, y;
	Int16       	item;
	Int16       	daysInMonth;
	Int16       	firstDayInMonth;
	Int16				cellWidth, cellHeight;
	Boolean			penDown;
	Boolean			selected;
	RectangleType	r;
	UInt8 phase;
//	DateType date;
//	UInt16 index;
//	Boolean isOldRecord;
//	Boolean dataExists;
//	Boolean beyondMaxIndex;
	
	firstDayInMonth = FirstDayOfMonth (monthP);
	daysInMonth = DaysInMonth (monthP->month, monthP->year);

	// Restrict the selection bounds.  The top should be moved down
	// to exclude selecting the day of week titles.  The bottom should
	// be moved up to the last week with a day in it.
	RctCopyRectangle (&(monthP->bounds), &r);
	r.topLeft.y += r.extent.y / linesInMonthPlusTitle;
	r.extent.y = (r.extent.y / linesInMonthPlusTitle) *
		(((firstDayInMonth + daysInMonth - 1) / 7) + 1);

	if (! RctPtInRectangle (eventP->screenX, eventP->screenY, &r))
		return false;

	// Compute the bounds of the day the pen is on.
	item = MapPointToItem (eventP->screenX, eventP->screenY, &monthP->bounds);

	cellWidth = monthP->bounds.extent.x / daysInWeek;
	cellHeight = monthP->bounds.extent.y / linesInMonthPlusTitle;
	r.topLeft.x = monthP->bounds.topLeft.x + (item % daysInWeek) * cellWidth + 1;
	r.topLeft.y = monthP->bounds.topLeft.y + ((item / daysInWeek) + 1) * cellHeight + 1;
	r.extent.x = cellWidth - 1;
	r.extent.y = cellHeight - 1;

	// Draw the selected day.

	//removed MonthSelectDayRect calls to speed up interface and reduce flashing
	//instead insert call to MonthDrawInversionEffect
//	MonthSelectDayRect (monthP, &r, selected);

	//added a bunch of stuff here to figure out the phase so I can determine the background color
	if (colorEnabled && (unsigned) (item - firstDayInMonth) < (unsigned) daysInMonth) 	//be sure is in current month, otherwise phase is 0
	{
//		date.day = item - firstDayInMonth + 1;
//		date.month = monthP->month;
//		date.year = monthP->year - firstYear;
		
//		index = 0;
//		isOldRecord = FindRecordFromDate(date, &index, &dataExists, &beyondMaxIndex);
//		phase = DeterminePhase(date, index, isOldRecord, dataExists, beyondMaxIndex, NULL);
		
		phase = phaseStor[item - firstDayInMonth + 1];
	}
	else
		phase = 0;
	selected	= true;
	MonthDrawInversionEffect (&r, 0, phase, selected);

	// Track the pen until it's released, highlight or unhighlight the
	// item as the pen move in and out of it.
	do 
		{
		PenGetPoint (&x, &y, &penDown);

		if (RctPtInRectangle (x, y, &r))
			{
			if (!selected)
				{
				selected = true;
//				MonthSelectDayRect (monthP, &r, selected);
				MonthDrawInversionEffect (&r, 0, phase, selected);
				}
			}

		else if (selected)
			{
			selected = false;
//			MonthSelectDayRect (monthP, &r, selected);
			MonthDrawInversionEffect (&r, 0, phase, selected);
			}

		} while (penDown);


	// Exit if the pen was released outside the item.
	if (!selected)
	{
		//MonthDrawInversionEffect (&r, 0, phase);
		return (false);
	}

	selected = false;
//	MonthSelectDayRect (monthP, &r, selected);
	MonthDrawInversionEffect (&r, 0, phase, selected);

	dateP->month = monthP->month;
	dateP->year = monthP->year - firstYear;

	// Is the selection outside of the month?
	if ((unsigned) (item - firstDayInMonth) >= (unsigned) daysInMonth)
		{
		// Move to the month before or after the current month.
		dateP->day = 1;
		DateAdjust (dateP, (item - firstDayInMonth));
		}
	else
		{
		dateP->day = item - firstDayInMonth + 1;
		}

	return true;
	}

/***********************************************************************
 *
 * FUNCTION:		MonthViewGotoDate
 *
 * DESCRIPTION:	This routine displays the date picker so that the 
 *						user can select a month to navigate to.  If the date
 *						picker is confirmed, the month selected month is 
 *                confirmed
 *
 *						This routine is called when a "go to" button is pressed.
 *              
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	5/20/96	Initial Revision
 *			jmp	10/31/99 Move WinEraseRectangle() & DrawAppointmentsInMonth()
 *								into DrawMonth() itself.
 *
 ***********************************************************************/
static void SECT4 MonthViewGotoDate (void)
	{
	Char *		title;
	MemHandle	titleH;
	MonthPtr 	monthP;
	Int16 		day;

	// Get the title for the date picker dialog box.
	titleH = DmGetResource (strRsc, GoToTitleString);
	title = (Char*)MemHandleLock (titleH);

	monthP = MonthViewGetMonthPtr ();

	// Display the date picker.
	day = 1;
	if (SelectDay (selectDayByMonth, &monthP->month, &day, &monthP->year, title))
		{
		MonthViewSetTitle (monthP);
		DrawMonth (monthP, drawDaySelected);
		}

	MemHandleUnlock (titleH);
	}



/***********************************************************************
 *
 * FUNCTION:    MonthViewScroll
 *
 * DESCRIPTION: This routine scrolls the month view foreward or backwards
 *              by a month.
 *
 * PARAMETERS:  monthP  - pointer to month object (MonthType)
 *              direction - winUp (backwards) or winDown (foreward)
 *
 * RETURNED:	 nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/8/96	Initial Revision
 *			jmp	10/31/99 Move WinEraseRectangle() & DrawAppointmentsInMonth()
 *								into DrawMonth() itself.
 *
 ***********************************************************************/
static void SECT4 MonthViewScroll (MonthPtr monthP, WinDirectionType direction)
{
	// Move backward one month.
	if (direction == winUp)
		{
		if (monthP->month > january)
			monthP->month--;
		else if (monthP->year > firstYear)
			{
			monthP->month = december;
			monthP->year--;
			}
		else
			return;
		}
		
	// Move foreward one month.
	else
		{
		if (monthP->month < december)
			monthP->month++;
		else if (monthP->year < lastYear)
			{
			monthP->month = january;
			monthP->year++;
			}
		else
			return;
		}
	
	//modify s_currentDate so that Chart can use it
	s_currentDate.day = 1;
	s_currentDate.month = monthP->month;
	s_currentDate.year = monthP->year - firstYear;
	
	MonthViewSetTitle (monthP);
	DrawMonth (monthP, drawDaySelected);
}


/***********************************************************************
 *
 * FUNCTION:    MonthViewDoCommand
 *
 * DESCRIPTION: This routine preforms the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    true if 
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	7/17/96	Initial Revision
 *			jmp	10/31/99 Move WinEraseRectangle() & DrawAppointmentsInMonth()
 *								into DrawMonth() itself.
 *
 ***********************************************************************/
/*
static Boolean MonthViewDoCommand (UInt16 command)
{
	MonthPtr monthP = MonthViewGetMonthPtr();
	
	// Preferences
	if (command == MonthPreferencesCmd)
		FrmPopupForm (PreferencesDialog);

	// Display Options
	else if (command == MonthDisplayOptionsCmd)
		FrmPopupForm (DisplayDialog);

	// About
	else if (command == MonthSecurityCmd)
		{
		DoSecurity();
		DrawMonth (monthP, drawDaySelected);
		}
		
	// About
	else if (command == MonthGetInfoCmd)
		AbtShowAbout (sysFileCDatebook);

	return (true);
}
*/

/***********************************************************************
 *
 * FUNCTION:    MonthViewInit
 *
 * DESCRIPTION: This routine initialize the month view.
 *
 * PARAMETERS:  frm - pointer of the Month View form
 *
 * RETURNED:	 nothing
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/8/96	Initial Revision
 *
 ***********************************************************************/
static void SECT4 MonthViewInit (FormPtr frm)
{
	UInt16 objIndex;
	Int16 width;
	MonthPtr monthP;
	UInt16 objectIndex;
	Char *pString;
	DateType today;
	Int16 cycleDay;
	UInt16 index;
	Boolean isOldRecord;
	Boolean dataExists;
	Boolean beyondMaxIndex;
	Err err;

	
	
	//set to temp pref so remembers to come back to this form
	temp_prefs.view = 1;

	// Create a month object, initialize it, and store a pointer to it in the 
	// month gadget of the Month View.
	monthP = (MonthPtr)MemPtrNew (sizeof (MonthType));
	if (monthP)
		{
		objIndex = FrmGetObjectIndex (frm, MonthViewDaySelectorGadget);

		FrmGetObjectBounds (frm, objIndex, &monthP->bounds);
		width = monthP->bounds.extent.x;
		monthP->bounds.extent.x = (width / daysInWeek * daysInWeek) + 1;
		monthP->bounds.topLeft.x = (width - monthP->bounds.extent.x) / 2;

		monthP->month = s_currentDate.month;
		monthP->year = s_currentDate.year + firstYear;

		FrmSetGadgetData (frm, objIndex, monthP);

		MonthViewSetTitle (monthP);
		}
		
	//set push button
	FrmSetControlGroupSelection(frm,1,MonthViewMonthSquarePushButton);
	
	//set Today
	DateSecondsToDate (TimGetSeconds (), &today);
	//do this to determine the cycle day
	index = 0;
	isOldRecord = FindRecordFromDate(today, &index, &dataExists, &beyondMaxIndex);
	DeterminePhase(today, index, isOldRecord, dataExists, beyondMaxIndex, &cycleDay, true, NULL);

	//set Today label
	objectIndex = FrmGetObjectIndex(frm,MonthViewTodayLabel);
	FrmHideObject(frm,objectIndex);
	pString = (Char*)MemPtrNew(maxStrIToALen);
	if(cycleDay != -1)
		StrIToA(pString,cycleDay);
	else
		StrCopy(pString,"");
		
	FrmCopyLabel(frm,MonthViewTodayLabel,pString);
	err = MemPtrFree(pString);
	FrmShowObject(frm,objectIndex); 
	
	return;
}


/***********************************************************************
 *
 * FUNCTION:    MonthViewHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the Month View
 *              of the Datebook application.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event was handled and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *			Name	Date		Description
 *			----	----		-----------
 *			art	4/8/96	Initial Revision
 *			CSS	06/22/99	Standardized keyDownEvent handling
 *								(TxtCharIsHardKey, commandKeyMask, etc.)
 *			rbb	6/2/99	Added button for Agenda view
 *			jmp	10/6/99	Fix bug #22497:  Perform a FrmDrawForm() before
 *								updating the rest of the form on frmUpdateEvents;
 *								also, replace FrmGetActiveForm() with FrmGetFormPtr()
 *								as that is more correct in these modern times of
 *								generating update events when bits fail to be allocated.
 *			jmp	10/31/99 Move WinEraseRectangle() & DrawAppointmentsInMonth()
 *								into DrawMonth() itself.
 *			Chad 11/29/02	Removed lots of extras
 *
 ***********************************************************************/
 Boolean MonthViewHandleEvent (EventType* event)
{
	FormPtr frm;
	DateType date;
	Boolean handled = false;
	MonthPtr monthP;
	DateType today;
    UInt16 cycleIndex;
    Boolean dataExists;
    CycleRecord* cycle;

	if (event->eType == penDownEvent)
		{
			monthP = MonthViewGetMonthPtr ();
			if (MonthSelectDay (monthP, event, &date))
			{
				DateSecondsToDate(TimGetSeconds(), &today);	//get today	
				if( DateTypeCmp(today,date) >= 0 )
				{
					s_currentDate = date;
					FrmGotoForm (DataEntryForm);			
				}
				else
				{
			        FrmAlert(NoFutureAlert);
			        //frm = FrmInitForm (NoFutureAlert);
		        	//FrmDoDialog (frm);                    
	    		    //FrmDeleteForm (frm);
				}
				handled = true;
			}
		}

	
	else if (event->eType == keyDownEvent)
		{
		if (TxtCharIsHardKey(event->data.keyDown.modifiers, event->data.keyDown.chr))
			{
//			DateSecondsToDate (TimGetSeconds (), &date);
//			s_currentDate = date;
//			if (EventInCurrentView ||
//				(event->data.keyDown.modifiers & poweredOnKeyMask))
//				FrmGotoForm (DataEntryForm);
//			handled = true;
//			EventInCurrentView = true;
			}
		else if (EvtKeydownIsVirtual(event))
			{
			if (event->data.keyDown.chr == vchrPageUp)
				{
				monthP = MonthViewGetMonthPtr ();
				MonthViewScroll (monthP, winUp);
				handled = true;
				}	
			else if (event->data.keyDown.chr == vchrPageDown)
				{
				monthP = MonthViewGetMonthPtr ();
				MonthViewScroll (monthP, winDown);
				handled = true;
				}
			
			EventInCurrentView = true;
			}
		}

	
	else if (event->eType == ctlSelectEvent)
		{
		switch (event->data.ctlSelect.controlID)
			{
			case MonthViewGoToButton:
				MonthViewGotoDate ();
				handled = true;
				break;
				
			case MonthViewChartViewPushButton:
			 	//set to end of month
				s_currentDate.day = DaysInMonth(s_currentDate.month, s_currentDate.year);
	
				//I used to have ChartView jump to the beginning of the cycle, 
				//but it makes more sense to either jump to the end of the cycle
				//because that is where the interesting stuff is or to keep the
				//last day of the month in Chart View because that is what the
				//user is interested in seeing.  I'm going to do the later
 
				  //find cycle that contains that day
				//FindCycleFromDate(s_currentDate, &cycleIndex, &dataExists);
	
				  //set currentDate to the start date of that cycle
				//cycle = CycleQueryCycle(cycleIndex);
				//if(cycle)
				//{
				//	s_currentDate = cycle->cycleStart;
				//	CycleReleaseCycle();
				//}
				FrmGotoForm(TempChartForm);
				handled = true;
				break;
				
			case MonthViewTodayViewPushButton:
		 		//set to today
				DateSecondsToDate (TimGetSeconds (), &s_currentDate);
		    	FrmGotoForm(DataEntryForm);
				handled = true;
				break;
			}
		}


	else if (event->eType == ctlRepeatEvent)
		{
		switch (event->data.ctlRepeat.controlID)
			{
			case MonthViewPrevMonthRepeating:
				monthP = MonthViewGetMonthPtr ();
				MonthViewScroll (monthP, winUp);
				break;
			
			case MonthViewNextMonthRepeating:
				monthP = MonthViewGetMonthPtr ();
				MonthViewScroll (monthP, winDown);
				break;
			}
		}


	else if (event->eType == menuEvent)
		{
			if(event->data.menu.itemID == ViewsCycleChart)
			{
			 	//set to end of month
				s_currentDate.day = DaysInMonth(s_currentDate.month, s_currentDate.year);
	
				//find cycle that contains that day
				FindCycleFromDate(s_currentDate, &cycleIndex, &dataExists);
	
				//set currentDate to the start date of that cycle
				cycle = CycleQueryCycle(cycleIndex);
				if(cycle)
				{
					s_currentDate = cycle->cycleStart;
					CycleReleaseCycle();
				}			
			}
		
			handled = MainFormDoCommand (event->data.menu.itemID);
		}
		

	else if (event->eType == frmOpenEvent)
		{
		frm = FrmGetFormPtr (MonthViewForm);
		MonthViewInit (frm);
		FrmDrawForm (frm);

		monthP = MonthViewGetMonthPtr ();
		DrawMonth (monthP, drawDaySelected);
		EventInCurrentView = false;
		handled = true;
		}


	else if (event->eType == frmUpdateEvent)
		{
		frm = FrmGetFormPtr (MonthViewForm);
		FrmDrawForm (frm);
		
		monthP = MonthViewGetMonthPtr ();
		DrawMonth (monthP, drawDaySelected);
		handled = true;		
		}

	else if (event->eType == frmCloseEvent)
		{
		monthP = MonthViewGetMonthPtr ();
		if (monthP)
			MemPtrFree (monthP);
		}

	return (handled);
}
