//    Copyright (C) 2003 Chad Brassil
//    This file is part of NFP.
//
//    NFP is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    NFP is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with NFP; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// NFP.h
//
// header file for NFP
//
// This wizard-generated code is based on code adapted from the
// stationery files distributed as part of the Palm OS SDK 4.0.
//
// Copyright (c) 1999-2000 Palm, Inc. or its subsidiaries.
// All rights reserved.

#ifndef NFP_H_
#define NFP_H_

// ********************************************************************
// Internal Structures
// ********************************************************************

typedef struct NFPPreferenceType
{
	//default marked with (d)
	UInt8 tempMethod; //1-(d)coverline, 2-average, 3-ext cover, 4-CCL full, 5-CCL strong, 6-CCL overall
	UInt8 postPeakDays; //number of days post peak before infertile stage; 3, 4-(d);
	UInt8 factorsForPostPeak; //out of temp, billings, cervix, and LHTest; 1, 2-(d), 3;
	UInt8 roetzerRule; //ie early days automatically considered infertile
					 	//0-don't use, 1-(d)automaticly adjust based on cycle history, 3, 5, 6;
	UInt8 dryDaysRule; //0-don't use, 1-(d)use
	
	UInt32 morningAlarmSound; //this is a UInt32 in datebook, called alarmSoundUniqueRecID
	DateTimeType morningAlarmTime;
	Boolean enableMorningAlarm;
	UInt8 morningAlarmRepeat;
	
	UInt32 eveningAlarmSound;
	DateTimeType eveningAlarmTime;
	Boolean enableEveningAlarm;
	UInt8 eveningAlarmRepeat;
	
	UInt8 tempUnits; //F = 1 or C = 2;
	
	//all of these are over the past 12 cycles
	UInt8 aveCycleLength; //(d=28)just store integer value because that is all I really use
	UInt8 shortestCycleLength; //(d=26)
	UInt8 aveLutealLength; //(d=14)
	
	//added in version 3
	UInt8 enablePassword;
	Char password[11]; //not sure if this is the correct way to do this //extra character for null
	UInt8 disableAlarms;  //disable during post-peak infertile phase
	
	//added in version 4
	UInt8 exportSeparator; //0-semicolon, 1-tab;
	
	UInt8 fluidIndicator; //0-double description, 1-Marquette, 2-Couple to Couple League
	UInt8 hormoneTest; //0-LH test, 1-clearplan
	UInt8 chartDisplay; //0-cervix position, 1-hormone test
	UInt8 presets; //0-custom, 1-defaults, 2-MM1, 3-MM2, 4-MM3 5-cclC 6-cclB 7-cclR 8-cclA
	UInt8 timeAdjustMode; //0-default of .1F/30min; 1-half rate of .1F/60min; (does not affect Celsius)
	
} NFPPreferenceType;

typedef struct NFPTempPreferenceType
{
	//default marked with (d)
	UInt8 view; //1(d) = month view, 2 = cycle chart
	UInt8 lastTemp; //last selected temp
	
	//added in version 3
	UInt32 lastDisplayedAlarm; //in seconds
} NFPTempPreferenceType;

// ********************************************************************
// Global variables
// ********************************************************************

extern NFPPreferenceType g_prefs;
extern NFPTempPreferenceType temp_prefs;

extern DateType s_currentDate;
extern DateType s_cycleStart;
extern DateType s_lastCycleCheckDate;
//extern UInt16 s_aveCycleLength;

extern UInt16 s_currentRecordNumber;
extern Boolean s_isOldRecord;
extern UInt16 s_lastCycleIndex;
extern Boolean s_displayedPregnancyMessage;
extern Boolean s_displayPregnancyMessage;

extern Boolean colorEnabled;
extern Boolean hrEnabled;

// ********************************************************************
// Internal Constants
// ********************************************************************

#define appFileCreator			'cjEB' //Registered on 12 Dec 2002
#define appName					"NFP"
#define appVersionNum			0x02 //don't actually appear to be using this anywhere
#define appPrefID				0x00
#define appPrefVersionNum		0x04

#define maxCycleCalcDays		60

#define sysAppLaunchFromAlarm 3500

#define defaultCycleLength		28
#define defaultShortestLength	23
#define defaultLutealLength		14


#define SAVEcycleStart				20
#define SAVEbillingsPeakDate		21
#define SAVEcervixPeakDate			22
#define SAVEtempPeakDate			23
#define SAVELHTestPeakDate			24
#define SAVEbillingsFertilityDate	25
#define SAVEpregnancyAchievedDate	26
#define	SAVEcoverlineStartDate		27
#define SAVEcoverlineLength			28
#define SAVEcoverlineTemp			29

//#define maxTempRecorded			27
#define numberOfFahrenheitTemps		46 //used to be 27; ie not counting "----"
#define numberOfCelsiusTemps		51 //used to be 21

//these lines used to be in NFPDatabase
#define RECORD_DB_TYPE  ('Data')
#define RECORD_DB_NAME  ("NFPdays")

#define RECORD_DB_TYPE2  ('Cycl')
#define RECORD_DB_NAME2  ("NFPcycles")

#define dayWidth					5

//can play with these to minimize size and maximize speed
//must 
#define SECT1    //don't put an attribute on section one so it resides in the default section
#define SECT2 __attribute__((section("sectwo")))
#define SECT3 __attribute__((section("secthree"))) //two and one got two big, so I started secthree
#define SECT4 __attribute__((section("secfour"))) //try leaving in section 2

// ********************************************************************
// Helper template functions
// ********************************************************************

// use this template like:
// ControlType *button; GetObjectPtr(button, MainOKButton);

template <class T>
void GetObjectPtr(/*typename*/ T * &ptr, UInt16 id)
{
	FormType * frmP;

	frmP = FrmGetActiveForm();
	ptr = static_cast<T *>(
		FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, id)));
}
/*old
template <class T>
void GetObjectPtr(typename T * &ptr, UInt16 id)
{
	FormType * frmP;

	frmP = FrmGetActiveForm();
	ptr = static_cast<T *>(
		FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, id)));
}
*/

// use this template like:
// ControlType *button = 
//     GetObjectPtr<ControlType>(MainOKButton);
/*
template <class T>
typename T * GetObjectPtr(UInt16 id)
{
	FormType * frmP;

	frmP = FrmGetActiveForm();
	return static_cast<T *>(
		FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, id)));
}
*/

//Actual structure used
typedef struct{
	DateType	recDate;
	UInt8 		basalTemp;
	UInt8		fluidSensation;
	UInt8		fluidColor;
	UInt8		cervix;
	UInt8		LHTest;
	UInt8		sex;
	UInt8		alcohol;
	UInt8		time;
	UInt8		other;
	Char*		note;
} NFPDataType;

//just for size calculation.  Include everything but note is just Char
typedef struct{
	DateType	recDate;
	UInt8 		basalTemp;
	UInt8		fluidSensation;
	UInt8		fluidColor;
	UInt8		cervix;
	UInt8		LHTest;
	UInt8		sex;
	UInt8		alcohol;
	UInt8		time;
	UInt8		other;
	Char		note;
} NFPDBRecord;


//Actual structure used
typedef struct{
	DateType	cycleStart;
	DateType	billingsPeakDate;
	DateType	cervixPeakDate;
	DateType	tempPeakDate;
	DateType	LHTestPeakDate;
	DateType	billingsFertilityDate;
	DateType	pregnancyAchievedDate;
	//the following three added in Cycles Database version 2
	DateType	coverlineStartDate;
	Int8		coverlineLength; //number of days to draw coverline forward from coverlineStartDate. 0 = no coverline exists
	Int8		coverlineTemp; //temp at which to draw coverline, positive => draw at that temp, negative => draw 0.5 degrees below that temp
	
} CycleRecord;

Boolean MainFormDoCommand(UInt16 command) SECT1;
void ForceQuit(void) SECT1;

//put side features into section two
Boolean PreferencesFormHandleEvent(EventType * eventP) SECT2;
Boolean SetPasswordFormHandleEvent(EventType * eventP) SECT2;
Boolean SetAlarmsFormHandleEvent(EventType * eventP) SECT2;
void SetPreferencesToForm( void ) SECT2;
void GetPreferencesFromForm( void ) SECT2;
void SetSummaryForm( FormType* frmP ) SECT2;

//put DataEntry stuff into section three
Boolean DataEntryFormHandleEvent(EventType * eventP) SECT3;
Boolean IsDataEntryNull(void) SECT3;
void SaveFormToStructure(NFPDataType*) SECT3;
void SetFormFromRecord(UInt16 index, Boolean isOldRecord, Boolean dataExists, Boolean beyondMaxIndex) SECT3;
UInt8 OKDataFunction( void ) SECT3;

//leave Chart and Phase stuff in section one
UInt8 DeterminePhase( DateType focalDate, UInt16 index, Boolean isOldRecord, Boolean dataExists, Boolean beyondMaxIndex, Int16 *cycleDay, Boolean allowGlobal, NFPPreferenceType* l_prefs ) SECT1;
Boolean TempChartFormHandleEvent(EventType * eventP) SECT4;
void CalcPhase( Boolean added ) SECT4;
RGBColorType PhaseColor (UInt8 phase) SECT4;

void CalcTemp ( void ) SECT4;
void CalcBillings( void ) SECT4;
UInt8 CalcAve(UInt8 *numberInAve) SECT4;

//Alarm prototypes 
void HandleAlarmTrigger( SysAlarmTriggeredParamType * cmdPBP ) SECT1;
Boolean DisplayAlarm( SysAlarmTriggeredParamType * cmdPBP ) SECT1;
void SetNFPAlarms(void) SECT1;
Int16 TimeDiff(DateTimeType a, DateTimeType b) SECT1;
void GoToView(void) SECT1;

//Chart prototypes
void PlotTemps(UInt16 cycleIndex,Boolean isOldCycle,UInt8 scrollNumber) SECT4;
Boolean ChartSelectDay(EventType* eventP,DateType* date,Int16* scroll) SECT4;
void ChartChangeCycle(Int8 direction) SECT4;
void ChartDeleteEntireCycle(void) SECT2;
void ChartExportCycle(void) SECT2;

//Database prototypes
void DataClose ( void ) SECT1;
void DataOpen ( void ) SECT1;
void DataFindClose(UInt16 *recordNum) SECT1;
Err DataNew ( NFPDataType* item,UInt16 *index ) SECT1;
Boolean DataSaveOld ( UInt16 *index ) SECT1;
Err DataDelete(UInt16 index) SECT1;
NFPDBRecord* DataGetData ( UInt16 uIndex ) SECT1;
NFPDBRecord* DataQueryData ( UInt16 uIndex ) SECT1;
MemHandle LockedHandle(void) SECT1;
void DataReleaseData ( void ) SECT1;
UInt16 DataGetCount ( void ) SECT1;
Boolean FindRecordFromDate(DateType,UInt16*,Boolean* dataExists, Boolean* beyondMaxIndex) SECT1;
Boolean FindRecordFromDateNonGlobal(DateType desiredDate, UInt16* index, Boolean* dataExists, Boolean* beyondMaxIndex, DmOpenRef db ) SECT1;
UInt16 DataGetCountNonGlobal ( DmOpenRef db ) SECT1;
NFPDBRecord* DataQueryDataNonGlobal ( UInt16 uIndex, DmOpenRef db, Boolean* isDirty) SECT1;
void DataReleaseDataNonGlobal (DmOpenRef db, NFPDBRecord* pDataStruct, UInt16 recordNum, Boolean isDirty) SECT1;
void CycleClose ( void ) SECT1;
void CycleOpen ( void ) SECT1;
Err CycleNew (  DateType startDate,UInt16 *index ) SECT1;
Err CycleSaveOld (UInt16 changeType, DateType changeDate,UInt16 *index,Int8 *value ) SECT1;
Err CycleDeleteCycle(UInt16 index) SECT1;
CycleRecord* CycleGetCycle ( UInt16 uIndex ) SECT1;
CycleRecord* CycleQueryCycle ( UInt16 uIndex ) SECT1;
CycleRecord* CycleQueryCycleNonGlobal ( UInt16 uIndex, DmOpenRef db, Boolean *isDirty ) SECT1;
void CycleReleaseCycle ( void ) SECT1;
void CycleReleaseCycleNonGlobal (DmOpenRef db, CycleRecord* pDataStruct, UInt16 recordNum, Boolean isDirty) SECT1;
UInt16 CycleGetCount ( void ) SECT1;
Boolean FindCycleFromDate(DateType,UInt16*,Boolean* dataExists) SECT1;
Boolean FindCycleFromDateNonGlobal(DateType desiredDate, UInt16* index, Boolean* dataExists, DmOpenRef db ) SECT1;

//Month Draw prototypes
void MonthDrawInversionEffect (const RectangleType *rP, UInt16 cornerDiam, UInt8 phase, Boolean selected) SECT4;
void DrawDayColor (RectanglePtr dayR, UInt8 phase) SECT4;

Int16 DateTypeCmp(DateType d1, DateType d2) SECT1;

//sounds
//void PlayAlarmSound(UInt32 uniqueRecID);
//void CreateMidiPickList(FormPtr frmP,ListPtr listP,UInt16 lstID, ListDrawDataFuncPtr funcP);
//void MidiPickListDrawItem (Int16 itemNum, RectanglePtr bounds, Char **unusedP);

//Find
void Search (FindParamsPtr findParams) SECT1;
void GoToItem (GoToParamsPtr goToParams, Boolean launchingApp) SECT1;
Boolean GotoEventHandler(EventType * eventP) SECT1;

//Export
Boolean ExportKey(	UInt16 cycleIndex, UInt16 itemStartIndex, UInt16 itemFinalIndex) SECT2;
UInt16 ExportConfirm() SECT2;

//extra byte is for null character of note string
#define sizeDBRecord (sizeof(NFPDBRecord) +1)

#define sizeCycleRecord (sizeof(CycleRecord))


#endif // NFP_H_

