//    Copyright (C) 2003 Chad Brassil
//    This file is part of NFP.
//
//    NFP is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Foobar is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <PalmOS.h>

// Handspring SDK headers
//#include <HSExt.h>
#include <SysEvtMgr.h>
#include "NFP.h"
#include "NFPRsc.h"

UInt16 s_currentRecordNumber;
//static UInt16 s_currentCycleNumber;
Boolean s_isOldRecord;


//*****************************************************************
//*****************************************************************

static DmOpenRef		s_dbNFP;  //need this
static MemHandle   	    s_hRec;	//need this
static NFPDBRecord*    	s_pDataStruct; //was s_pText //need this
static UInt16			s_recordNum = 0;  //need this
static Boolean			s_DataIsLocked;
static Boolean			s_isDirty;
static Boolean			s_isBusy;

//   DataGetCount:
//   Returns the count of records in the database
UInt16 DataGetCount ( void )
{
   LocalID dbID;
   static UInt16 uDataCount = 0;
   static UInt32 old_modNum = 0;
   UInt32 modNum;
   
   if ( s_dbNFP )
   {
	  DmOpenDatabaseInfo (s_dbNFP, &dbID, NULL, NULL, NULL, NULL);
	  DmDatabaseInfo (0, dbID, NULL, NULL, NULL, NULL, NULL, NULL, &modNum, NULL, NULL, NULL, NULL);
	  // DmNumRecordsInCategory is a slow function. Only call it on first run, or if DB has changed
	  if (old_modNum==0 || modNum!=old_modNum)
	  {
		  uDataCount = DmNumRecordsInCategory ( s_dbNFP, dmAllCategories );
		  old_modNum = modNum;
	  }
	}
    return uDataCount;
}

UInt16 DataGetCountNonGlobal ( DmOpenRef db )
{
   UInt16 uDataCount = 0;
   if ( db )
   {
      uDataCount = DmNumRecordsInCategory ( db, dmAllCategories );
   }
   return uDataCount;
}


//  FUNCTION: FindRecordFromDate
//
//  DESCRIPTION:  Give a date, see if there is a record number 
//
//  PARAMETERS:	desiredDate: date of interest
//				index: pass the desired starting index.  Wise use will speed up program.
//					Pointer returns either the index for the desiredDate, or the index
//					where the desiredDate should be inserted if there is no Date
//				dataExists: optional pointer (i.e. can be NULL) that return true if there are day records, false if there are no day records at all
//				beyondMaxIndex: optional pointer (i.e. can be NULL) that returns true if index is one greater than the actual number of records.
//					This would be the case if the desiredDate should be entered at the end of the database (i.e. the normal daily situation)
//
//  RETURNS: true if there is a record on that date, false if there is no record on that date
//		also returns on index, the location where record for that date should be inserted.
Boolean FindRecordFromDate(DateType desiredDate, UInt16* index, Boolean* dataExists, Boolean* beyondMaxIndex)
{
	Boolean isOldRecord;
	Boolean sameDirection;
	NFPDBRecord* item;
	Int16 dateDiff;
	Int16 change;
	UInt16 maxRecord;
//	Err err;
//	LocalID dbID;
	
//	dbID = DmFindDatabase(0, RECORD_DB_NAME);
//	err = DmDatabaseSize(0, dbID, &maxRecord, NULL, NULL);
	maxRecord = DataGetCount();
	

	if(maxRecord !=0)
	{
		//be sure index is not out of range
		if((*index) + 1 > maxRecord) *index = maxRecord-1;
	
		item = DataQueryData(*index);
		dateDiff=DateTypeCmp(desiredDate,item->recDate);
		DataReleaseData();
	
		if(dateDiff>0) change=1;
		else change=-1;
		sameDirection = true;
	
		while(dateDiff!=0 && ((change == 1 && (*index) + 1 < maxRecord ) || (change==-1 && *index > 0)) && sameDirection)
		{
			(*index) += change;
		
			item = DataQueryData(*index);
			dateDiff=DateTypeCmp(desiredDate,item->recDate);
			DataReleaseData();
		
			sameDirection = false;
			if(change==1 && dateDiff>0) sameDirection = true;
			if(change==-1 && dateDiff <0) sameDirection = true;
		}
	
		if(dateDiff==0) isOldRecord = true;
		else
		{
		 isOldRecord = false;
		 if(change==-1 && dateDiff>0) (*index)++;	//if back-up, move one forward for correct insertion
		 if(((*index) + 1) == maxRecord && change == 1 && dateDiff>0) (*index)++;  //if at max and want to keep going
		}
	}
	
	else
	{
		isOldRecord = false;
		//ensure index is 0
		*index = 0;
	}
	
	if(beyondMaxIndex)
	{
		if(*index > maxRecord)
			*beyondMaxIndex = true;
		else
			*beyondMaxIndex = false;
	
	}
	
	if(dataExists)
	{
		if(maxRecord>0)
			*dataExists = true;
		else
			*dataExists = false;
	}
	
		
	return isOldRecord;
}

Boolean FindRecordFromDateNonGlobal(DateType desiredDate, UInt16* index, Boolean* dataExists, Boolean* beyondMaxIndex, DmOpenRef db )
{
	Boolean isOldRecord;
	Boolean sameDirection;
	NFPDBRecord* item;
	Int16 dateDiff;
	Int16 change;
	UInt16 maxRecord;
	Boolean isDirty = false;
//	Err err;
//	LocalID dbID;
	
//	dbID = DmFindDatabase(0, RECORD_DB_NAME);
//	err = DmDatabaseSize(0, dbID, &maxRecord, NULL, NULL);

	maxRecord = DataGetCountNonGlobal(db);
	

	if(maxRecord !=0)
	{
		//be sure index is not out of range
		if((*index) + 1 > maxRecord) *index = maxRecord-1;
	
		item = DataQueryDataNonGlobal(*index, db, &isDirty);
		dateDiff=DateTypeCmp(desiredDate,item->recDate);
		DataReleaseDataNonGlobal (db, item, *index, isDirty);
	
		if(dateDiff>0) change=1;
		else change=-1;
		sameDirection = true;
	
		while(dateDiff!=0 && ((change == 1 && (*index) + 1 < maxRecord ) || (change==-1 && *index > 0)) && sameDirection)
		{
			(*index) += change;
		
			item = DataQueryDataNonGlobal(*index, db, &isDirty);
			dateDiff=DateTypeCmp(desiredDate,item->recDate);
			DataReleaseDataNonGlobal (db, item, *index, isDirty);
		
			sameDirection = false;
			if(change==1 && dateDiff>0) sameDirection = true;
			if(change==-1 && dateDiff <0) sameDirection = true;
		}
	
		if(dateDiff==0) isOldRecord = true;
		else
		{
		 isOldRecord = false;
		 if(change==-1 && dateDiff>0) (*index)++;	//if back-up, move one forward for correct insertion
		 if(((*index) + 1) == maxRecord && change == 1 && dateDiff>0) (*index)++;  //if at max and want to keep going
		}
	}
	
	else
	{
		isOldRecord = false;
		//ensure index is 0
		*index = 0;
	}
	
	if(beyondMaxIndex)
	{
		if(*index > maxRecord)
			*beyondMaxIndex = true;
		else
			*beyondMaxIndex = false;
	
	}
	
	if(dataExists)
	{
		if(maxRecord>0)
			*dataExists = true;
		else
			*dataExists = false;
	}
	
		
	return isOldRecord;
}

//PARAMETERS:  index points to 
//RETURNS: True if record found, false if not found
//			if true than index of record with given date on index pointer
//			if false than index of before which record should be inserted
//Boolean FindRecordByDate(UInt16 *index, UInt16 searchIndex, Boolean backDirection)
//{

//}


Err DataNew ( NFPDataType *item,UInt16 *index )
{
	Err			err;
	NFPDBRecord* nilP=0; 
	MemPtr 		recordP;  	
	MemHandle 	recordH;
//	MemHandle*  oldRecordH;
	UInt16		size;
	Char		zero=0;
	UInt16		newIndex;
	
	//  Compute the size of the new to do record
	size = sizeDBRecord;
	if(item->note) size += StrLen (item->note);
	
	//Allocate a chunk in the database for the new record
	recordH = (MemHandle)DmNewHandle(s_dbNFP,size);
	if (recordH == NULL)
		return dmErrMemError;
		
		
	//Pack the data into the new record
	//Use nilP to determine offset location and size of record
	recordP = MemHandleLock(recordH);
	
	DmWrite(recordP, (UInt32)&nilP->recDate, &item->recDate, sizeof(nilP->recDate));
	DmWrite(recordP, (UInt32)&nilP->basalTemp, &item->basalTemp, sizeof(nilP->basalTemp));
	DmWrite(recordP, (UInt32)&nilP->fluidSensation, &item->fluidSensation, sizeof(nilP->fluidSensation));
	DmWrite(recordP, (UInt32)&nilP->fluidColor, &item->fluidColor, sizeof(nilP->fluidColor));
	DmWrite(recordP, (UInt32)&nilP->cervix, &item->cervix, sizeof(nilP->cervix));
	DmWrite(recordP, (UInt32)&nilP->LHTest, &item->LHTest, sizeof(nilP->LHTest));
	DmWrite(recordP, (UInt32)&nilP->sex, &item->sex, sizeof(nilP->sex));
	DmWrite(recordP, (UInt32)&nilP->alcohol, &item->alcohol, sizeof(nilP->alcohol));
	DmWrite(recordP, (UInt32)&nilP->time, &item->time, sizeof(nilP->time));
	DmWrite(recordP, (UInt32)&nilP->other, &item->other, sizeof(nilP->other));	
	
	if (item->note) DmStrCopy(recordP, (UInt32)&nilP->note, item->note);
	else DmWrite(recordP, (UInt32)&nilP->note, &zero, 1);
		
	newIndex = s_currentRecordNumber; //Create a function that returns index
	
	MemPtrUnlock(recordP);
	
	//Insert the record
	err = DmAttachRecord(s_dbNFP, &newIndex, recordH, NULL);  //NULL in final position means will insert and shift down old records
	if (err) MemHandleFree(recordH);
	
	//Apparently I don't need this because I called DmAttachRecord instead of DmNewRecord
	//DmReleaseRecord ( s_dbNFP, newIndex, true);  //true means release dirty
	
	//I may not need these lines anymore...
	// Remember the index of the current record.
	s_recordNum = newIndex;
	
	//return new index on pointer
	*index = newIndex;

	return err;
}

//  FUNCTION: DataSaveOld
//
//  DESCRIPTION: Checks each field in DataEntry and compares it to the day record
//   			information that needs to be re-saved is written to the database.
//				Otherwise it is left alone.  Also marks record as dirty if need be.
//				The dirty feature is really used at this point, but may be important
//				if a conduit is ever written.
//
//  PARAMETERS:	pointer to the index of the record item to re-save
//
//  RETURNS: true if record has been changed, false if no changes to record
Boolean DataSaveOld ( UInt16 *index )
{
	
	NFPDBRecord* nilP=0; 
	NFPDBRecord* item;
	ControlType *ctlP;
	ListType *lstP;
	FieldType *fldP;
	FormType * frmP;
	UInt8 value;
	UInt16 id;
	Boolean wasNoteDirty = false;
	Err err = errNone;
	
	//use wasNoteDirty throughout to check for a dirty record
	//because s_isDirty carries with it information about if the record was
	//previously dirty
	
	frmP = FrmGetActiveForm();
	
	//Process note field
	GetObjectPtr(fldP,DataEntryNoteField);
	
	// Was the note string modified by the user.
	if (FldDirty (fldP))
		{
		// Release any free space in the note field.
		FldCompactText (fldP);
		
		//hide note so I don't have to worry about redraw issues
//		FrmHideObject(frmP,FrmGetObjectIndex(frmP,DataEntryNoteField));

		// Mark the record dirty.
		wasNoteDirty = true;	
		}

	//hide note so I don't have to worry about redraw issues
	//moved this out here because if cursor is selected, but not modified,
	//won't flag as dirty, but will still have redraw problems with "recalculating"
	FrmHideObject(frmP,FrmGetObjectIndex(frmP,DataEntryNoteField));

	// Clear the handle value in the field, otherwise the handle
	// will be freed when the form is disposed of,  this call also unlocks
	// the handle that contains the note string.
	//I had to move this line until after "recalculating" is completed
	//but now I just hide the field above.  This was the only way I could think
	//of dealing with the double delete issue--the record is old, but is deleted.
	FldSetTextHandle (fldP, NULL);


	//process all other fields
	
	item = DataGetData(*index);
	
	//throughout function, switch to record to dirty if anything was modified
	//otherwise leave at whatever DataGetData found
	if(wasNoteDirty)
		s_isDirty = true;
	
	GetObjectPtr(lstP,DataEntryTemperatureList);
	value=LstGetSelection(lstP);
	if(value) //if not blank
		temp_prefs.lastTemp = value; //store last temp for use in auto scroll down
	if(item->basalTemp != value && err == errNone)
	{
		s_isDirty = true;
		wasNoteDirty = true;
		err = DmWrite (item, (UInt32)&nilP->basalTemp, &value, sizeof(nilP->basalTemp));
	}

	GetObjectPtr(lstP,DataEntryFluidSensationList);
	value=LstGetSelection(lstP);
	if(g_prefs.fluidIndicator==1) //Marquette rating, map list to values
	{
		switch(value)
		{
			case 5: //2: not dry
				value=12;
				break;

			case 6: //3: moist
				value=13;
				break;
				
			case 7: //4:
				value=5;  //to match Moist
				break;

			case 8: //5:
				value=15;
				break;
				
			case 9: //6:
				value=16;
				break;

			case 10: //7:
				value=6; //to match Lubricated
				break;

			case 11: //8:
				value=18;
				break;

			default: //list numbers 1-4 match up
				break;
		}

	}
	else // 0 or 2
	{
		switch(value)
		{
				
			case 7: //----
				value=0;  //to match Moist
				break;

			case 8: //Begin PostPardum
				value=50;
				break;
		
			default: //list numbers 0-6 match up
				break;
		}		
	}
	if(item->fluidSensation != value && err == errNone)
	{
		s_isDirty = true;
		wasNoteDirty = true;
		err = DmWrite (item, (UInt32)&nilP->fluidSensation, &value, sizeof(nilP->fluidSensation));	
	}

	GetObjectPtr(lstP,DataEntryFluidColorList);
	value=LstGetSelection(lstP);
	if(item->fluidColor != value && err == errNone)
	{
		s_isDirty = true;
		wasNoteDirty = true;
		err = DmWrite (item, (UInt32)&nilP->fluidColor, &value, sizeof(nilP->fluidColor));
	}

	GetObjectPtr(lstP,DataEntryTimeList);
	value=LstGetSelection(lstP);
	if(item->time != value && err == errNone)
	{
		s_isDirty = true;
		wasNoteDirty = true;
		err = DmWrite (item, (UInt32)&nilP->time, &value, sizeof(nilP->time));
	}

	//Cervix Group 3
	id=FrmGetObjectId (frmP, FrmGetControlGroupSelection(frmP,3));
	value = id-DataEntryCervixNPushButton;
	if(item->cervix != value && err == errNone)
	{
		s_isDirty = true;
		wasNoteDirty = true;
		err = DmWrite (item, (UInt32)&nilP->cervix, &value, sizeof(nilP->cervix));
	}
	
	//LHTest Group 2
	id=FrmGetObjectId (frmP, FrmGetControlGroupSelection(frmP,2));
	value=id-DataEntryLHTestNPushButton;
	if(item->LHTest != value && err == errNone)
	{
		s_isDirty = true;
		wasNoteDirty = true;
		err = DmWrite (item, (UInt32)&nilP->LHTest, &value, sizeof(nilP->LHTest));
	}

	//Sex Group 1
	id=FrmGetObjectId (frmP, FrmGetControlGroupSelection(frmP,1));
	value=id-DataEntrySexNPushButton;
	if(item->sex != value && err == errNone)
	{
		s_isDirty = true;
		wasNoteDirty = true;
		err = DmWrite (item, (UInt32)&nilP->sex, &value, sizeof(nilP->sex));	
	}
	
	GetObjectPtr(ctlP,DataEntryOtherCheckbox);
	value=CtlGetValue(ctlP);
	if(item->other != value && err == errNone)
	{
		s_isDirty = true;
		wasNoteDirty = true;
		err = DmWrite (item, (UInt32)&nilP->other, &value, sizeof(nilP->other));
	}
	
	GetObjectPtr(ctlP,DataEntryAlcoholCheckbox);
	value=CtlGetValue(ctlP);
	if(item->alcohol != value && err == errNone)
	{
		s_isDirty = true;
		wasNoteDirty = true;
		err = DmWrite (item, (UInt32)&nilP->alcohol, &value, sizeof(nilP->alcohol));
	}
	
	DataReleaseData();

	if (err != errNone )
	{
		FrmAlert(OutOfMemoryAlert);
		ForceQuit(); 			
	} 

	return wasNoteDirty;
}

Err DataDelete(UInt16 index)
{
	Err err;
	
	err= DmRemoveRecord(s_dbNFP, index);  //should be changed to DmDeleteRecord so sync works correctly
											//but then I have to rethink the way I use indexes, because
											//I assume no deleted records
	ErrFatalDisplayIf(err,"error deleting record");

	return err;
}


//   DataGetData:
//   Gets a handle to a Data record at index uIndex
//   Parms: uIndex = 0 based index of record in db
//   Returns: NULL or a valid record handle
NFPDBRecord* DataGetData ( UInt16 uIndex )
{
	s_recordNum = 0;
	UInt16 maxRecord;
	UInt16 attr;
	
	if(s_DataIsLocked)
		ErrDisplay("Data Get called when previous record still locked");

	//check to see if the requested index exceeds the maxnumber of current records	
	maxRecord = DataGetCount ();
	if(uIndex + 1 > maxRecord)
	{
		s_pDataStruct=NULL;
	}
	else
	{

	   if ( s_dbNFP )
	   {
		  //first get info about if it is dirty or clean
		  DmRecordInfo(s_dbNFP,uIndex,&attr,NULL,NULL);
		  if(attr == dmRecAttrDirty)
		  	s_isDirty = true;
		  else
		  	s_isDirty = false;

	      // get the record
	      s_recordNum = uIndex;
		  s_hRec = (MemHandle)DmGetRecord ( s_dbNFP, s_recordNum );
	      if ( s_hRec )
	      {
	       // lock it down and get a ptr
	   	   s_pDataStruct = (NFPDBRecord*)MemHandleLock ( s_hRec );
	   	   s_DataIsLocked = true;
	   	   s_isBusy = true;//because used DmGetRecord
	      }
	      else
	      	ErrDisplay("Record could not be opened in DataGetData");   	
	   }
	}

   return s_pDataStruct;
}


NFPDBRecord* DataQueryData ( UInt16 uIndex )
{
	s_recordNum = 0;
	UInt16 maxRecord;
	UInt16 attr;
	
	if(s_DataIsLocked)
		ErrDisplay("Data Query called when previous record still locked");	

	//check to see if the requested index exceeds the maxnumber of current records	
	maxRecord = DataGetCount ();
	if(uIndex + 1 > maxRecord)
	{
		s_pDataStruct=NULL;
	}
	else
	{

	   if ( s_dbNFP )
	   {
		  //first get info about if it is dirty or clean
		  DmRecordInfo(s_dbNFP,uIndex,&attr,NULL,NULL);
		  if(attr == dmRecAttrDirty)
		  	s_isDirty = true;
		  else
		  	s_isDirty = false;

	      // get the record
		  s_recordNum = uIndex;
		  s_hRec = (MemHandle)DmQueryRecord ( s_dbNFP, s_recordNum );
	      if ( s_hRec )
	      {
	         // lock it down and get a ptr
	   	   s_pDataStruct = (NFPDBRecord*)MemHandleLock ( s_hRec );
	   	   s_DataIsLocked = true;
	   	   s_isBusy = false;//because used DmGetRecord
	      }
	      else
	      	ErrDisplay("Record could not be opened in DataQueryData");	      
	   }
	}
   return s_pDataStruct;
}


NFPDBRecord* DataQueryDataNonGlobal ( UInt16 uIndex, DmOpenRef db, Boolean *isDirty )
{
	UInt16 maxRecord;
	NFPDBRecord* pDataStruct;
	MemHandle hRec;
	UInt16 attr;
	
	//check to see if the requested index exceeds the maxnumber of current records	
	maxRecord = DataGetCountNonGlobal ( db );
	if(uIndex + 1 > maxRecord)
	{
		pDataStruct=NULL;
	}
	else
	{

	   if ( db )
	   {
   		  //first get info about if it is dirty or clean
		  DmRecordInfo(db,uIndex,&attr,NULL,NULL);
		  if(attr == dmRecAttrDirty)
		  	*isDirty = true;
		  else
		  	*isDirty = false;
	    
	      // get the record
		  hRec = (MemHandle)DmQueryRecord ( db, uIndex );
	      if ( hRec )
	      {
	         // lock it down and get a ptr
	   	   pDataStruct = (NFPDBRecord*)MemHandleLock ( hRec );
//	   	   s_DataIsLocked = true;
	      }
	      else
		      pDataStruct=NULL;
	   }
	   else
	   {
	   		pDataStruct=NULL;
	   }
	}

   return pDataStruct;
}


MemHandle LockedHandle()
{
	return s_hRec;
}

/*
   DataReleaseData:
   Releases the currently edited fish back to the database
   Parms: none
   Returns: none
*/
void DataReleaseData ( void )
{
	if(s_pDataStruct) //Check to see if NULL
   	{
   		// unlock the record
	    if ( s_hRec )
	    {
		   MemHandleUnlock ( s_hRec );
		   s_DataIsLocked = false;
		}
		
		// Release the record, dirty or not dirty depends on how it was opened.
		if(s_isBusy) //ie if opened with DmGetRecord, then release
			DmReleaseRecord ( s_dbNFP, s_recordNum, s_isDirty);

	   // reset the current record number
	   s_recordNum = 0;
	}

   return;
}

void DataReleaseDataNonGlobal (DmOpenRef db, NFPDBRecord* pDataStruct, UInt16 recordNum, Boolean isDirty)
{
	if(pDataStruct) //Check to see if NULL
   	{	
		//unlock the record
		if( pDataStruct )		
			MemPtrUnlock(pDataStruct);
			
		//Note: since this is always called in response to a Query, don't need DmReleaseRecord
		// Release the record, dirty or not dirty depending on what is passed.
		//DmReleaseRecord ( db, recordNum, isDirty);
	}

   return;
}

//used in find; takes index, closes all forms, re-finds index.
void DataFindClose(UInt16 *recordNum)
{
	UInt32 uniqueID;

	DmRecordInfo (s_dbNFP, *recordNum, NULL, &uniqueID, NULL); 
	FrmCloseAllForms ();
	DmFindRecordByID (s_dbNFP, uniqueID, recordNum);
	
	return;
}

/*
   DataOpen:
   Opens the NFP database, retaining a ref number to the db for further
   operations
   Parms: none
   Returns: none
*/
void DataOpen ( void )
{
	LocalID dbID; 
	UInt16 attributes;
	UInt16 version;
	s_DataIsLocked = false;
	UInt16 index;
	UInt16 noRecords;
	Int8 translateTemp;
	NFPDBRecord* nilP=0; 
	NFPDBRecord* item;

   // open the db
	s_dbNFP = DmOpenDatabaseByTypeCreator ( RECORD_DB_TYPE, appFileCreator, dmModeReadWrite);
	
	if ( 0 == s_dbNFP )
	{
      // not there - create it now
		Err err = DmCreateDatabase ( 0, RECORD_DB_NAME, appFileCreator, RECORD_DB_TYPE, false);
	      // handle unexpected err
		ErrFatalDisplayIf ( err, "Could not create new database.");

		//set info of new database
		attributes = dmHdrAttrBackup;
		version = 2;
		dbID = DmFindDatabase( 0, RECORD_DB_NAME );
		DmSetDatabaseInfo( 0, dbID, NULL, &attributes, &version,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
		
	  // lets try to open it again...
	  	s_dbNFP = DmOpenDatabaseByTypeCreator ( RECORD_DB_TYPE, appFileCreator, dmModeReadWrite);

      // handle unexpected err
		if ( 0 == s_dbNFP )
	    {
		   ErrFatalDisplayIf ( true, "Could not open new database.");
    	}
   }
   else //already exists so check version number
   {
   		dbID = DmFindDatabase( 0, RECORD_DB_NAME );   
   
   		DmDatabaseInfo(0, dbID, NULL, &attributes, &version,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
   
   		if(version == 1) //translate temperatures to new scales
   		{
			//go through all records and translate coverlineTemp
   			noRecords = DataGetCount ();
			translateTemp = 0; //use translateTemp to store new temp
	   		for (index = 0; index < noRecords; index++)
	   		{
	   			//get old temp
	   			item = DataGetData ( index );
	   			translateTemp = item->basalTemp;
	   		
	   			//translate to new temp
	   			if(g_prefs.tempUnits==1) //F
	   			{
		   			if(translateTemp != 0)
		   				translateTemp = translateTemp + 15;
	   			}
	   			else //C
	   			{
					if(translateTemp != 0)
	   					translateTemp = 9 + (2 * translateTemp); 
	   			}
   				//store
				s_isDirty = true;
				DmWrite (item, (UInt32)&nilP->basalTemp, &translateTemp, sizeof(nilP->basalTemp));
					
				DataReleaseData();
   			}
   	
   			//reset attributes
   			attributes = dmHdrAttrBackup;
			version = 2;
			DmSetDatabaseInfo( 0, dbID, NULL, &attributes, &version,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL); 			
   		}
   		
   		if(version > 2)
   			ErrDisplay("Days Database is newer than program version!");
   }

   return;
}

/*
   DataClose:
   Closes the NFP db
   Parms: none
   Returns: none
*/
void DataClose ( void )
{
	if ( s_dbNFP )
	{
   
		if(s_DataIsLocked)
			ErrDisplay("DataClose called when a day record is still locked");
   
   		DmCloseDatabase ( s_dbNFP );
		s_dbNFP = 0;
	}
	return;
}



//*****************************************************************
//*****************************************************************
//Cycle Database Functions
//*****************************************************************

static DmOpenRef		s_dbNFP2;  //need this
static MemHandle   	    s_hRec2;	//need this
static CycleRecord*    	s_pDataStruct2; //was s_pText //need this
static UInt16			s_recordNum2 = 0;  //need this
static Boolean 			s_CycleIsLocked;
static Boolean			s_CycleIsDirty;
static Boolean			s_CycleIsBusy;

//   CycleGetCount:
//   Returns the count of records in the Cycle database
//   Parms: none
//   Returns: count  
UInt16 CycleGetCount ( void )
{
   UInt16 uDataCount = 0;
   if ( s_dbNFP2 )
   {
      uDataCount = DmNumRecordsInCategory ( s_dbNFP2, dmAllCategories );
   }
   return uDataCount;
}


//returns Boolean true if there is a record on that date
// false if there is no record on that date
// also returns on index, the location where record for that
// date should be inserted.
Boolean FindCycleFromDate(DateType desiredDate, UInt16* index, Boolean* dataExists )
{
	Boolean isOldRecord;
	Boolean sameDirection;
	CycleRecord* item;
	Int16 dateDiff;
	Int16 change;
	UInt16 maxRecord;
//	Err err;
//	LocalID dbID;
	
//	dbID = DmFindDatabase(0, RECORD_DB_NAME2);
//	err = DmDatabaseSize(0, dbID, &maxRecord, NULL, NULL);
	maxRecord = CycleGetCount();
	

	if(maxRecord !=0)
	{
		//be sure index is not out of range
		if((*index) + 1 > maxRecord) *index = maxRecord-1;
	
		item = CycleQueryCycle(*index);
		dateDiff=DateTypeCmp(desiredDate,item->cycleStart);
		CycleReleaseCycle();
	
		if(dateDiff>0) change=1;
		else change=-1;
		sameDirection = true;
	
		while(dateDiff!=0 && ((change == 1 && (*index) + 1 < maxRecord ) || (change==-1 && *index > 0)) && sameDirection)
		{
			(*index) += change;
		
			item = CycleQueryCycle(*index);
			dateDiff=DateTypeCmp(desiredDate,item->cycleStart);
			CycleReleaseCycle();
		
			sameDirection = false;
			if(change==1 && dateDiff>0) sameDirection = true;
			if(change==-1 && dateDiff <0) sameDirection = true;
		}
	
		if(dateDiff==0) isOldRecord = true;
		else
		{
		 isOldRecord = true;
		 //if(change==-1 && dateDiff>0) (*index)++;	//if back-up, move one forward for correct insertion
		 if(change==1 && dateDiff<0) (*index)--;	//if went forward and then passed correct cycle, move one back for correct location
		 if(*index==0 && change==-1 && dateDiff<0) isOldRecord = false; //back-up to front of cycle database and wanted to keep going
		}
	}
	
	else
	{
		isOldRecord = false;
		//ensure index is 0
		*index = 0;
	}
	
	if(dataExists)
	{
		if(maxRecord>0)
			*dataExists = true;
		else
			*dataExists = false;
	}
	
		
	return isOldRecord;
}

//returns Boolean true if there is a record on that date
// false if there is no record on that date
// also returns on index, the location where record for that
// date should be inserted.
Boolean FindCycleFromDateNonGlobal(DateType desiredDate, UInt16* index, Boolean* dataExists, DmOpenRef db )
{
	Boolean isOldRecord;
	Boolean sameDirection;
	CycleRecord* item;
	Int16 dateDiff;
	Int16 change;
	UInt16 maxRecord;
	
	Boolean isDirty = false;
//	Err err;
//	LocalID dbID;
	
//	dbID = DmFindDatabase(0, RECORD_DB_NAME2);
//	err = DmDatabaseSize(0, dbID, &maxRecord, NULL, NULL);
	maxRecord = DataGetCountNonGlobal(db); //just reuse DataGetCountNonGlobal
	

	if(maxRecord !=0)
	{
		//be sure index is not out of range
		if((*index) + 1 > maxRecord) *index = maxRecord-1;
	
		item = CycleQueryCycleNonGlobal(*index,db,&isDirty);
		dateDiff=DateTypeCmp(desiredDate,item->cycleStart);
		CycleReleaseCycleNonGlobal(db, item, *index, isDirty);
	
		if(dateDiff>0) change=1;
		else change=-1;
		sameDirection = true;
	
		while(dateDiff!=0 && ((change == 1 && (*index) + 1 < maxRecord ) || (change==-1 && *index > 0)) && sameDirection)
		{
			(*index) += change;
		
			item = CycleQueryCycleNonGlobal(*index,db,&isDirty);
			dateDiff=DateTypeCmp(desiredDate,item->cycleStart);
			CycleReleaseCycleNonGlobal(db, item, *index, isDirty);
		
			sameDirection = false;
			if(change==1 && dateDiff>0) sameDirection = true;
			if(change==-1 && dateDiff <0) sameDirection = true;
		}
	
		if(dateDiff==0) isOldRecord = true;
		else
		{
		 isOldRecord = true;
		 //if(change==-1 && dateDiff>0) (*index)++;	//if back-up, move one forward for correct insertion
		 if(change==1 && dateDiff<0) (*index)--;	//if went forward and then passed correct cycle, move one back for correct location
		 if(*index==0 && change==-1 && dateDiff<0) isOldRecord = false; //back-up to front of cycle database and wanted to keep going
		}
	}
	
	else
	{
		isOldRecord = false;
		//ensure index is 0
		*index = 0;
	}
	
	if(dataExists)
	{
		if(maxRecord>0)
			*dataExists = true;
		else
			*dataExists = false;
	}
	
		
	return isOldRecord;
}


//PARAMETERS:  index points to 
//RETURNS: True if record found, false if not found
//			if true than index of record with given date on index pointer
//			if false than index of before which record should be inserted
//Boolean FindRecordByDate(UInt16 *index, UInt16 searchIndex, Boolean backDirection)
//{

//}

//this new function assumes everything is zero except the startDate
Err CycleNew ( DateType startDate,UInt16 *index )
{
	Err			err;
	CycleRecord* nilP=0;
	CycleRecord newCycle;
	CycleRecord* item;
	MemPtr 		recordP;  	
	MemHandle 	recordH;
	UInt16		size;
//	Char		zero=0;
	
	item = &newCycle;
	
	newCycle.cycleStart = startDate;
	newCycle.billingsPeakDate.day = 1; //the day and month settings are really unneeded dirty records
	newCycle.billingsPeakDate.month = 1;
	newCycle.billingsPeakDate.year = 0;
	newCycle.cervixPeakDate.day = 1;
	newCycle.cervixPeakDate.month = 1;
	newCycle.cervixPeakDate.year = 0;
	newCycle.tempPeakDate.day = 1;
	newCycle.tempPeakDate.month = 1;
	newCycle.tempPeakDate.year = 0;
	newCycle.LHTestPeakDate.day = 1;
	newCycle.LHTestPeakDate.month = 1;
	newCycle.LHTestPeakDate.year = 0;
	newCycle.billingsFertilityDate.day = 1;
	newCycle.billingsFertilityDate.month = 1;
	newCycle.billingsFertilityDate.year =0;
	newCycle.pregnancyAchievedDate.day = 1;
	newCycle.pregnancyAchievedDate.month = 1;
	newCycle.pregnancyAchievedDate.year =0;
	newCycle.coverlineStartDate.year =0;
	newCycle.coverlineLength =0;
	newCycle.coverlineTemp =0;
	
	//  Compute the size of the new to do record
	size = sizeCycleRecord;
	
	//Allocate a chunk in the database for the new record
	recordH = (MemHandle)DmNewHandle(s_dbNFP2,size);
	if (recordH == NULL)
		return dmErrMemError;
		
		
	//Pack the data into the new record
	//Use nilP to determine offset location and size of record
	recordP = MemHandleLock(recordH);
	
	DmWrite(recordP, (UInt32)&nilP->cycleStart, &item->cycleStart, sizeof(nilP->cycleStart));
	DmWrite(recordP, (UInt32)&nilP->billingsPeakDate, &item->billingsPeakDate, sizeof(nilP->billingsPeakDate));
	DmWrite(recordP, (UInt32)&nilP->cervixPeakDate, &item->cervixPeakDate, sizeof(nilP->cervixPeakDate));
	DmWrite(recordP, (UInt32)&nilP->tempPeakDate, &item->tempPeakDate, sizeof(nilP->tempPeakDate));
	DmWrite(recordP, (UInt32)&nilP->LHTestPeakDate, &item->LHTestPeakDate, sizeof(nilP->LHTestPeakDate));
	DmWrite(recordP, (UInt32)&nilP->billingsFertilityDate, &item->billingsFertilityDate, sizeof(nilP->billingsFertilityDate));
	DmWrite(recordP, (UInt32)&nilP->pregnancyAchievedDate, &item->pregnancyAchievedDate, sizeof(nilP->pregnancyAchievedDate));
	DmWrite(recordP, (UInt32)&nilP->coverlineStartDate, &item->coverlineStartDate, sizeof(nilP->coverlineStartDate));
	DmWrite(recordP, (UInt32)&nilP->coverlineLength, &item->coverlineLength, sizeof(nilP->coverlineLength));
	DmWrite(recordP, (UInt32)&nilP->coverlineTemp, &item->coverlineTemp, sizeof(nilP->coverlineTemp));

	
	MemPtrUnlock(recordP);
	
	err = DmAttachRecord(s_dbNFP2, index, recordH, NULL);  //NULL in final position means will insert and shift down old records
	if (err) MemHandleFree(recordH);
	
	//Not needed with DmAttachRecord
	//DmReleaseRecord ( s_dbNFP2, *index, true); //true indicates cycle is dirty

	return err;
}


//value can be NULL if not needed to pass coverlineLength or coverlineTemp
Err CycleSaveOld (UInt16 changeType, DateType changeDate,UInt16 *index,Int8 *value )
{
	Err			err=0;
	CycleRecord* nilP=0;
	CycleRecord* item;
	
	item = CycleGetCycle(*index);
	
	switch(changeType)
	{
	
		case SAVEcycleStart:
			if( DateTypeCmp(item->cycleStart,changeDate) != 0 )
			{
				s_CycleIsDirty = true;
				err=DmWrite (item, (UInt32)&nilP->cycleStart, &changeDate, sizeof(nilP->cycleStart));
			}
			break;
	
		case SAVEbillingsPeakDate:
			if( DateTypeCmp(item->billingsPeakDate,changeDate) != 0 )
			{
				s_CycleIsDirty = true;
				err=DmWrite (item, (UInt32)&nilP->billingsPeakDate, &changeDate, sizeof(nilP->billingsPeakDate));
			}
			break;
			
		case SAVEcervixPeakDate:
			if( DateTypeCmp(item->cervixPeakDate,changeDate) != 0 )
			{
				s_CycleIsDirty = true;
				err=DmWrite (item, (UInt32)&nilP->cervixPeakDate, &changeDate, sizeof(nilP->cervixPeakDate));
			}
			break;
	
		case SAVEtempPeakDate:
			if( DateTypeCmp(item->tempPeakDate,changeDate) != 0 )
			{
				s_CycleIsDirty = true;
				err=DmWrite (item, (UInt32)&nilP->tempPeakDate, &changeDate, sizeof(nilP->tempPeakDate));
			}
			break;

		case SAVELHTestPeakDate:
			if( DateTypeCmp(item->LHTestPeakDate,changeDate) != 0 )
			{
				s_CycleIsDirty = true;
				err=DmWrite (item, (UInt32)&nilP->LHTestPeakDate, &changeDate, sizeof(nilP->LHTestPeakDate));
			}
			break;

		case SAVEbillingsFertilityDate:
			if( DateTypeCmp(item->billingsFertilityDate,changeDate) != 0 )
			{
				s_CycleIsDirty = true;
				err=DmWrite (item, (UInt32)&nilP->billingsFertilityDate, &changeDate, sizeof(nilP->billingsFertilityDate));
			}
			break;

		case SAVEpregnancyAchievedDate:
			if( DateTypeCmp(item->pregnancyAchievedDate,changeDate) != 0 )
			{
				s_CycleIsDirty = true;
				err=DmWrite (item, (UInt32)&nilP->pregnancyAchievedDate, &changeDate, sizeof(nilP->pregnancyAchievedDate));
			}
			break;
			
		case SAVEcoverlineStartDate:
			if( DateTypeCmp(item->coverlineStartDate,changeDate) != 0 )
			{
				s_CycleIsDirty = true;
				err=DmWrite (item, (UInt32)&nilP->coverlineStartDate, &changeDate, sizeof(nilP->coverlineStartDate));
			}
			break;			

		case SAVEcoverlineLength:
			if( item->coverlineLength != *value )
			{
				s_CycleIsDirty = true;
				err=DmWrite (item, (UInt32)&nilP->coverlineLength, value, sizeof(nilP->coverlineLength));
			}
			break;	

		case SAVEcoverlineTemp:
			if( item->coverlineTemp != *value )
			{
				s_CycleIsDirty = true;
				err=DmWrite (item, (UInt32)&nilP->coverlineTemp, value, sizeof(nilP->coverlineTemp));
			}
			break;	
								
		default:
			break;
	}
	
	CycleReleaseCycle();
	
	return err;
}


/*
Err CycleNew ( CycleRecord *item,UInt16 *index )
{
	Err			err;
	CycleRecord* nilP=0; 
	MemPtr 		recordP;  	
	MemHandle 	recordH;
	UInt16		size;
	Char		zero=0;
	
	//  Compute the size of the new to do record
	size = sizeCycleRecord;
	
	//Allocate a chunk in the database for the new record
	recordH = (MemHandle)DmNewHandle(s_dbNFP2,size);
	if (recordH == NULL)
		return dmErrMemError;
		
		
	//Pack the data into the new record
	//Use nilP to determine offset location and size of record
	recordP = MemHandleLock(recordH);
	
	DmWrite(recordP, (UInt32)&nilP->cycleStart, &item->cycleStart, sizeof(nilP->cycleStart));
	DmWrite(recordP, (UInt32)&nilP->billingsPeakDate, &item->billingsPeakDate, sizeof(nilP->billingsPeakDate));
	DmWrite(recordP, (UInt32)&nilP->cervixPeakDate, &item->cervixPeakDate, sizeof(nilP->cervixPeakDate));
	DmWrite(recordP, (UInt32)&nilP->tempPeakDate, &item->tempPeakDate, sizeof(nilP->tempPeakDate));
	DmWrite(recordP, (UInt32)&nilP->LHTestPeakDate, &item->LHTestPeakDate, sizeof(nilP->LHTestPeakDate));
	DmWrite(recordP, (UInt32)&nilP->billingsFertilityDate, &item->billingsFertilityDate, sizeof(nilP->billingsFertilityDate));
	DmWrite(recordP, (UInt32)&nilP->pregnancyAchievedDate, &item->pregnancyAchievedDate, sizeof(nilP->pregnancyAchievedDate));
	
	MemPtrUnlock(recordP);
	
	err = DmAttachRecord(s_dbNFP2, index, recordH, NULL);  //NULL in final position means will insert and shift down old records
	if (err) MemHandleFree(recordH);
	
	DmReleaseRecord ( s_dbNFP2, *index, true); //true indicates cycle is dirty

	return err;
}
*/

Err CycleDeleteCycle(UInt16 index)
{
	Err err;
	
	err= DmRemoveRecord(s_dbNFP2, index); //may to change to DmDeleteRecord, although then must change way I deal with index

	return err;
}


//   CycleGetCycle:
//   Gets a handle to a Cycle record at index uIndex
//   Parms: uIndex = 0 based index of record in db
//   Returns: NULL or a valid record handle
CycleRecord* CycleGetCycle ( UInt16 uIndex )
{
	UInt16 maxRecord;
	UInt16 attr;	
	
	if(s_CycleIsLocked)
		ErrDisplay("Cycle Get called when previous record still locked");


	//check to see if the requested index exceeds the maxnumber of current records	
	maxRecord = CycleGetCount ();
	if(uIndex + 1 > maxRecord)
	{
		s_pDataStruct2=NULL;
	}
	else
	{

	   if ( s_dbNFP2 )
	   {

		  //first get info about if it is dirty or clean
		  DmRecordInfo(s_dbNFP2,uIndex,&attr,NULL,NULL);
		  if(attr == dmRecAttrDirty)
		  	s_CycleIsDirty = true;
		  else
		  	s_CycleIsDirty = false;
		  	
	      // get the record
	      s_recordNum2 = uIndex;
		  s_hRec2 = (MemHandle)DmGetRecord ( s_dbNFP2, s_recordNum2 );
	      if ( s_hRec2 )
	      {
	         // lock it down and get a ptr
	   	   s_pDataStruct2 = (CycleRecord*)MemHandleLock ( s_hRec2 );
	   	   s_CycleIsLocked = true;
	   	   s_CycleIsBusy = true;
	      }
	   }
	}

   return s_pDataStruct2;
}


CycleRecord* CycleQueryCycle ( UInt16 uIndex )
{
	UInt16 maxRecord;
	UInt16 attr;
	
	if(s_CycleIsLocked)
		ErrDisplay("Cycle Query called when previous record still locked");


	//check to see if the requested index exceeds the maxnumber of current records	
	maxRecord = CycleGetCount ();
	if(uIndex + 1 > maxRecord)
	{
		s_pDataStruct2=NULL;
	}
	else
	{

	   if ( s_dbNFP2 )
	   {
		  //first get info about if it is dirty or clean
		  DmRecordInfo(s_dbNFP2,uIndex,&attr,NULL,NULL);
		  if(attr == dmRecAttrDirty)
		  	s_CycleIsDirty = true;
		  else
		  	s_CycleIsDirty = false;

		  s_recordNum2 = uIndex;
	      // get the record
		  s_hRec2 = (MemHandle)DmQueryRecord ( s_dbNFP2, s_recordNum2 );
	      if ( s_hRec2 )
	      {
	         // lock it down and get a ptr
	   	   s_pDataStruct2 = (CycleRecord*)MemHandleLock ( s_hRec2 );
	   	   s_CycleIsLocked = true;
	   	   s_CycleIsBusy = false;
	      }
	   }
	}

   return s_pDataStruct2;
}


CycleRecord* CycleQueryCycleNonGlobal ( UInt16 uIndex, DmOpenRef db, Boolean *isDirty )
{
	UInt16 maxRecord;
	CycleRecord* pDataStruct;
	MemHandle hRec;
	UInt16 attr;
	
	//check to see if the requested index exceeds the maxnumber of current records	
	maxRecord = DataGetCountNonGlobal ( db );
	if(uIndex + 1 > maxRecord)
	{
		pDataStruct=NULL;
	}
	else
	{

	   if ( db )
	   {
   		  //first get info about if it is dirty or clean
		  DmRecordInfo(db,uIndex,&attr,NULL,NULL);
		  if(attr == dmRecAttrDirty)
		  	*isDirty = true;
		  else
		  	*isDirty = false;
	    
	      // get the record
		  hRec = (MemHandle)DmQueryRecord ( db, uIndex );
	      if ( hRec )
	      {
	         // lock it down and get a ptr
	   	   pDataStruct = (CycleRecord*)MemHandleLock ( hRec );
//	   	   s_DataIsLocked = true;
	      }
	      else
		      pDataStruct=NULL;
	   }
	   else
	   {
	   		pDataStruct=NULL;
	   }
	}

   return pDataStruct;
}



//   CycleReleaseCycle:
//   Releases the currently edited cycle back to the database
//   Parms: none
//   Returns: none
void CycleReleaseCycle ( void )
{
	if(s_pDataStruct2) //Check to see if NULL
   	{
   		// unlock the record
	    if ( s_hRec2 )
	    {
		   MemHandleUnlock ( s_hRec2 );
		   s_CycleIsLocked = false;
		}

		// Release the record, dirty or not dirty depending on how opened.
		if(s_CycleIsBusy)
			DmReleaseRecord ( s_dbNFP2, s_recordNum2, s_CycleIsDirty);

	   // reset the current record number
	   s_recordNum2 = 0;
	}

   return;
}


void CycleReleaseCycleNonGlobal (DmOpenRef db, CycleRecord* pDataStruct, UInt16 recordNum, Boolean isDirty)
{
	if(pDataStruct) //Check to see if NULL
   	{	
		//unlock the record
		if( pDataStruct )		
			MemPtrUnlock(pDataStruct);
			
		//Note: since this is always called in response to a Query, don't need DmReleaseRecord
		// Release the record, dirty or not dirty depending on what is passed.
		//DmReleaseRecord ( db, recordNum, isDirty);
	}

   return;
}

//  FUNCTION: CycleOpen
//
//  DESCRIPTION: Opens the NFPCycles database, retaining a ref number to the db for further
//   			operations
//
//  PARAMETERS:
//
//  RETURNS: 
void CycleOpen ( void )
{
	LocalID dbID; 
	UInt16 attributes;
	UInt16 version;
	UInt16 index;
	UInt16 noRecords;
	DateType tempDate;
	Int8 cLength;
	Err err;
	CycleRecord* cycle;

	s_CycleIsLocked = false;   

   // open the db
	s_dbNFP2 = DmOpenDatabaseByTypeCreator ( RECORD_DB_TYPE2, appFileCreator, dmModeReadWrite);

	if ( 0 == s_dbNFP2 )
	{
      // not there - create it now
		err = DmCreateDatabase ( 0, RECORD_DB_NAME2, appFileCreator, RECORD_DB_TYPE2, false);

      // handle unexpected err
		ErrFatalDisplayIf ( err, "Could not create new cycle database.");
		
	  //set info of new database
		attributes = dmHdrAttrBackup;
		version = 3;
		dbID = DmFindDatabase( 0, RECORD_DB_NAME2 );
		DmSetDatabaseInfo( 0, dbID, NULL, &attributes, &version,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);

	  // lets try to open it again...
	  	s_dbNFP2 = DmOpenDatabaseByTypeCreator ( RECORD_DB_TYPE2, appFileCreator, dmModeReadWrite);
	  	
	  	err = DmGetLastErr();

      // handle unexpected err
		if ( 0 == s_dbNFP2 )
	    {
		   ErrFatalDisplayIf ( true, "Could not open new cycle database.");
    	}
   }

   else //already exists, so check the version number
   {
		dbID = DmFindDatabase( 0, RECORD_DB_NAME2 );   
   
   		DmDatabaseInfo(0, dbID, NULL, &attributes, &version,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);

   		if(version == 1) //original release version, doesn't contain coverline information
   		{
			//go through all records and expand them to new size
			//inserting zero into coverlineLength
   			noRecords = CycleGetCount ();
			cLength = 0;
	   		for (index = 0; index < noRecords; index++)
	   		{
   				DmResizeRecord(s_dbNFP2,index,sizeCycleRecord);
   				//store zero length coverlines in old cycles
				err = CycleSaveOld ( SAVEcoverlineLength, tempDate, &index, &cLength );
   			}
   	
   			//reset attributes
   			attributes = dmHdrAttrBackup;
			version = 2;
			DmSetDatabaseInfo( 0, dbID, NULL, &attributes, &version,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);

   			
   		}
   		
   		if(version <= 2) //translate temperatures to new scales
   		{
			//go through all records and translate coverlineTemp
   			noRecords = CycleGetCount ();
			cLength = 0; //use cLength to store new temp
	   		for (index = 0; index < noRecords; index++)
	   		{
	   			//get old temp
	   			cycle = CycleQueryCycle ( index );
	   			cLength = cycle->coverlineTemp;
	   			CycleReleaseCycle();
	   		
	   			//translate to new temp
	   			if(g_prefs.tempUnits==1) //F
		   			cLength = cLength + 15;
	   			else //C
	   				cLength = - 9 + (2 * cLength);  //temp is stored as negative, so flip operation to account for negative
   				//store
				err = CycleSaveOld ( SAVEcoverlineTemp, tempDate, &index, &cLength );
   			}
   	
   			//reset attributes
   			attributes = dmHdrAttrBackup;
			version = 3;
			DmSetDatabaseInfo( 0, dbID, NULL, &attributes, &version,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
   		}
   		
   		if(version > 3)
   			ErrDisplay("Cycle Database is newer than program version!");
   		
   
   }

   return;
}


//  FUNCTION: CycleClose
//
//  DESCRIPTION: Closes the NFPCycles database
//
//  PARAMETERS:
//
//  RETURNS: 
void CycleClose ( void )
{
	if ( s_dbNFP2 )
	{
		if(s_CycleIsLocked)
			ErrDisplay("Cycle Close called when a cycle record is still locked");

   		DmCloseDatabase ( s_dbNFP2 );
		s_dbNFP2 = 0;
	}
	return;
}