//    Copyright (C) 2003 Chad Brassil
//    This file is part of NFP.
//
//    NFP is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    NFP is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with NFP; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

// NFPMain.c
//
// main file for NFP

#include <PalmOS.h>
#include <SonyTypes.h>
#include <SonySystemPublic.h>
#include <SonyHRLib.h>

#include "NFP.h"
#include "NFPRsc.h"
#include "DateMonth.h"

// ********************************************************************
// Entry Points
// ********************************************************************

// ********************************************************************
// Global variables
// ********************************************************************

// g_prefs
// cache for application preferences during program execution
NFPPreferenceType g_prefs;
NFPTempPreferenceType temp_prefs;
Boolean s_displayedPregnancyMessage;
Boolean s_displayPregnancyMessage;
Boolean	colorEnabled;
Boolean hrEnabled;

// ********************************************************************
// Internal Constants
// ********************************************************************

// Define the minimum OS version we support
#define ourMinVersion    sysMakeROMVersion(3,1,0,sysROMStageDevelopment,0)
#define kPalmOS10Version sysMakeROMVersion(1,0,0,sysROMStageRelease,0)


// ********************************************************************
// Internal Functions
// ********************************************************************

void ForceQuit(void)
{
	EventType newEvent;
	
	newEvent.eType = keyDownEvent;
	newEvent.data.keyDown.chr = launchChr;
	newEvent.data.keyDown.modifiers = commandKeyMask;
	EvtAddEventToQueue (&newEvent);
	
	return;
}

//return false if set password and confirm password do not match
static Boolean SECT1 GetEnteredPassword(FormType * frmP)
{
	FieldType *fldP;
	Char *pString;
	Char *pSecondString;
	Boolean match;
	Int16 compare;

	fldP = (FieldType*) FrmGetObjectPtr(frmP,FrmGetObjectIndex(frmP,EnterPasswordEnterPasswordField));
	pString = FldGetTextPtr(fldP);
	pSecondString = &g_prefs.password[0];
		
	if(pString!=NULL && pSecondString!=NULL)
	{
		compare = StrCompare(pString,pSecondString);
		if(compare==0)
			match = true;
		else
			match = false;
	}
	else if((pString==NULL || *pString==NULL) && (pSecondString==NULL || *pSecondString==NULL))
		match = true;
	else
		match = false;
		
	return match;
}

//returns true if password was good
static Boolean SECT1 PasswordCheck(void)
{
	FormType * frmP;
	FormType * frmPII;
	UInt16 button;
	Boolean goodPassword;

	//I don't know if I still need all this BlankForm stuff?
	//actually I think I do inorder to clear the background when WakeUp is called
	frmPII = FrmInitForm(BlankForm);
	FrmDrawForm(frmPII);


    frmP = FrmInitForm (EnterPasswordForm);
    FrmSetFocus (frmP, FrmGetObjectIndex(frmP,EnterPasswordEnterPasswordField));
    button = FrmDoDialog (frmP);
	if(button == EnterPasswordOKButton)
	{
		goodPassword = GetEnteredPassword(frmP);
		FrmDeleteForm (frmP); // this needs to be deleted before alert otherwise I get something hanging around in the Draw stack
		if(goodPassword == false)
			FrmAlert(PasswordMatchAlert);	
	}
	else
	{
		FrmDeleteForm (frmP); 
		goodPassword = false;
	}

	//I don't know if I still need all this BlankForm stuff?	
    frmP = FrmGetActiveForm();
    FrmSetActiveForm (frmPII);
    FrmEraseForm (frmPII);
    FrmDeleteForm (frmPII);
    FrmSetActiveForm(frmP);
    
    return goodPassword;
};

//used when WakeUp from sleep
static void SECT1 WakeUpCheck(SysNotifyParamType* notif)
{
	
   	if(notif->notifyType == sysNotifyLateWakeupEvent)
	{
	 	//don't bother with password if device is locked since security already exists
	    if(g_prefs.enablePassword && !(Boolean) PrefGetPreference(prefDeviceLocked))
		{
			//first be sure password check is not already displayed
		    if( EnterPasswordForm != FrmGetActiveFormID() )
			{
				if(PasswordCheck()==false)
					ForceQuit();
			}
			//otherwise just go about regular business
		}
	}
		
	return;
}

// FUNCTION: AppHandleEvent
//
// DESCRIPTION: 
//
// This routine loads form resources and set the event handler for
// the form loaded.
//
// PARAMETERS:
//
// event
//     a pointer to an EventType structure
//
// RETURNED:
//     true if the event was handled and should not be passed
//     to a higher level handler.

static Boolean SECT1 AppHandleEvent(EventType * eventP)
{
    UInt16 formId;
    FormType * frmP;

    if (eventP->eType == frmLoadEvent)
    {
        // Load the form resource.
        formId = eventP->data.frmLoad.formID;
        frmP = FrmInitForm(formId);
        FrmSetActiveForm(frmP);

        // Set the event handler for the form.  The handler of the 
        // currently active form is called by FrmHandleEvent each 
        // time is receives an event.
        switch (formId)
        {
//        case MainForm:
//            FrmSetEventHandler(frmP, MainFormHandleEvent);
//            break;
            
        case DataEntryForm:
            FrmSetEventHandler(frmP, DataEntryFormHandleEvent);
            break;
            
        case MonthViewForm:
        	FrmSetEventHandler(frmP, MonthViewHandleEvent);
        	break;
        	
        case PreferencesForm:
        	FrmSetEventHandler(frmP, PreferencesFormHandleEvent);
        	break;
        	
        case SetAlarmsForm:
        	FrmSetEventHandler(frmP, SetAlarmsFormHandleEvent);
        	break;
        	
        case TempChartForm:
        	FrmSetEventHandler(frmP, TempChartFormHandleEvent);
        	break;
        	
        case SetPasswordForm:
        	FrmSetEventHandler(frmP, SetPasswordFormHandleEvent);
        	break;	 			        	
        	
        default:
            break;

        }
        return true;
    }
  
  	//I added this, not orthodox position, but avoids flicker  
    if (eventP->eType == frmGotoEvent)
    {
    	return GotoEventHandler(eventP);
    }
    
    //added this to deal with WakeUp events when application is running
    if (eventP->eType == frmLoadEvent)
    {
    
    }

    return false;
}

// FUNCTION: AppEventLoop
//
// DESCRIPTION: This routine is the event loop for the application.

static void SECT1 AppEventLoop(void)
{
    UInt16 error;
    EventType event;

    do {
        // change timeout if you need periodic nilEvents
        EvtGetEvent(&event, evtWaitForever);

        if (! SysHandleEvent(&event))
        {
            if (! MenuHandleEvent(0, &event, &error))
            {
                if (! AppHandleEvent(&event))
                {
                    FrmDispatchEvent(&event);
                }
            }
        }
    } while (event.eType != appStopEvent);
}

//checks for and starts Sony High Resolution Mode
static Boolean SECT1 StartHRMode()
{
	Err error = 0;
	Err errorOpen;
	UInt16 refNum/*, version*/;
	UInt32 width, height; //, depth;
	Boolean started = false;
	
	if ((error = SysLibFind(sonySysLibNameHR, &refNum)))
	{
		if (error == sysErrLibNotFound)
		{
			//couldn't find the lib, so try to load it
			error = SysLibLoad( 'libr', sonySysFileCHRLib, &refNum );
		}
	}
	
	if (!error)
	{
		errorOpen = HROpen(refNum);
	//	HRGetAPIVersion(refNum,&version);
		
	//	if( version < HR_VERSION_SUPPORT_FNTSIZE)
		if(!errorOpen)
		{
			width = hrWidth;
			height = hrHeight;
			//depth = 4; //try setting this to NULL so that it just accepts default.
						//  This may mean it doesn't utilize full bits in black and white modes, but otherwise
						// I have to deal with enabling color mode
			error = HRWinScreenMode ( refNum, winScreenModeSet, &width, &height,NULL/*&depth*/, NULL );
		
		}
	
	}
	
	if (!error)
		started = true;
	else
		started = false;

	return started;
}

// FUNCTION: AppStart
//
// DESCRIPTION:  Get the current application's preferences.
//
// RETURNED:
//     errNone - if nothing went wrong

static Err SECT1 AppStart(void)
{
    Err error;
    UInt16 prefsSize;
    Int16 prevPrefVersion = noPreferenceFound;
   	UInt32			romVersion;
	UInt32			minROM;
	UInt32 value;
	UInt16 cardNo;
	LocalID dbID;
	DmSearchStateType searchInfo;
    
    // Read the saved preferences / saved-state information.
    prefsSize = sizeof(NFPPreferenceType);
    prevPrefVersion = PrefGetAppPreferences(appFileCreator, appPrefID, &g_prefs, &prefsSize, true);
   	if ( prevPrefVersion > appPrefVersionNum )//if from a future release, user has synced to an older version, start fresh 
   		prevPrefVersion = noPreferenceFound;
    //check id preference file did not exist or of previous version, so put defaults into g_prefs
    if ( prevPrefVersion == noPreferenceFound )
    { 
		//version 1 and 2
    	g_prefs.tempMethod = 1;
		g_prefs.postPeakDays = 4;
		g_prefs.factorsForPostPeak = 2;
		g_prefs.roetzerRule = 1;
		g_prefs.dryDaysRule = 1;
		g_prefs.morningAlarmSound = 0;
		g_prefs.morningAlarmTime.hour = 8;
		g_prefs.morningAlarmTime.minute = 00;	
		g_prefs.enableMorningAlarm = 0;
		g_prefs.morningAlarmRepeat = 1;
		g_prefs.eveningAlarmSound = 0;
		g_prefs.eveningAlarmTime.hour = 22;
		g_prefs.eveningAlarmTime.minute = 00;
		g_prefs.enableEveningAlarm = 0;
		g_prefs.eveningAlarmRepeat = 1;
		g_prefs.tempUnits = 1; //F = 1, C = 2
		g_prefs.aveCycleLength = defaultCycleLength;
		g_prefs.shortestCycleLength = defaultShortestLength;
		g_prefs.aveLutealLength = defaultLutealLength;
    }
    
    if ( (prevPrefVersion == noPreferenceFound) || (prevPrefVersion < 3) )
    {
    	//version 3
    	g_prefs.enablePassword = 0;
    	g_prefs.password[0] = NULL;
    	g_prefs.disableAlarms = 0;
    }
    
    if ( (prevPrefVersion == noPreferenceFound) || (prevPrefVersion < 4) )
    {
		//version 4
		g_prefs.exportSeparator = 0;
		
		g_prefs.fluidIndicator = 0;
		g_prefs.hormoneTest = 0;
		g_prefs.chartDisplay = 0;
		g_prefs.presets = 0;
		g_prefs.timeAdjustMode = 0;
    }
    
    // Read the non-saved preferences / saved-state information.
    prefsSize = sizeof(NFPTempPreferenceType);
    prevPrefVersion = PrefGetAppPreferences(appFileCreator, appPrefID, &temp_prefs, &prefsSize, false);
   	if ( prevPrefVersion > appPrefVersionNum )//if from a future release, user has synced to an older version, start fresh 
   		prevPrefVersion = noPreferenceFound;  
    //check id preference file did not exist or of previous version, so put defaults into g_prefs
    if ( prevPrefVersion == noPreferenceFound )
    { 
    	//version 1
    	temp_prefs.view = 1; //month view
    }
    
    if ( (prevPrefVersion == noPreferenceFound) || (prevPrefVersion < 2) )
    {
    	//version 2
    	temp_prefs.lastTemp = 1; //lowest temp
    }
    
    if ( (prevPrefVersion == noPreferenceFound) || (prevPrefVersion < 3) )
    {
    	//version 3
    	temp_prefs.lastDisplayedAlarm = 0;
    }
    
   // init the database
   DataOpen ();
   CycleOpen ();
   
   //set current date
   DateSecondsToDate (TimGetSeconds (), &s_currentDate);
   
   //initialization needed for borrowed calendar
   // Get the starting day of the week from the system preferences.
   StartDayOfWeek = PrefGetPreference(prefWeekStartDay);
   
   //could put these into the unsaved Preferences database
   s_cycleStart.year = 0;
   s_lastCycleCheckDate.year = 0;
   s_lastCycleIndex = 0;
   s_currentRecordNumber = 0;
   
   s_displayedPregnancyMessage = false;
   s_displayPregnancyMessage = false;
   
   //determined if colorEnabled
	colorEnabled = false;
	FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
	minROM = sysMakeROMVersion(3,5,0,sysROMStageRelease,0);
	if(romVersion >= minROM)
		WinScreenMode(winScreenModeGetSupportsColor,NULL,NULL,NULL,&colorEnabled);
	
	//determine if Sony High Res Enabled
	minROM = sysMakeROMVersion(5,0,0,sysROMStageRelease,0);
	//Only check for High Res support if less than Palm 5.
	//If Palm 5, just use bitmap families
	if(romVersion < minROM)
		hrEnabled = StartHRMode();
	else
		hrEnabled = false;
		
	//Register for wake-up notification in order to manage password
	error = DmGetNextDatabaseByTypeCreator (true, &searchInfo, sysFileTApplication, appFileCreator, true, &cardNo, &dbID);
	FtrGet(sysFtrCreator, sysFtrNumNotifyMgrVersion, &value);
	if(value)
		SysNotifyRegister(cardNo, dbID, sysNotifyLateWakeupEvent, NULL, sysNotifyNormalPriority, NULL);
    
   return errNone;
}

// FUNCTION: AppStop
//
// DESCRIPTION: Save the current state of the application.

static void SECT1 AppStop(void)
{
	Err error;
	UInt16 refNum;
	UInt32 value;
	UInt16 cardNo;
	LocalID dbID;
	DmSearchStateType searchInfo;

    // Write the saved preferences / saved-state information.  This 
    // data will be saved during a HotSync backup.
    PrefSetAppPreferences(
        appFileCreator, appPrefID, appPrefVersionNum, 
        &g_prefs, sizeof(NFPPreferenceType), true);
        
    PrefSetAppPreferences(
        appFileCreator, appPrefID, appPrefVersionNum, 
        &temp_prefs, sizeof(NFPTempPreferenceType), false);
        
    // Close all the open forms.
    FrmCloseAllForms();
    
 	// Close the application's database.
	DataClose ();
	CycleClose();
	
	if(hrEnabled)
	{
		SysLibFind(sonySysLibNameHR, &refNum);
		error = HRWinScreenMode(refNum, winScreenModeSetToDefaults, NULL, NULL, NULL, NULL);
		error = HRClose(refNum);
	}
	
	//Unregister for wake-up notification
	error = DmGetNextDatabaseByTypeCreator (true, &searchInfo, sysFileTApplication, appFileCreator, true, &cardNo, &dbID);
	FtrGet(sysFtrCreator, sysFtrNumNotifyMgrVersion, &value);
	if(value)
		SysNotifyUnregister(cardNo, dbID, sysNotifyLateWakeupEvent, sysNotifyNormalPriority);

	return;
}

// all code from here to end of file should use no global variables
//CODEWARRIOR SPECIFIC:  next line commented out for prc-tools
//#pragma warn_a5_access on

// FUNCTION: RomVersionCompatible
//
// DESCRIPTION: 
//
// This routine checks that a ROM version is meet your minimum 
// requirement.
//
// PARAMETERS:
//
// requiredVersion
//     minimum rom version required
//     (see sysFtrNumROMVersion in SystemMgr.h for format)
//
// launchFlags
//     flags that indicate if the application UI is initialized
//     These flags are one of the parameters to your app's PilotMain
//
// RETURNED:
//     error code or zero if ROM version is compatible

static Err SECT1 RomVersionCompatible(UInt32 requiredVersion, UInt16 launchFlags)
{
    UInt32 romVersion;

    // See if we're on in minimum required version of the ROM or later.
    FtrGet(sysFtrCreator, sysFtrNumROMVersion, &romVersion);
    if (romVersion < requiredVersion)
    {
        if ((launchFlags & 
            (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp)) ==
            (sysAppLaunchFlagNewGlobals | sysAppLaunchFlagUIApp))
        {
            FrmAlert (RomIncompatibleAlert);

            // Palm OS 1.0 will continuously relaunch this app unless 
            // we switch to another safe one.
            if (romVersion <= kPalmOS10Version)
            {
                AppLaunchWithCommand(
                    sysFileCDefaultApp, 
                    sysAppLaunchCmdNormalLaunch, NULL);
            }
        }

        return sysErrRomIncompatible;
    }

    return errNone;
}

// FUNCTION: NFPPalmMain
//
// DESCRIPTION: This is the main entry point for the application.
//
// PARAMETERS:
//
// cmd
//     word value specifying the launch code. 
//
// cmdPB
//     pointer to a structure that is associated with the launch code
//
// launchFlags
//     word value providing extra information about the launch
//
// RETURNED:
//     Result of launch, errNone if all went OK

static UInt32 SECT1 NFPPalmMain(
    UInt16 cmd, 
    MemPtr cmdPBP, 
    UInt16 launchFlags)
{
    Err error;
    
   	LocalID dbID;
	DmSearchStateType searchInfo;
	UInt16 cardNo;

    error = RomVersionCompatible (ourMinVersion, launchFlags);
    if (error) return (error);
    
//    if(launchFlags == sysAppLaunchFlagSubCall)
//    {
//    	2+3;
//    }

    switch (cmd)
    {
    case sysAppLaunchCmdNormalLaunch:
        error = AppStart();
        if (error) 
            return error;

        // start application by opening the main form
        // and then entering the main event loop
		// check password
		if(g_prefs.enablePassword)
		{
			if(PasswordCheck())
				GoToView();
			else
				ForceQuit();
	    }
		else
			GoToView();
        AppEventLoop();
        
        AppStop();
        break;
        
     //defined this myself to mean a launch from Alarm Notification 
    case sysAppLaunchFromAlarm:
        error = AppStart();
        if (error) 
            return error;

        FrmGotoForm(DataEntryForm);
        AppEventLoop();
        
        AppStop();
        break;
             
    case sysAppLaunchCmdAlarmTriggered:
	    HandleAlarmTrigger((SysAlarmTriggeredParamType *)cmdPBP);
       	break;
    	
    case sysAppLaunchCmdDisplayAlarm:
    	if( DisplayAlarm((SysAlarmTriggeredParamType *)cmdPBP) )
    	{  	
   			DmGetNextDatabaseByTypeCreator (true, &searchInfo, sysFileTApplication, appFileCreator, true, &cardNo, &dbID);
   			
			SysUIAppSwitch(cardNo,dbID,sysAppLaunchFromAlarm,NULL);
    	}
    	break;
    	
	//this is sent when a HotSync has been completed,
	//could use this to modify dirty/clean if that actually matters to what gets synced
	//but right now both of these are just used to reset alarm times
    case sysAppLaunchCmdTimeChange:
    case sysAppLaunchCmdSyncNotify:
    	SetNFPAlarms();
    	break;
    	
    //find a text string
    case sysAppLaunchCmdFind:
    	Search ((FindParamsPtr)cmdPBP);
    	break;
    	
	//go to a record and display it
    case sysAppLaunchCmdGoTo:
    	Boolean	launched;
		launched = launchFlags & sysAppLaunchFlagNewGlobals;	
	
		if (launched) 
		{
			error = AppStart();
	        if (error) 
	            return error;

	        // start application by opening the requested data entry form
	        // and then entering the main event loop
	        
	        if(g_prefs.enablePassword)
			{
				if(PasswordCheck())
					GoToItem ((GoToParamsPtr) cmdPBP, launched);
				else
					ForceQuit();
		    }
			else
				GoToItem ((GoToParamsPtr) cmdPBP, launched);

	        AppEventLoop();
	        
	        AppStop();
		}
		else //don't need a password here because program is already started
			GoToItem ((GoToParamsPtr) cmdPBP, launched);

    	break;
    	
    case sysAppLaunchCmdNotify:
		WakeUpCheck ((SysNotifyParamType*) cmdPBP);
		break;

    default:
        break;
    }

    return errNone;
}

// FUNCTION: PilotMain
//
// DESCRIPTION: This is the main entry point for the application.
// 
// PARAMETERS:
//
// cmd
//     word value specifying the launch code. 
//
// cmdPB
//     pointer to a structure that is associated with the launch code
//
// launchFlags
//     word value providing extra information about the launch.
//
// RETURNED:
//     Result of launch, errNone if all went OK

UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
    return NFPPalmMain(cmd, cmdPBP, launchFlags);
}

// turn a5 warning off to prevent it being set off by C++
// static initializer code generation
// I now comment this out because I'm back to using gcc in the Palm OS Developer Suite
//#pragma warn_a5_access reset
