//    Copyright (C) 2003 Chad Brassil
//    This file is part of NFP.
//
//    NFP is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    NFP is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with NFP; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <PalmOS.h>

// Handspring SDK headers
//#include <HSExt.h>
#include <SysEvtMgr.h>
#include "NFP.h"
#include "NFPRsc.h"

#define MAXMENSES	10 //this used to be 7, which I think generated some errors

void CalcCervix( void ) SECT4;
void CalcLuteal( void ) SECT4;
UInt16 s_lastCycleIndex;


//* DeterminePhase
//*  Output is phase:
//*  0 is undetermined
//*	 1 is menses
//*	 2 is phase 1
//*	 3 is fertile
//*	 4 is peak
//*  5 is phase 2
//*  6 is baby
//note: from what I can tell I don't need to include beyondMaxIndex here.

RGBColorType PhaseColor (UInt8 phase)
{
	RGBColorType color;

	switch(phase)
	{
		case 2: //infertile
		case 5: //infertile
		case 0: //undetermined
			UIColorGetTableEntryRGB(UIObjectFill,&color);
			break;

		case 1: //m
			color.r = 255;
			color.g = 147;
			color.b = 147;
			break;

		case 3: //fertile
			color.r = 5;
			color.g = 245;
			color.b = 245;
			break;
		
		case 4: //peak
			color.r = 253; //244
			color.g = 183; //183
			color.b = 87; //168
			break;	
			
		case 6: //baby
			color.r = 250;
			color.g = 250;
			color.b = 53;
			break;	

		default:
			break;
	}
		
	return color;

}

//what is index used for. I think it just points to focalDate or ?most previous date?
//final pointer can be null if allowGlobal is true, otherwise pass a pointer to local prefs
UInt8 DeterminePhase( DateType focalDate, UInt16 index, Boolean isOldRecord, Boolean dataExists, Boolean beyondMaxIndex, Int16 *cycleDay, Boolean allowGlobal, NFPPreferenceType* l_prefs )
{
	Boolean isOldCycle;
	UInt16 cycleIndex;
	Boolean cycleDataExists;
	DateType today;
	UInt8 phase;
	Int32 dayDiff;
	Int32 tempDayDiff;
	Int32 billingsDayDiff;
	Int32 cervixDayDiff;
	Int32 LHTestDayDiff;
	Int32 pregnancyAchievedDayDiff;
	Int32 lutealDiff;
	CycleRecord* cycle;
	NFPDBRecord* item;
	UInt16 itemIndex;
	float cycleRatio;
	UInt8 numberFact;
	FormType* frmP;
	Char* pString;
	DateType dueDate;
	UInt16 objectIndex;
	Err err;
	Int16 numberCalc;
	UInt8 effectivePostPeakBillings;
	Boolean inPostPartumCycle = false;
	DmOpenRef dbNFP;
	Boolean isDirtyDays = false;
	
	DmOpenRef dbDays;
	DmOpenRef dbCycles;
	Boolean isDirty = false;
	Boolean isDirtyII = false;
	NFPPreferenceType* p_prefs;
	
	phase = 0; //default

	//find relevant cycle
	if (allowGlobal)
	{
		p_prefs = &g_prefs;
		cycleIndex = s_lastCycleIndex;  //create static variable to use previous index
		isOldCycle = FindCycleFromDate(focalDate, &cycleIndex, &cycleDataExists);
		//to keep compiler happy, initialize these with NULL
		dbCycles = NULL;
		dbDays = NULL;
	}
	else
	{
		p_prefs = l_prefs;
		cycleIndex = 0;
		//Open Database
		dbCycles = DmOpenDatabaseByTypeCreator ( RECORD_DB_TYPE2, appFileCreator, dmModeReadOnly );
		dbDays = DmOpenDatabaseByTypeCreator ( RECORD_DB_TYPE, appFileCreator, dmModeReadOnly );
		isOldCycle = FindCycleFromDateNonGlobal(focalDate, &cycleIndex, &cycleDataExists, dbCycles);
		if(dbCycles == 0 || dbDays == 0) //if one didn't open, close what I can and return default phase
		{
			if(dbCycles) DmCloseDatabase ( dbCycles );
			if(dbDays) DmCloseDatabase ( dbDays );
			return phase;
		}
	}

	if( p_prefs->presets == 7 ) //CCL Rule R
		effectivePostPeakBillings = 3;
	else if ( p_prefs->presets == 8 ) //CCL Rule A
		effectivePostPeakBillings = 2; 
	else //For CCL Rules C and B, postPeakDays should already be 4
		effectivePostPeakBillings = p_prefs->postPeakDays; //store this in a new variable up front so it can be adjusted for CCL presets



	if(isOldCycle) //confirm we have a cycleStart date
	{
		//calculate current cycle day
		if (allowGlobal)
			cycle = CycleGetCycle(cycleIndex);
		else
			cycle = CycleQueryCycleNonGlobal(cycleIndex, dbCycles, &isDirty);
		
		dayDiff = DateToDays(focalDate) - DateToDays(cycle->cycleStart) + 1;
		
		//Temperature method
		tempDayDiff = -1000; //code meaning not calculated
		if(cycle->tempPeakDate.year != 0)
			tempDayDiff = DateToDays(focalDate) - DateToDays(cycle->tempPeakDate);
		
		//Billings method
		billingsDayDiff = -1000; //code meaning not calculated
		if(cycle->billingsPeakDate.year != 0)
			billingsDayDiff = DateToDays(focalDate) - DateToDays(cycle->billingsPeakDate);
	
		//Cervix method
		cervixDayDiff = -1000; //code meaning not calculated
		if(cycle->cervixPeakDate.year != 0)
			cervixDayDiff = DateToDays(focalDate) - DateToDays(cycle->cervixPeakDate);
				
		//LHTest method
		LHTestDayDiff = -1000; //code meaning not calculated
		if(cycle->LHTestPeakDate.year != 0)
			LHTestDayDiff = DateToDays(focalDate) - DateToDays(cycle->LHTestPeakDate);
		
		//pregnancyAchieved
		pregnancyAchievedDayDiff = -1000; //code meaning not calculated
		if(cycle->pregnancyAchievedDate.year != 0)
			pregnancyAchievedDayDiff = DateToDays(focalDate) - DateToDays(cycle->pregnancyAchievedDate);
		
		
		if(allowGlobal)
			CycleReleaseCycle();
		else
			CycleReleaseCycleNonGlobal(dbCycles, cycle, cycleIndex, isDirty);
		
		if(cycleDay) *cycleDay = dayDiff;

		//future specific stuff
		cycleRatio = 0;//set for calculations on past dates		
		DateSecondsToDate(TimGetSeconds(), &today);	//get today	
		//Extrapolate M's only in the future or for today
		//currently this does not adjust second predictions and beyond for adjustments
		//in the first prediction due to late cycles or luteal adjustments, although this could be implemented
		if(DateTypeCmp(focalDate,today) >= 0) //today or in the future
		{
			lutealDiff = 1000; //set to a high max
			//first use Luteal Length if at least one peak day has been recorded
			if(tempDayDiff>0 || billingsDayDiff>0 || cervixDayDiff>0 || LHTestDayDiff>0)
			{
				//find smallest one, that is still greater than 0
				if(tempDayDiff>0)
				{
					if(tempDayDiff < lutealDiff)
						lutealDiff = tempDayDiff;
				}
				if(billingsDayDiff>0)
				{
					if(billingsDayDiff < lutealDiff)
						lutealDiff = billingsDayDiff;
				}
				if(cervixDayDiff>0)
				{
					if(cervixDayDiff < lutealDiff)
						lutealDiff = cervixDayDiff;
				}
				if(LHTestDayDiff>0)
				{
					if(LHTestDayDiff < lutealDiff)
						lutealDiff = LHTestDayDiff;
				}
				
					
				//if today
				if(DateTypeCmp(focalDate,today) == 0)
				{
					//and m was predicted for today or in the past	
					if(lutealDiff >= p_prefs->aveLutealLength)
					{
						if(!isOldRecord)
							phase = 1;
					}
				
				}
				else //future
				{
					// if m is predicted for focalDate
					if(lutealDiff == p_prefs->aveLutealLength)
					{
							phase = 1;
					}
					else //see if m was predicted for past day, focalDate is tomorrow, and there is data for "today"
					{
						if((lutealDiff > p_prefs->aveLutealLength) && (DateToDays(focalDate) - DateToDays(today) == 1))
						{
							//check to see if data for today, which has to be most previous index
							if( index>1 )
							{
								//presumably, if three is no data, index points one position forward,
								//therefore I've backed up index by one to see if the most recent data
								//record is "today"
								if(allowGlobal)
									item = DataQueryData(index-1);
								else
									item = DataQueryDataNonGlobal(index-1,dbDays,&isDirtyII);
								if(DateTypeCmp(item->recDate,today) == 0) //if date exists, then there is data for that day
									phase = 1;
								if(allowGlobal)
									DataReleaseData();
								else
									DataReleaseDataNonGlobal(dbDays,item,index-1,isDirtyII);
							}
						}				
					}
				}
			}
			
			//if NonGlobal, can't use cycleRatio comparisons because it references a global library,
			//but this shouldn't be needed anyway in cases where NonGlobal is called
			if(allowGlobal) 
			{
			//second use cycleRatio
			//if lutealDiff == 1000, then wasn't calculated, use all cycleRatio info
			//if lutealDiff != 1000, then only use cycleRatio info > 2.0
			cycleRatio = float(dayDiff - 1)/float(p_prefs->aveCycleLength); //used to be s_aveCycleLength
			if( lutealDiff==1000 || cycleRatio >= 2.0 )
			{
				//this marks m on today
				if(DateTypeCmp(focalDate,today) == 0)
				{
					if( cycleRatio >= 1.0 )
					{
						if(!isOldRecord)  //this is important so that I don't mark an m on today once data has been entered
							phase = 1;
					}
				}
				//this marks m on future days
				else
				{
					//if m is predicted for focalDate
					if( float(cycleRatio) == Int16(cycleRatio) )
					{
						phase = 1; //this is important so that I don't mark an m on today once data has been entered(don't really need this anymore)
					}
					else //see if m was predicted for past day, focalDate is tomorrow, and there is data for "today"
					{				
						if((cycleRatio > 1.0) && (DateToDays(focalDate) - DateToDays(today) == 1))
						{
							//check to see if data for today, which has to be most previous index
							if( index>1 )
							{
								//presumably, if three is no data, index points one position forward,
								//therefore I've backed up index by one to see if the most recent data
								//record is "today"
								item = DataQueryData(index-1);
								if(DateTypeCmp(item->recDate,today) == 0) //if date exists, then there is data for that day
									phase = 1;
								DataReleaseData();					
							}
						}		
					}
				}	
			}		
			} //end of if(allowGlobal)

//			//if previous day (i.e. today) has data, perhaps that was the predicted m day, and focalDate should get m mark
//			if(index) //not zero
//			{
//			//note: I could speed this up now that I have restricted future data
//			//only check to see if this is looking at tomorrow
//				item = DataQueryData(index-1);
//				//if previous day (i.e. today) had data on it and we are predicting an m
//				if(!isOldRecord && cycleRatio > 1.0 && DateToDays(focalDate) - DateToDays(item->recDate) == 1 )
//				{
//					phase = 1;
//				}
//				DataReleaseData();
//				
//			}			
		}
		
		//past specific stuff
		if(DateTypeCmp(focalDate,today) <= 0 && phase != 1) //today or in the past and phase hasn't already been set to 1
		{
			if (allowGlobal)
			{
				cycle = CycleGetCycle(cycleIndex);

				//establish if we are in a post-partum cycle
				FindRecordFromDate(cycle->cycleStart, &itemIndex, NULL, NULL);
				item = DataQueryData(itemIndex);
				if(item)
				{	
					if(item->fluidSensation == 50)
						inPostPartumCycle = true;
					else
						inPostPartumCycle = false;
					DataReleaseData();
				}
			}
			else
			{
				cycle = CycleQueryCycleNonGlobal(cycleIndex, dbCycles, &isDirty);
	
				//Open Database
				dbNFP = DmOpenDatabaseByTypeCreator ( RECORD_DB_TYPE, appFileCreator, dmModeReadOnly );
				if(dbNFP != 0)
				{
					//establish if we are in a post-partum cycle
					FindRecordFromDateNonGlobal(cycle->cycleStart, &itemIndex, NULL, NULL, dbNFP);
					item = DataQueryDataNonGlobal(itemIndex, dbNFP, &isDirtyDays);
					if(item)
					{	
						if(item->fluidSensation == 50)
							inPostPartumCycle = true;
						else
							inPostPartumCycle = false;
						DataReleaseDataNonGlobal(dbNFP, item, itemIndex, isDirtyDays);
					}
				}
				if(dbNFP) DmCloseDatabase ( dbNFP );
			}
			
	

			//roetzer Six Day Rule
			if(!inPostPartumCycle && (p_prefs->presets == 2 || p_prefs->presets == 3 || p_prefs->presets == 4)) //use slightly modified special rule for Marquette
			{
				//find out number of cycles
				numberCalc = 1;
				if (allowGlobal)
					numberCalc = CycleGetCount(); //instead of using CalcAve, just get the CycleGetCount, this should be correct in nearly all cases
				if(numberCalc <= 6) //for first six cycles
				{
					//use 5 day rule
					if (dayDiff > 0 && dayDiff < 6) phase = 2;
					if (dayDiff >= 6 && dayDiff < maxCycleCalcDays) phase = 3; //if today or in the past
				}
				else //assume numberCalc > 6
				{
					//reuse numberCalc
					numberCalc = p_prefs->shortestCycleLength - p_prefs->aveLutealLength - 7;
					if (numberCalc < 0)
						numberCalc = 0;
		
					//use numberCalc day rule
					numberCalc++;
					if (dayDiff > 0 && dayDiff < numberCalc) phase = 2;
					if (dayDiff >= numberCalc && dayDiff < maxCycleCalcDays) phase = 3; //if today or in the past	
				}
			}
			else if(!inPostPartumCycle && (p_prefs->presets == 5 || p_prefs->presets == 6 ||p_prefs->presets == 7)) //modified automatic rule for CCL presets
			{
				if(p_prefs->shortestCycleLength >= 26) //use 6 day rule
				{
					if (dayDiff > 0 && dayDiff < 7) phase = 2;
					if (dayDiff >= 7 && dayDiff < maxCycleCalcDays) phase = 3; //if today or in the past
				}
				else if(p_prefs->shortestCycleLength > 22 && p_prefs->shortestCycleLength < 26 ) //use 5 day rule
				{
					if (dayDiff > 0 && dayDiff < 6) phase = 2;
					if (dayDiff >= 6 && dayDiff < maxCycleCalcDays) phase = 3; //if today or in the past
				}
				else if(p_prefs->shortestCycleLength == 22 ) //use 4 day rule
				{
					if (dayDiff > 0 && dayDiff < 5) phase = 2;
					if (dayDiff >= 5 && dayDiff < maxCycleCalcDays) phase = 3; //if today or in the past
				}
				else if(p_prefs->shortestCycleLength <= 21 ) //use 3 day rule
				{
					if (dayDiff > 0 && dayDiff < 4) phase = 2;
					if (dayDiff >= 4 && dayDiff < maxCycleCalcDays) phase = 3; //if today or in the past
				}
			}
			else if(!inPostPartumCycle && p_prefs->roetzerRule == 1) //1 == automatic based on cycle length history
			{
				if(p_prefs->shortestCycleLength > 25 && p_prefs->aveCycleLength >= 27) //use 6 day rule
				{
					if (dayDiff > 0 && dayDiff < 7) phase = 2;
					if (dayDiff >= 7 && dayDiff < maxCycleCalcDays) phase = 3; //if today or in the past
				}
				else if(p_prefs->shortestCycleLength > 25 ) //use 5 day rule
				{
					if (dayDiff > 0 && dayDiff < 6) phase = 2;
					if (dayDiff >= 6 && dayDiff < maxCycleCalcDays) phase = 3; //if today or in the past
				}
				else if(p_prefs->shortestCycleLength <= 25 ) //use 3 day rule
				{
					if (dayDiff > 0 && dayDiff < 4) phase = 2;
					if (dayDiff >= 4 && dayDiff < maxCycleCalcDays) phase = 3; //if today or in the past
				}
			}
			else if (!inPostPartumCycle)
			{
				if (dayDiff > 0 && dayDiff < (p_prefs->roetzerRule + 1)) phase = 2;
				if (dayDiff >= (p_prefs->roetzerRule + 1) && dayDiff < maxCycleCalcDays) phase = 3; //if today or in the past
			}
			else // inPostPartumCycle == true
				phase = 3;

			//Dry Days Rule
			//should add requirment for temperature phase in previous cycle
			if( isOldRecord && p_prefs->dryDaysRule )
			{
				if(allowGlobal)
					item = DataQueryData(index);
				else
					item = DataQueryDataNonGlobal(index,dbDays,&isDirtyII);
				if(cycle->billingsFertilityDate.year == 0 || DateTypeCmp(focalDate, cycle->billingsFertilityDate) < 0 )
				{
					if(item->fluidSensation == 4 || item->fluidSensation == 13 || item->fluidSensation == 12) phase = 2;  //if fluidSensation == Dry, then phase == 1
				}
				if(allowGlobal)
					DataReleaseData();
				else
					DataReleaseDataNonGlobal(dbDays,item,index,isDirtyII);
			}
			
			if(allowGlobal)
				CycleReleaseCycle();
			else
				CycleReleaseCycleNonGlobal(dbCycles, cycle, cycleIndex, isDirty);

			numberFact = 0;
			//Sort through transitions to phase 2
			if( tempDayDiff >= p_prefs->postPeakDays && dayDiff < maxCycleCalcDays) numberFact++;
			if( billingsDayDiff >= effectivePostPeakBillings && dayDiff < maxCycleCalcDays) numberFact++;
			if( cervixDayDiff >= p_prefs->postPeakDays && dayDiff < maxCycleCalcDays) numberFact++;
			if( LHTestDayDiff >= p_prefs->postPeakDays && dayDiff < maxCycleCalcDays) numberFact++;
			
			
			if( numberFact >= p_prefs->factorsForPostPeak) phase = 5;
			if(tempDayDiff == 0) phase = 4;
			if(billingsDayDiff == 0) phase = 4;
			if(cervixDayDiff == 0) phase = 4;
			if(LHTestDayDiff == 0) phase = 4;
			
			//may be pregnant, check for 20 high temps
			if(pregnancyAchievedDayDiff == 0)
			{
				phase = 6;
				
				if(allowGlobal) //only display Pregnancy Message if called globally
				{
				if( !s_displayedPregnancyMessage )
				{
					cycle = CycleGetCycle(cycleIndex);
					dueDate = cycle->tempPeakDate;
					DateAdjust(&dueDate,266); //based on St. Agnes Hospital Info
					CycleReleaseCycle();
				
			        frmP = FrmInitForm (PregnantForm);

		        	//Draw in due date	    
					objectIndex = FrmGetObjectIndex(frmP,PregnantDueDateLabel);
					FrmHideObject(frmP,objectIndex);
					pString = (Char*)MemPtrNew(longDateStrLength);
					DateToAscii(dueDate.month, dueDate.day, dueDate.year + firstYear, (DateFormatType)PrefGetPreference(prefLongDateFormat), pString); //dfDMYLongWithComma
					FrmCopyLabel(frmP,PregnantDueDateLabel,pString);
					err = MemPtrFree(pString);
					FrmShowObject(frmP,objectIndex);

	    		    s_displayPregnancyMessage = true;
	    		   // s_displayedPregnancyMessage = true;       
			       // FrmDoDialog (frmP);                    
	    		   // FrmDeleteForm (frmP);
    		    }
    		    }
			}
			if(pregnancyAchievedDayDiff > 0) phase = 6;
		}
	}
	
	else //no current cycle
	{
		if(cycleDay)
			*cycleDay = -1; //meaning we have no cycle day
	}
	
	//If current flow, then override previous calculations and mark as M
	if(isOldRecord)
	{
		if(allowGlobal)
			item = DataQueryData(index);
		else
			item = DataQueryDataNonGlobal(index,dbDays,&isDirtyII);
		if(item->fluidSensation==1 || item->fluidSensation==2)
			phase = 1;
		if(allowGlobal)
			DataReleaseData();
		else
			DataReleaseDataNonGlobal(dbDays,item,index,isDirtyII);
	}
	
	if(allowGlobal)
		s_lastCycleIndex = cycleIndex;
	else
	{
		if(dbCycles) DmCloseDatabase ( dbCycles );
		if(dbDays) DmCloseDatabase ( dbDays );
	}
	
	return phase;
}

//this should really be labeled CalcLHTest
void CalcLuteal( void )
{
	NFPDBRecord* item;
	CycleRecord* cycle;
	Boolean isOldCycle;
	Boolean dataExists;
	UInt16 cycleIndex;
	UInt16 itemIndex;
	DateType nextCycleStart;
	Boolean isPeakPattern;
	Err err;
	DateType tempDate;
	Int16 numberCalc;
	Int32 dayDiff;
	UInt16 backCount;
	Int32 latestClearplan;
	Boolean atBegin;
//	Char errMsg[maxStrIToALen];
	
	tempDate.day = 1;
	tempDate.month  = 1;
	tempDate.year = 0;
	
	isPeakPattern = false;
	
	cycleIndex = s_lastCycleIndex;
	
	isOldCycle = FindCycleFromDate(s_currentDate, &cycleIndex, &dataExists);

	if(isOldCycle)
	{
		nextCycleStart.year = 0;
		cycle = CycleQueryCycle(cycleIndex + 1);
		if(cycle)
		{
			nextCycleStart = cycle->cycleStart;
			CycleReleaseCycle();
		}
	
		cycle = CycleGetCycle(cycleIndex);
		FindRecordFromDate(cycle->cycleStart, &itemIndex, NULL, NULL);
		while(true)
		{
			item = DataQueryData(itemIndex);
			if(!item)
				break;
			if(DateTypeCmp(item->recDate,nextCycleStart)==0)
			{
				DataReleaseData();
				break;
			}
			
			if( item->LHTest == 2 ) //positive test or P for clearplan
			{	
				isPeakPattern = true;
				CycleReleaseCycle();		
				err = CycleSaveOld ( SAVELHTestPeakDate, item->recDate, &cycleIndex, NULL );//record date of + as peak date

				cycle = CycleGetCycle(cycleIndex);
			}
		
			DataReleaseData();
		
			itemIndex++;
			//keep going, if multiple +'s, will use the last +;
		}
		
		CycleReleaseCycle();
		
		//if MMII and i haven't found a peak, use alternative
		//after 6 months, the latest peak day in the last six months plus 3 days
		if(g_prefs.presets==3 && !isPeakPattern) //MMII
		{
			numberCalc = CycleGetCount(); //instead of using CalcAve, just get the CycleGetCount, this should be correct in nearly all cases
			if(numberCalc > 6) //needs to be greater because current incomplete cycle will count as 1
			{
			//need to find the longest cycle length which I don't have therefore I have to go into the cycles
				atBegin = false;
				if(cycleIndex > 0)
					cycleIndex--;
				else
					atBegin = true;
				cycle = CycleGetCycle(cycleIndex);
				
				backCount = 0;
				latestClearplan = 0;
				
				while(backCount<6 && !atBegin)
				{
					if(cycle->LHTestPeakDate.year != 0)
					{
						backCount++;
						dayDiff = DateToDays(cycle->LHTestPeakDate) - DateToDays(cycle->cycleStart);
						if(dayDiff > latestClearplan)
							latestClearplan = dayDiff;
					}
					CycleReleaseCycle();
					if(cycleIndex > 0)
						cycleIndex--;
					else
						atBegin = true;
					cycle = CycleGetCycle(cycleIndex);	
				}
				CycleReleaseCycle();
				cycleIndex = s_lastCycleIndex; //replace cycleIndex to current cycle
				
				//just put in the date, regardless of if it is in current cycle
				if(backCount >= 6)
				{
					cycle = CycleGetCycle(cycleIndex);
					DateDaysToDate(DateToDays(cycle->cycleStart) + latestClearplan + 3, &tempDate);
					CycleReleaseCycle();
					isPeakPattern = true;
					err = CycleSaveOld ( SAVELHTestPeakDate, tempDate, &cycleIndex, NULL );

				}

			//	StrIToA(errMsg, tempDate.day);
			//	ErrDisplay(errMsg);

			}
		}
	
		//be sure that tempPeakDate is set to 0 in case prior data was deleted
		if(!isPeakPattern)
		{
			cycle = CycleQueryCycle(cycleIndex);
			CycleReleaseCycle();
			tempDate.year = 0;
			err = CycleSaveOld ( SAVELHTestPeakDate, tempDate, &cycleIndex, NULL );//record date of + as peak date
		}
	}
	
	return;
}


void CalcCervix( void )
{
	NFPDBRecord* item;
	CycleRecord* cycle;
	Boolean isOldCycle;
	Boolean dataExists;
	UInt16 cycleIndex;
	UInt16 itemIndex;
	UInt8 cervixBuffer[3];
	DateType cervixDates[3];
	DateType nextCycleStart;
	Boolean isPeakPattern;
	Err err;
	DateType tempDate;
	
	tempDate.day = 1;
	tempDate.month  = 1;
	tempDate.year = 0;
	
	isPeakPattern = false;
	for(UInt8 i=0;i<=2;i++)
		cervixDates[i].year = 0;
	
	cycleIndex = s_lastCycleIndex;
	
	isOldCycle = FindCycleFromDate(s_currentDate, &cycleIndex, &dataExists);

	if(isOldCycle)
	{
		nextCycleStart.year = 0;
		cycle = CycleQueryCycle(cycleIndex + 1);
		if(cycle)
		{
			nextCycleStart = cycle->cycleStart;
			CycleReleaseCycle();
		}
	
		cycle = CycleGetCycle(cycleIndex);
		FindRecordFromDate(cycle->cycleStart, &itemIndex, NULL, NULL);
		while(true)
		{
			item = DataQueryData(itemIndex);
			if(!item)
				break;
			if(DateTypeCmp(item->recDate,nextCycleStart)==0)
			{
				DataReleaseData();
				break;
			}
			
			if( item->cervix !=0 ) 
			{	
				//move down dates
				for(UInt8 i=2;i>0;i--)
				{
					cervixDates[i] = cervixDates[i-1];
					cervixBuffer[i] = cervixBuffer[i-1];
				}
				cervixDates[0] = item->recDate;
				cervixBuffer[0] = item->cervix;
			}
		
			//if peak has already been detected, don't check
			if(!isPeakPattern)
			{
				//buffer is full, check for a positive result
				if( cervixDates[2].year != 0 )
				{
		
					//confirm first one is at 3, second is at 2 or 1, third is at 1
					if( cervixBuffer[2] == 3 && ( cervixBuffer[1] == 2 ||  cervixBuffer[1] == 1) &&  cervixBuffer[0] == 1 )
					{
						isPeakPattern = true;
					}
	
					if(isPeakPattern)
					{			
						CycleReleaseCycle();		
						err = CycleSaveOld ( SAVEcervixPeakDate, cervixDates[2], &cycleIndex, NULL );

						cycle = CycleGetCycle(cycleIndex);			
					}
				}
			}
				
			//check for a second peak
			if(isPeakPattern)
			{
				if(item->cervix >1)
					isPeakPattern = false;
			}
		
			DataReleaseData();
		
			itemIndex++;
		}
		
		CycleReleaseCycle();
	
		//be sure that tempPeakDate is set to 0 in case prior data was deleted
		if(!isPeakPattern)
		{
			cycle = CycleQueryCycle(cycleIndex);		
			CycleReleaseCycle();
			tempDate.year = 0;
			err = CycleSaveOld ( SAVEcervixPeakDate, tempDate, &cycleIndex, NULL );

		}
	}
	
	return;
}


void CalcBillings( void ) //technically this refers to the ovulation method and not the billings
{
	NFPDBRecord* item;
	CycleRecord* cycle;
	Boolean isOldCycle;
	Boolean dataExists;
	UInt16 cycleIndex;
	UInt16 itemIndex;
	UInt8 fluidSensationBuffer[6];
	UInt8 fluidColorBuffer[6];
	UInt8 dryBuffer[4];
	DateType fluidDates[6];
	DateType nextCycleStart;
	Boolean isPeakPattern;
	Boolean isStillDryDays;
	Err err;
	DateType tempDate;
	UInt8 effectivePostPeak;
	Boolean inPostPartumCycle = false;
	
	if( g_prefs.presets == 7 ) //CCL Rule R
		effectivePostPeak	= 3;
	else if ( g_prefs.presets == 8 ) //CCL Rule A
		effectivePostPeak	= 2; 
	else //For CCL Rules C and B, postPeakDays should already be 4
		effectivePostPeak	= g_prefs.postPeakDays; //store this in a new variable up front so it can be adjusted for CCL presets
	
	tempDate.day = 1;
	tempDate.month  = 1;
	tempDate.year = 0;
	
	for(UInt8 i=0;i<=5;i++)
		fluidDates[i].year = 0;
	
	cycleIndex = s_lastCycleIndex;
	
	isOldCycle = FindCycleFromDate(s_currentDate, &cycleIndex, &dataExists);
	
	if(isOldCycle == false)
		return;

	nextCycleStart.year = 0;
	cycle = CycleQueryCycle(cycleIndex + 1);
	if(cycle)
	{
		nextCycleStart = cycle->cycleStart;
		CycleReleaseCycle();
	}
	
	cycle = CycleGetCycle(cycleIndex);
	FindRecordFromDate(cycle->cycleStart, &itemIndex, NULL, NULL);
	
	isPeakPattern = false;
	isStillDryDays = true;
	for(Int8 i=3;i>=0;i--)
		dryBuffer[i] = 0;

	//establish if we are in a post-partum cycle
	item = DataQueryData(itemIndex);
	if(item)
	{	
		if(item->fluidSensation == 50)
			inPostPartumCycle = true;
		else
			inPostPartumCycle = false;
		DataReleaseData();
	}

	while(true)
	{
		item = DataQueryData(itemIndex);
		if(!item)
			break;
		if(DateTypeCmp(item->recDate,nextCycleStart)==0)
		{
			DataReleaseData();
			break;
		}
			
		//check for dry day
		if(isStillDryDays)
		{
			//if greater than dry, and not M Rating 2 or 3 which are supposed to be basic infertility pattern
			if(item->fluidSensation > 4 && item->fluidSensation != 12 && item->fluidSensation != 13 && item->fluidSensation != 50)
			{
				isStillDryDays = false;				
				CycleReleaseCycle();
				err = CycleSaveOld ( SAVEbillingsFertilityDate, item->recDate, &cycleIndex, NULL );

				cycle = CycleGetCycle(cycleIndex);
			}
		}
		//isStillDryDays == false && current is dry
		//only do this section if in post-partum cycle
		else if( inPostPartumCycle == true && (item->fluidSensation == 4 || item->fluidSensation == 12 || item->fluidSensation == 13))
		{
			for(Int8 i=3;i>=0;i--)
			{
				if( dryBuffer[i] == 0 )
				{
					dryBuffer[i] = 1;
					break;
				}
			}

			//if full dry Buffer, then reset everything
			if( dryBuffer[0] == 1 )
			{
				isStillDryDays = true;
				for(Int8 i=3;i>=0;i--)
					dryBuffer[i] = 0;
			}
		}
		//clear dry Buffer if sensation or color is something besides dry
		else if( inPostPartumCycle == true && item->fluidSensation != 0 && item->fluidSensation != 4 && item->fluidSensation != 12 && item->fluidSensation != 13 ) 
		{
			for(Int8 i=3;i>=0;i--)
				dryBuffer[i] = 0;
		}

		if( (item->fluidColor!=0 || item->fluidSensation!=0) && item->fluidSensation != 1 && item->fluidSensation != 2 && item->fluidSensation != 50) 
		{	
			//move down dates
			for(UInt8 i=5;i>0;i--)
			{
				fluidDates[i] = fluidDates[i-1];
				fluidColorBuffer[i] = fluidColorBuffer[i-1];
				fluidSensationBuffer[i] = fluidSensationBuffer[i-1];
			}
			fluidDates[0] = item->recDate;
			fluidSensationBuffer[0] = item->fluidSensation;
			fluidColorBuffer[0] = item->fluidColor;
		}
		
		//currently doesn't account for ovulatory spotting
		//if peak has already been detected, don't check
		if(!isPeakPattern)
		{
			//buffer is full, check for a positive coverline result
			if( fluidDates[ effectivePostPeak + 1 ].year != 0 )
			{
				//confirm first one is at least sticky/cloudy (2) or moist(5), in versions 2.1 and eariler required this to be lubri (6).  But this meant one needed two lubricated days in a row which seemed unneccesary
				//at least moist(5) and not M Rating 2 or 3 (12 or 13) which are Basic Infertility Pattern
				if( fluidColorBuffer[effectivePostPeak + 1]>=2 || (fluidSensationBuffer[effectivePostPeak + 1] >= 5 && fluidSensationBuffer[effectivePostPeak + 1] != 12 && fluidSensationBuffer[effectivePostPeak + 1] != 13 && fluidSensationBuffer[effectivePostPeak + 1] != 50))
				{
					//confirm peak day is at least creamy (3) or lubi (6) or M Rating 7-8 (6 or 18)
					if(fluidColorBuffer[effectivePostPeak]>=3 || fluidSensationBuffer[effectivePostPeak] == 6 || fluidSensationBuffer[effectivePostPeak] == 18)
					{
						//confirm drop in fertility on third day; In version 2.4 modified sensation to be not lubi (6) or M Rating 7-8 (6 or 18)
						if( fluidColorBuffer[effectivePostPeak - 1] < fluidColorBuffer[effectivePostPeak] || (fluidSensationBuffer[effectivePostPeak - 1] != 6 && fluidSensationBuffer[effectivePostPeak - 1] != 18))
						{
							//if effectivePostPeak == 2, then make sure 4th is dry
							if( effectivePostPeak == 2 )
							{
								if( (fluidSensationBuffer[0]==4 || fluidSensationBuffer[0]==12 || fluidSensationBuffer[0]==13) && fluidColorBuffer[0] < 3 )
									isPeakPattern = true;
							}
							//or confirm 4th is less than peak; In version 2.4 modified sensation to be not lubi (6) or M Rating 7-8 (6 or 18)
							else if( fluidColorBuffer[effectivePostPeak - 2] < fluidColorBuffer[effectivePostPeak] || (fluidSensationBuffer[effectivePostPeak - 2] != 6 && fluidSensationBuffer[effectivePostPeak - 2] != 18) )
							{
								//confirm 5th and 6th are dry or M Rating 2 or 3 (12 or 13)
								if( (fluidSensationBuffer[effectivePostPeak - 3]==4 || fluidSensationBuffer[effectivePostPeak - 3]==12 || fluidSensationBuffer[effectivePostPeak - 3]==13 ) && fluidColorBuffer[effectivePostPeak - 3] < 3)
								{
									if(effectivePostPeak == 4 && (fluidSensationBuffer[0]==4 || fluidSensationBuffer[0]==12 || fluidSensationBuffer[0]==13) && fluidColorBuffer[0] < 3)
									{
										isPeakPattern = true;
									}
									else if(effectivePostPeak == 3)
									{
										isPeakPattern = true;
									}
								}
						
							}
						}
					}
				}
	
				if(isPeakPattern)
				{			
					CycleReleaseCycle();		
					err = CycleSaveOld ( SAVEbillingsPeakDate, fluidDates[effectivePostPeak], &cycleIndex, NULL );

					cycle = CycleGetCycle(cycleIndex);			
				}
		
			}
		}
		
		//check for a second peak
		if(isPeakPattern)
		{
			if(item->fluidSensation == 6 || item->fluidSensation == 18 || item->fluidColor >1)
				isPeakPattern = false;
		}
		
		DataReleaseData();
		
		itemIndex++;
	}
	
	CycleReleaseCycle();
	
	//be sure that tempPeakDate is set to 0 in case prior data was deleted
	if(!isPeakPattern)
	{
		cycle = CycleQueryCycle(cycleIndex);				
		CycleReleaseCycle();		
		tempDate.year = 0;
		err = CycleSaveOld ( SAVEbillingsPeakDate, tempDate, &cycleIndex, NULL );
		
	}
	
	
	//if StillDryDays hasn't flipped to false, then be sure FertilityDate is still blank
	if(isStillDryDays)
	{
		cycle = CycleGetCycle(cycleIndex);				
		CycleReleaseCycle();		
		tempDate.year = 0;
		err = CycleSaveOld ( SAVEbillingsFertilityDate, tempDate, &cycleIndex, NULL );
		
	}

	return;
}


void CalcTemp ( void )
{
	NFPDBRecord* item;
	CycleRecord* cycle;
	Boolean isOldCycle;
	Boolean dataExists;
	UInt16 cycleIndex;
	UInt16 itemIndex;
	UInt8 coverlineBuffer[9];
	DateType coverlineDates[9];
	UInt8 pregBuffer[20];
	DateType pregDates[20];	
	DateType nextCycleStart;
	UInt8 maxTemp = 0; //gets rid of prc-tools warning
	UInt8 maxTempOverLine;
	UInt8 maxTempForExtended[4]; //records the highest temp
	Int16 tempAdjust;
	Boolean isOverCoverline = false; //gets rid of prc-tools warning
	Boolean isOverPregline;
	Err err;
	UInt8 i;
	DateType tempDate;
	Int8 cLength;
	Int8 cTemp;
	
	tempDate.day = 1;
	tempDate.month  = 1;
	tempDate.year = 0;

	maxTempForExtended[0] = 0;

	for(UInt8 i=0;i<=8;i++)
		coverlineDates[i].year = 0;
	
	cycleIndex = s_lastCycleIndex;
	
	isOldCycle = FindCycleFromDate(s_currentDate, &cycleIndex, &dataExists);
	
	if(isOldCycle == false)
		return;

	nextCycleStart.year = 0;
	cycle = CycleQueryCycle(cycleIndex + 1);
	if(cycle)
	{
		nextCycleStart = cycle->cycleStart;
		CycleReleaseCycle();
	}
	
	cycle = CycleGetCycle(cycleIndex);
	FindRecordFromDate(cycle->cycleStart, &itemIndex, NULL, NULL);
	
	while(true)
	{
		item = DataQueryData(itemIndex);
		if(!item)
			break;
		if(DateTypeCmp(item->recDate,nextCycleStart)==0)
		{
			DataReleaseData();
			break;
		}
			
		//ignore temperature if alcohol is involved	or more than 2 hours off		
		if( item->basalTemp!=0 && item->alcohol==0 && (item->time!=0 || item->time!=8)) 
		{	
			//move down dates
			for(UInt8 i=8;i>0;i--)
			{
				coverlineDates[i] = coverlineDates[i-1];
				coverlineBuffer[i] = coverlineBuffer[i-1];
			}
			for(UInt8 i=3;i>0;i--)
			{
				maxTempForExtended[i] = maxTempForExtended[i-1];			
			}
			coverlineDates[0] = item->recDate;

			if(g_prefs.timeAdjustMode == 0 || g_prefs.timeAdjustMode == 1)
			{
				//time adjustments of 0.1 F or 0.05 C per half hour (earlier versions were 0.1 C per hour
				tempAdjust = item->time - 4; //so that entry of 4 is 0 adjustment
				if(g_prefs.timeAdjustMode == 1 /*&& g_prefs.tempUnits==1*/) //previousl on half rate in F, now in both
					//change time adjustments to 1/10 F per hour
					tempAdjust = tempAdjust/2;		
				//if(g_prefs.tempUnits==2) //if in C, then 1/10 C per hour
				//		tempAdjust = tempAdjust/2;
			}
			else
				tempAdjust = 0;
			
			coverlineBuffer[0] = item->basalTemp - tempAdjust;
			
			//if past the first four days of the cycle, record max in maxTempForExtended
			if(DateToDays(item->recDate)-DateToDays(cycle->cycleStart) >= 4) //ie ignore day 1-4
			{
				if(coverlineBuffer[0] > maxTempForExtended[1])
					maxTempForExtended[0] = coverlineBuffer[0];
				else
					maxTempForExtended[0] = maxTempForExtended[1];
			}
		}
		
		isOverCoverline = false;
		//buffer is full, check for a positive coverline result
		//if postPeakDays==4, then check position 8, if postPeakDays==3, then check position 7
		if( coverlineDates[4 + g_prefs.postPeakDays ].year != 0 )
		{
			if(g_prefs.tempMethod == 1 || g_prefs.tempMethod == 4 || g_prefs.tempMethod == 5 || g_prefs.tempMethod == 6) //coverline methods
			{
				//find max of last 6
				maxTemp = 0;
				if(g_prefs.postPeakDays==4)
				{
					for(UInt8 i=3; i<=8; i++)
					{
						if(coverlineBuffer[i] > maxTemp)
							maxTemp=coverlineBuffer[i];
					}
				}
				else if(g_prefs.postPeakDays==3)
				{
					for(UInt8 i=2; i<=7; i++)
					{
						if(coverlineBuffer[i] > maxTemp)
							maxTemp=coverlineBuffer[i];
					}
				}
		
				//isOverCoverline == true means all three recent temps are over the coverline
				//assume they are over
				isOverCoverline = true;
				maxTempOverLine = 1;
				
				if(g_prefs.tempMethod == 4) //full thermal
				{
					for(UInt8 i=0; i<=(g_prefs.postPeakDays-2); i++)
					{
						//use same code for F and C because increments are similar with revised scalling in version 2.6
						//if any one temp is not 0.4 F over low, mark as false
						if(coverlineBuffer[i] <= maxTemp + 3)
							isOverCoverline = false;
					}
				}
				else if(g_prefs.tempMethod == 5) //strong thermal
				{
					// most recent temp must be 0.4 F over low
					if(coverlineBuffer[0] <= maxTemp + 3)
						isOverCoverline = false;
					//remaining temps must be 0.2 F over low
					for(UInt8 i=1; i<=(g_prefs.postPeakDays-2); i++)
					{
						//use same code for F and C because increments are similar with revised scalling in version 2.6
						//if any one temp is not 0.2 F over low, mark as false
						if(coverlineBuffer[i] <= maxTemp + 1)
							isOverCoverline = false;
					}
				}
				else if(g_prefs.tempMethod == 6) //overall thermal
				{
					//but only do this section if don't find a strong thermal shift 
					
					//strong thermal shift code
					// most recent temp must be 0.4 F over low
					if(coverlineBuffer[0] <= maxTemp + 3)
						isOverCoverline = false;
					//remaining temps must be 0.2 F over low
					for(UInt8 i=1; i<=(g_prefs.postPeakDays-2); i++)
					{
						//use same code for F and C because increments are similar with revised scalling in version 2.6
						//if any one temp is not 0.2 F over low, mark as false
						if(coverlineBuffer[i] <= maxTemp + 1)
							isOverCoverline = false;
					}
	
					if( isOverCoverline == false )
					{
						isOverCoverline = true; //reset and try again using overall shift rules
						for(UInt8 i=1; i<=(g_prefs.postPeakDays-2); i++)
						{
							//use same code for F and C because increments are similar with revised scalling in version 2.6
							// temp must be 0.1 F over low, ie through coverline
							if(coverlineBuffer[i] <= maxTemp)
								isOverCoverline = false;
							//at the same time track max
							if(coverlineBuffer[i] > maxTempOverLine)
								maxTempOverLine = coverlineBuffer[i];
						}
						//at least one temp needs to be 0.4F above line
						if(maxTempOverLine <= maxTemp + 3)
							isOverCoverline = false;
						//vague statement that there needs to be a rising pattern
						//therefore, ensure last temp is higher than first temp
						if(coverlineBuffer[0] <= coverlineBuffer[g_prefs.postPeakDays-2])
							isOverCoverline = false;
					}				
				}
				else //standard coverline method, g_prefs.tempMethod == 1
				{
					for(UInt8 i=0; i<=(g_prefs.postPeakDays-2); i++)
					{
						if(g_prefs.tempUnits==1)//F
						{
							//if any one temp is not over, mark as false
							if(coverlineBuffer[i] <= maxTemp +1 )
								isOverCoverline = false;
						}
						else //C
						{
							//if any one temp is not over, mark as false
							if(coverlineBuffer[i] <= maxTemp)
								isOverCoverline = false;
							//in addition record max of these two or three temps, at least one must be greater than maxTemp + 3;
							if(coverlineBuffer[i] > maxTempOverLine)
								maxTempOverLine = coverlineBuffer[i];
						}
					}
					if(g_prefs.tempUnits==2)//C
					{
						if(maxTempOverLine <= maxTemp + 3) //this was +1 before C was re-scaled
							isOverCoverline = false;
					}
				}
			}
			else if (g_prefs.tempMethod == 2) //average method
			{
				//find average of last 6
				maxTemp = 0;
				if(g_prefs.postPeakDays==4)
				{
					for(UInt8 i=3; i<=8; i++)
					{
						maxTemp += coverlineBuffer[i];
					}
				}
				else if(g_prefs.postPeakDays==3)
				{
					for(UInt8 i=2; i<=7; i++)
					{
						maxTemp += coverlineBuffer[i];
					}
				
				}
							
				//confirm this works without first converting to floats
				maxTemp = maxTemp / 6;
				
				//isOverCoverline == true means all three recent temps are over the coverline
				//assume they are over
				isOverCoverline = true;
				for(UInt8 i=0; i<=(g_prefs.postPeakDays-2); i++)
				{
					if(g_prefs.tempUnits==1)//F
					{
						//if any one temp is not over, mark as false
						if(coverlineBuffer[i] <= maxTemp + 3 ) //need to be 0.4 over average to be considered over line
							isOverCoverline = false;
					}
					else //C
					{
						//if any one temp is not over, mark as false
						//I'm basing this on the F rule: 0.3 over average to be considered over line
						//0.2 would be a round down, but 0.3 is more conservative
						//With this version, change to 0.25, there for instead of +2, have +4
						if(coverlineBuffer[i] <= maxTemp + 4 ) 
							isOverCoverline = false;
					}
				}
			}	
			else if(g_prefs.tempMethod == 3) //extended coverline
			{
				//just use maxTempForExtended instead of the buffer
				//because it will contain the buffer max and prebuffer max
				maxTemp = maxTempForExtended[g_prefs.postPeakDays-1];
		
				//isOverCoverline == true means all three recent temps are over the coverline
				//assume they are over
				isOverCoverline = true;
				maxTempOverLine = 1;
				for(UInt8 i=0; i<=(g_prefs.postPeakDays-2); i++)
				{
					if(g_prefs.tempUnits==1)//F
					{
						//if any one temp is not over, mark as false
						if(coverlineBuffer[i] <= maxTemp +1 )
							isOverCoverline = false;
					}
					else //C
					{
						//if any one temp is not over, mark as false
						if(coverlineBuffer[i] <= maxTemp)
							isOverCoverline = false;
						//in addition record max of these two or three temps, at least one must be greater than maxTemp + 1
						if(coverlineBuffer[i] > maxTempOverLine)
							maxTempOverLine = coverlineBuffer[i];
					}
				}
				if(g_prefs.tempUnits==2)//C
				{
					if(maxTempOverLine <= maxTemp + 1)
						isOverCoverline = false;
				}
			}


			if(isOverCoverline)
			{
				DateAdjust(&coverlineDates[ g_prefs.postPeakDays-2 ],-2); //just use date in list to adjust for peak date
				tempDate = cycle->cycleStart; //used to store Extended Coverline coverlineStartDate
				CycleReleaseCycle();		
				err = CycleSaveOld ( SAVEtempPeakDate, coverlineDates[ g_prefs.postPeakDays-2 ], &cycleIndex, NULL );
				
				//save coverline info into cycle database
				//save coverlineStartDate and coverlineLength		
				if(g_prefs.tempMethod == 1 || g_prefs.tempMethod == 2 || g_prefs.tempMethod == 4 || g_prefs.tempMethod == 5 || g_prefs.tempMethod == 6)
				{
					if(g_prefs.postPeakDays == 4)
					{
						err = CycleSaveOld ( SAVEcoverlineStartDate, coverlineDates[ 8 ], &cycleIndex, NULL );
						cLength = DateToDays(coverlineDates[0]) - DateToDays(coverlineDates[8]) + 1;
					}
					else //g_prefs.postPeakDays == 3
					{
						err = CycleSaveOld ( SAVEcoverlineStartDate, coverlineDates[ 7 ], &cycleIndex, NULL );
						cLength = DateToDays(coverlineDates[0]) - DateToDays(coverlineDates[7]) + 1;
					}
				}
				else //(g_prefs.tempMethod == 3)
				{
					DateAdjust(&tempDate, 4); //extended coverline always starts on day 5 of the cycle
					err = CycleSaveOld ( SAVEcoverlineStartDate, tempDate, &cycleIndex, NULL );
					cLength = DateToDays(coverlineDates[0]) - DateToDays(tempDate) + 1;
				}
				err = CycleSaveOld ( SAVEcoverlineLength, tempDate, &cycleIndex, &cLength );
				
				//save coverlineTemp
				if(g_prefs.tempMethod == 1 || g_prefs.tempMethod == 3 || g_prefs.tempMethod == 4 || g_prefs.tempMethod == 5 || g_prefs.tempMethod == 6)
				{
					cTemp = maxTemp + 1; //draw above the maxTemp
					if(g_prefs.tempUnits==2) //for C drop down 0.5 degrees when plotting, indicate this by negative
						cTemp = -cTemp;
				}
				else //g_prefs.tempMethod == 2
					cTemp = maxTemp; //put line at the average

				err = CycleSaveOld ( SAVEcoverlineTemp, tempDate, &cycleIndex, &cTemp );

				cycle = CycleGetCycle(cycleIndex);
			}
		
		}
		DataReleaseData();	
		if(isOverCoverline) break;
		itemIndex++;
	}
	
	//continue with temps to check for pregnancy achieved
	if(isOverCoverline)
	{
		//copy coverline buffers into pregBuffers
		//using different order, now most recent is highest number
		for(UInt8 i=0;i<=(g_prefs.postPeakDays-2);i++)
		{
			pregDates[i] = coverlineDates[(g_prefs.postPeakDays-2)-i];
			pregBuffer[i] = coverlineBuffer[(g_prefs.postPeakDays-2)-i];
		}
		
		//set final spot to zero so I know when full
		pregDates[19].year = 0;
	
	}
	
	i=(g_prefs.postPeakDays-2)+1;
	itemIndex++;
	while(isOverCoverline && pregDates[19].year == 0)
	{
		item = DataQueryData(itemIndex);
		if(!item)
			break;
		if(DateTypeCmp(item->recDate,nextCycleStart)==0)
		{
			DataReleaseData();
			break;
		}
	
		//if no alcohol or not too large time adjust	
		if( item->basalTemp!=0 && item->alcohol==0 && (item->time!=0 || item->time!=8))
		{
			pregDates[i] = item->recDate;

			if(g_prefs.timeAdjustMode == 0 || g_prefs.timeAdjustMode == 1)
			{
				//time adjustments of 0.1 F or 0.05 C per half hour (earlier versions were 0.1 C per hour
				tempAdjust = item->time - 4; //so that entry of 4 is 0 adjustment
				if(g_prefs.timeAdjustMode == 1 /*&& g_prefs.tempUnits==1*/) //previousl on half rate in F, now in both
					//change time adjustments to 1/10 F per hour
					tempAdjust = tempAdjust/2;		
				//if(g_prefs.tempUnits==2) //if in C, then 1/10 C per hour
				//		tempAdjust = tempAdjust/2;
			}
			else
				tempAdjust = 0;
			
			pregBuffer[i] = item->basalTemp - tempAdjust;
		}
		
		DataReleaseData();
		itemIndex++;
		i++;
	}
	
	isOverPregline = false;
	//if filled pregBuffer
	if(pregDates[19].year != 0)
	{	
		isOverPregline = true;
		if(g_prefs.tempMethod == 1 || g_prefs.tempMethod==3 || g_prefs.tempMethod==4 || g_prefs.tempMethod==5 || g_prefs.tempMethod==6)
		{
			for(UInt8 i=0; i<=19; i++)
			{
				if(g_prefs.tempUnits==1)//F
				{	
					//if any one temp is not over, mark as false
					if(pregBuffer[i] <= maxTemp +1 )
						isOverPregline = false;
				}
				else //C
				{
					//if any one temp is not over, mark as false; add +1 when rescaled to 0.05
					if(pregBuffer[i] <= maxTemp )
						isOverPregline = false;
				}
			}
		}
		else if(g_prefs.tempMethod == 2)
		{
			for(UInt8 i=0; i<=19; i++)
			{			
				if(g_prefs.tempUnits==1)//F
				{
					//if any one temp is not over, mark as false
					if(pregBuffer[i] <= maxTemp + 3 ) //need to be 0.4 over average to be considered over line
						isOverPregline = false;
				}
				else
				{
					//if any one temp is not over, mark as false
					if(pregBuffer[i] <= maxTemp + 4 ) //need to be 0.25 over average to be considered over line
						isOverPregline = false;				
				}
			}
		}
	}
	
	CycleReleaseCycle();
	
	if(isOverPregline)
	{
		cycle = CycleQueryCycle(cycleIndex);				
		CycleReleaseCycle();		
		err = CycleSaveOld ( SAVEpregnancyAchievedDate, pregDates[19], &cycleIndex, NULL );
	
	}
	
	else if(!isOverPregline)
	{
		cycle = CycleQueryCycle(cycleIndex);				
		CycleReleaseCycle();		
		tempDate.year = 0;
		err = CycleSaveOld ( SAVEpregnancyAchievedDate, tempDate, &cycleIndex, NULL );
	
	}
	
	//be sure that tempPeakDate is set to 0 in case prior data was deleted
	if(!isOverCoverline)
	{
		cycle = CycleQueryCycle(cycleIndex);				
		CycleReleaseCycle();		
		tempDate.year = 0;
		err = CycleSaveOld ( SAVEtempPeakDate, tempDate, &cycleIndex, NULL );
		cLength = 0;
		err = CycleSaveOld ( SAVEcoverlineLength, tempDate, &cycleIndex, &cLength );
	}
	
	return;
}

//  FUNCTION: CalcPhase
//
//  DESCRIPTION: Recalculates cycle start date and mananges calling of functions
//				that calculate peak dates.  A lot of logic is required here.
//				perhaps this could be optimized somewhat		
//
//  PARAMETERS: take true if record was just added, takes false if record was just deleted
//
//	RETURNS: 
void CalcPhase( Boolean added )
{
	NFPDBRecord* item;
	CycleRecord* cycle;
	Int32 dayDiff;
	Int32 dayDiffNext = 0; //eliminates prc-tools warning
	Boolean isOldCycle;
	Boolean dataExists;
	Boolean foundM;
	Boolean nextCycleExists;
	UInt16 cycleIndex;
	UInt16 index;
	Err err;
	DateType tempDate;
	DateType tempDate2;
	DateType nextStartDate;
//	ProgressPtr progP;
	FormType* frmP;
	FormType* frmPII;
//	DateType thirdStartDate;

	//display recalculating form
	//should use this even when not added, but causes crash because message
	//has already been deleted, so can't redraw
//	if(added)
//	{
		frmPII = FrmGetActiveForm();
		frmP = FrmInitForm(RecalculateForm);
		FrmDrawForm(frmP);
//	}

	cycleIndex = s_lastCycleIndex;


	//find relevant cycle
	isOldCycle = FindCycleFromDate(s_currentDate, &cycleIndex, &dataExists);

	//if no cycle records, figure out what to do
	//dataExists == false, but also isOldCyle == false


	if(added && isOldCycle)
	{
		//Calendar method
		//find cycle start
		
		//grab currentDate record
		item = DataQueryData(s_currentRecordNumber);
		//compare next cycleStart and s_currentDate to see if it should adjust next cycle
		nextCycleExists = false;
		cycle = CycleQueryCycle(cycleIndex + 1);
		if(cycle) //in case next cycle does not exist
		{
			nextStartDate = cycle->cycleStart;
			dayDiffNext = DateToDays(s_currentDate)-DateToDays(cycle->cycleStart);
			CycleReleaseCycle();
			nextCycleExists = true;
		}
		
		//compare cycleStart and s_currentDate
		cycle = CycleQueryCycle(cycleIndex);
		//dayDiff should be >= 0 because of isOldCycle flag
		dayDiff = DateToDays(s_currentDate)-DateToDays(cycle->cycleStart);
		CycleReleaseCycle();
		
		//this is just some added to code to clean up an errors that might exist
		if(nextCycleExists && dayDiff == dayDiffNext)
		{
			//ERROR found: this cycle and the next have the same start dates
			//delete this cycle and use next
			//cycleIndex; //not sure why this line was here?  commented it out.
			err = CycleDeleteCycle( cycleIndex );
			
			//now the next cycle has become this cycle, redo the top two steps
	
			//compare next cycleStart and s_currentDate to see if it should adjust next cycle
			nextCycleExists = false;
			cycle = CycleQueryCycle(cycleIndex + 1);
			if(cycle) //in case next cycle does not exist
			{
				dayDiffNext = DateToDays(s_currentDate)-DateToDays(cycle->cycleStart);
				CycleReleaseCycle();
				nextCycleExists = true;
			}
		
			//compare cycleStart and s_currentDate
			cycle = CycleQueryCycle(cycleIndex);
			dayDiff = DateToDays(s_currentDate)-DateToDays(cycle->cycleStart);
			CycleReleaseCycle();
			
			//OK, with error corrected, carry on
		}
	
		//first look back because this is where I first look if a day 1 is deleted
		//although exclude dayDiff == 0 for later in the chain
		if( dayDiff <= MAXMENSES && dayDiff!=0 && (item->fluidSensation==1 || item->fluidSensation==2 || item->fluidSensation==50))
		{
			//do nothing, just let new m exists in current cycle
		
		}
		//look forward; compare to next cycle should be ammended to new, create new Cycle Record
		//reset PeakDates
		//in this case dayDiff is always negative
		else if ( nextCycleExists && dayDiffNext >= -MAXMENSES && (item->fluidSensation==1 || item->fluidSensation==2 || item->fluidSensation==50) ) 
		{				
			cycleIndex++;				
			err = CycleDeleteCycle( cycleIndex );
			err = CycleNew ( s_currentDate, &cycleIndex );
			
		//Exlcude the remaining code, this could lead to a cascade of re-parsing.
		//just accept the fact that this parsing alogrithm can't deal with really complicated
		//parsing situations, but it should deal well with real biology
		//if  0 length cycles are generated, they'll be removed
		//Small wierd cycles greater than 0 can be removed by the user
/*			//be sure to look forward from this new start date and parse out any m that
			//might be beyond MAXMENSES.  This code is really just to prevent ghost cycles being generated in unusually cases of entry.
			
			//get 3rd cycle start date
			thirdStartDate.year = 1000; //set really big date
			cycle = CycleQueryCycle(cycleIndex + 1);
			if(cycle) //in case next cycle does not exist
			{
				thirdStartDate = cycle->cycleStart;
				CycleReleaseCycle();
			}
			
			//I can start at the old start date and use dayDiffNext as the starting difference
			//need to look forward from old start date at least MAXMENSES days or until I reach next start date, or until I reach the end of the records
			//if I find an m beyond MAXMENSES from the new start date, that needs to become a cycle
			foundM = false;
			index = s_currentRecordNumber;
			DataReleaseData(); 
			FindRecordFromDate(nextStartDate,&index,NULL,NULL); //know this data point will exist
		
			index++;
			DataQueryData(index);
			//search for other M's beyond MAXMENSES of new start date and within MAXMENSES days after nextStartDate
			while(item && (DateToDays(item->recDate)-DateToDays(nextStartDate)) <= MAXMENSES && DateTypeCmp(item->recDate,thirdStartDate) < 0 )
			{
				//currentDate is the new Start Date
				if ((DateToDays(item->recDate)-DateToDays(s_currentDate)) > MAXMENSES && (item->fluidSensation==1 || item->fluidSensation==2 || item->fluidSensation==50) )
				{
					foundM = true;
					break;
				}
				DataReleaseData();
				index++;
				item = DataQueryData(index);
			}
			if(item) DataReleaseData(); //this only works because there will always be an item and Data to be Released unless it grabbed a NULL item and exited that way.
		
			 //another M beyond MAXMENSES if start date and within MAXMENSES days of old Start Date
			 //this should be its own cycle therefore create new one
			if(foundM)
			{
				item = DataQueryData(index);		
				err = CycleNew ( item->recDate, &cycleIndex );
				tempDate2 = item->recDate;
				DataReleaseData();
				
				//bump up current date and redo cycle calculations for this forward cycle
				//calculations for cycle which current date has joined will be done at end

				tempDate = s_currentDate;
				s_currentDate = tempDate2;
				CalcTemp();
				CalcBillings();
				CalcLuteal();
				CalcCervix();			
				s_currentDate = tempDate;
			
			}
			//otherwise no M found, great, do nothing
			else
			{
				//do nothing
			}

		//set item to NULL so that it doesn't try to release it.
			item = NULL;	
*/
		}
		//if no cycle start by looking back and no cycle start looking forward
		//then new cycle, create new Cycle Record
		else if ( dayDiff > MAXMENSES && (item->fluidSensation==1 || item->fluidSensation==2 || item->fluidSensation==50) ) //used to be dayDiff >7
		{		
			cycleIndex++;
			err = CycleNew ( s_currentDate, &cycleIndex );
		
		}
		
		//what if day 1 has been changed so that it is no longer heavy or medium flow?
		else if (dayDiff == 0 && (item->fluidSensation!=1 && item->fluidSensation!=2 && item->fluidSensation!=50))
		{
			foundM = false;
			index = s_currentRecordNumber;
			//shouldn't have to look backwards.  If there was an M back there it would already be the start day
			//it was a start date; search for other M's within MAXMENSES days after date and make it the start date
			while(item && (DateToDays(item->recDate)-DateToDays(s_currentDate)) <= MAXMENSES ) //used to be 7
			{
	
				if ( (item->fluidSensation==1 || item->fluidSensation==2 || item->fluidSensation==50) )
				{
					foundM = true;
					break;
				}
				DataReleaseData();
				index++;
				item = DataQueryData(index);
			}
			if(item) DataReleaseData(); //this only works because there will always be an item and Data to be Released unless it grabbed a NULL item and exited that way.
		
			if(foundM) //another M within MAXMENSES days (or could be the same M), delete current cycle and create new one
			{
				item = DataQueryData(index);		
				err = CycleDeleteCycle( cycleIndex );
				err = CycleNew ( item->recDate, &cycleIndex );  //is it inserting in properly?
				tempDate2 = item->recDate;
			
				DataReleaseData();
				
				//bump up current date and redo cycle calculations for this forward cycle
				//calculations for cycle which current date has joined will be done at end

				tempDate = s_currentDate;
				s_currentDate = tempDate2;
				CalcTemp();
				CalcBillings();
				CalcLuteal();
				CalcCervix();			
				s_currentDate = tempDate;
			
			}
			//otherwise no M found, just delete current cycle
			else
			{
				err = CycleDeleteCycle( cycleIndex );
				if(cycleIndex > 0) cycleIndex--;
			}

		//set item to NULL so that it doesn't try to release it.
			item = NULL;
		}
		
		if(item) DataReleaseData();  //bad line because if something has already 
									//been released it will still have a non-null
									//item, although I put in a work around above
		
		//Do Temperature calculations
		CalcTemp();
		
		//Do Billings calculations
		CalcBillings();
		
		//Do LHTest calculations
		//- results really don't tell you anything
		CalcLuteal();
		
		//Do Cervix calculations
		CalcCervix();
		
	}
	
	
	else if(added && !isOldCycle && dataExists) //ie at cycle index 0, but record comes before current cycle
	{
		//Calendar method
		//find cycle start
		
		//compare cycleStart and s_currentDate
		item = DataQueryData(s_currentRecordNumber);
		cycle = CycleQueryCycle(cycleIndex);
		dayDiff = DateToDays(item->recDate)-DateToDays(cycle->cycleStart);
		CycleReleaseCycle();
	
		//new cycle, create new Cycle Record
		if ( dayDiff < -MAXMENSES && (item->fluidSensation==1 || item->fluidSensation==2 || item->fluidSensation==50) ) 
		{
			err = CycleNew ( s_currentDate, &cycleIndex );
		
		}
		
		//next cycle should be ammended to new , create new Cycle Record
		//reset PeakDates
		if ( dayDiff >= -MAXMENSES && (item->fluidSensation==1 || item->fluidSensation==2 || item->fluidSensation==50) ) 
		{			
			err = CycleDeleteCycle( cycleIndex );
			err = CycleNew ( s_currentDate, &cycleIndex );
		
		}

		DataReleaseData();
		
		//don't need to do any PeakDate calculations	
	}
	
	
	else if(added && !dataExists)
	{
		//Calendar method
		//find cycle start

		item = DataQueryData(s_currentRecordNumber);
		
		//new cycle, create new Cycle Record
		if ( (item->fluidSensation==1 || item->fluidSensation==2 || item->fluidSensation==50) ) 
		{			
			err = CycleNew ( s_currentDate, &cycleIndex );
		}
		
		DataReleaseData();

		//don't need to do any PeakDate calculations
		
	}
	
	
	else if(!added && dataExists) //either deleted or did nothing, and relevant to a cycle
	{
		//Calendar method
		//find cycle start
		
		//check to see if we are sitting on a start date in which a start date record was deleted
		//compare cycleStart and s_currentDate

		cycle = CycleQueryCycle(cycleIndex);
		dayDiff = DateTypeCmp(s_currentDate,cycle->cycleStart);
		CycleReleaseCycle();
		

		//if was a start date, search for other M's within MAXMENSES days after date and make it the start date
		if(dayDiff == 0)
		{
	
			index = s_currentRecordNumber;
			item = DataQueryData(index);	
			foundM = false;
			while(item && (DateToDays(item->recDate)-DateToDays(s_currentDate)) <= MAXMENSES )
			{
	
				if ( (item->fluidSensation==1 || item->fluidSensation==2 || item->fluidSensation==50) )
				{
					foundM = true;
					break;
				}
				DataReleaseData();
				index++;
				item = DataQueryData(index);
			}
			if(item) DataReleaseData();
		
			if(foundM) //another M within MAXMENSES days, delete current cycle and create new one
			{
				item = DataQueryData(index);	
				err = CycleDeleteCycle( cycleIndex );
				err = CycleNew ( item->recDate, &cycleIndex );
			
				DataReleaseData();
				
				//recalculate Peak Dates
				CalcTemp();
				CalcBillings();
				CalcCervix();
				CalcLuteal();
			
			}
			//otherwise delete current cycle and recalculate Peak Dates for previous cycles, which this date has now joined
			else
			{
				err = CycleDeleteCycle( cycleIndex );
				if(cycleIndex > 0) cycleIndex--;
				//recalculate Peak Dates
				CalcTemp();
				CalcBillings();
				CalcCervix();
				CalcLuteal();
			}
	
		}
		else //possible deleted date is not start date, but could affect Peak Date calculations
		{
			//recalculate Peak Dates
			CalcTemp();
			CalcBillings();
			CalcCervix();
			CalcLuteal();
		}
	
		
	}
	
	else if(!added && !isOldCycle) //either deleted or did nothing, but date sits previous to any defined cycle, so do nothing
	{
	}

	else if(!added && !dataExists) //either deleted or did nothing, but no cycle data Exists, so do nothing
	{
	}

	s_lastCycleIndex = cycleIndex;

	//calculate average cycle length
	CalcAve(NULL);

	//remove recalculating message
//	if(added)
//	{
		FrmEraseForm(frmP);
		FrmDeleteForm(frmP);
		FrmSetActiveForm(frmPII);
//		FrmDrawForm(frmPII);
//	}

//	FrmReturnToForm(0);
	
	return;
}


//returns 0 if none calculated
// 1 if Only Average and Shortest, but not Luteal;
// 2 is all three calculated;
UInt8 CalcAve(UInt8 *numberInAve)
{
	CycleRecord* cycle;
	UInt16 cycleIndex;
	UInt32 aveCycle;
	UInt32 aveLuteal;
	Int32 dayDiff;
	UInt8  shortestCycle;
	Int16 numberOfCycles;
	DateType prevCycleStart;
	DateType prevLatePeakDate;
	UInt8 startCount;
	UInt8 lutealCount;
	UInt8 numberCalculated;
	Err err;
	Boolean errorInThisCycle;
	Boolean isPostPartumCycle;
	UInt16 itemIndex;
	NFPDBRecord* item;
	
	//just calculate over last 12 cycles, and realize that final cycle is not completed

	//if encounter a PostPartum phase, don't count that cycle and start over
	//so that after pregnancy, history begins again because it may be very different
	isPostPartumCycle = false;
	
	//get max cycle number
	numberOfCycles = CycleGetCount();
	
	if(numberOfCycles - 13 < 0)
		cycleIndex = 0;
	else
		cycleIndex = numberOfCycles - 13;
	
	numberCalculated = 0;
	aveCycle = 0;
	aveLuteal = 0;
	lutealCount = 0;
	startCount = 0;
	shortestCycle = 250;//used to be 400, but reduced so as to fit into UInt8
	prevCycleStart.year = 0;
	prevLatePeakDate.year = 0;
	cycle = CycleGetCycle(cycleIndex);		
	while(cycle)
	{
		errorInThisCycle=false;
		if(prevCycleStart.year)
		{
			dayDiff = DateToDays(cycle->cycleStart) - DateToDays(prevCycleStart);
			//This cleans up the error in the Binx data, although I'm not sure how the error was generated in the first place
			if(dayDiff == 0) //Error detected in cycle database, clean it up.  May need to redo CalcPhase, but this will work itself out.  Based on Binx data, not needed.
			{
				//delete the first one
				CycleReleaseCycle();
				cycleIndex--; //guaranteed there is a previous one
				err = CycleDeleteCycle( cycleIndex );
				//don't need to move cycleIndex forward
				cycle = CycleGetCycle(cycleIndex);	//this never be NULL
				
				errorInThisCycle=true;
			}
			else //normal no error situation
			{
				//check to see if PostPartum cycle
				FindRecordFromDate(cycle->cycleStart, &itemIndex, NULL, NULL);
				item = DataQueryData(itemIndex);
				if(item)
				{	
					if(item->fluidSensation == 50)
						isPostPartumCycle = true;
					else
						isPostPartumCycle = false;
					DataReleaseData();
				}
				
				if(isPostPartumCycle == true)
				{
					aveCycle = 0;
					startCount = 0;
					shortestCycle = 0;
					aveLuteal = 0;
					lutealCount = 0;
				}
				else //false
				{
					aveCycle += dayDiff;
					startCount++;
			
					if(dayDiff < shortestCycle)
						shortestCycle = dayDiff;
				}
			}
		}
	
		if(prevLatePeakDate.year && !errorInThisCycle && isPostPartumCycle == false)
		{
			aveLuteal += DateToDays(cycle->cycleStart) - DateToDays(prevLatePeakDate);
			lutealCount++;
		}
		
		//set up for next round
		prevCycleStart = cycle->cycleStart;
		
		//find latest Peak Date (so I don't have to deal with 0 year issue)
		if(DateTypeCmp(cycle->billingsPeakDate,cycle->cervixPeakDate) <=0)
			prevLatePeakDate = cycle->cervixPeakDate;
		else
			prevLatePeakDate = cycle->billingsPeakDate;
		if(DateTypeCmp(prevLatePeakDate,cycle->tempPeakDate) < 0)
			prevLatePeakDate = cycle->tempPeakDate;
		if(DateTypeCmp(prevLatePeakDate,cycle->LHTestPeakDate) < 0)
			prevLatePeakDate = cycle->LHTestPeakDate;
		
		CycleReleaseCycle();
		cycleIndex++;
		cycle = CycleGetCycle(cycleIndex);	
	}

	//don't need a Release because goes until cycle is NULL

	
	if(startCount > 0)
	{
	 aveCycle = (UInt32)(float(aveCycle)/float(startCount) + 0.5);//adding 0.5 effectively rounds the average
	 numberCalculated++;
 	}
 	if(aveCycle == 0)
 		g_prefs.aveCycleLength = defaultCycleLength;
 	else
		g_prefs.aveCycleLength = aveCycle;
	if(numberInAve)
		*numberInAve = startCount;
	
	if(lutealCount > 0)
	{
		numberCalculated++;
		aveLuteal = (UInt32)(float(aveLuteal)/float(lutealCount) + 0.5);//adding 0.5 effectively rounds the average
	}
	if(aveLuteal == 0)
		g_prefs.aveLutealLength = defaultLutealLength;
	else
		g_prefs.aveLutealLength = aveLuteal;
	
	if(shortestCycle != 400)
	{
		if(aveCycle == 0)
			g_prefs.shortestCycleLength = defaultShortestLength ;
		else
			g_prefs.shortestCycleLength = shortestCycle;
	}

	return numberCalculated;
}
