//    Copyright (C) 2003 Chad Brassil
//    This file is part of NFP.
//
//    NFP is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Foobar is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <PalmOS.h>
#include <SysEvtMgr.h>

#include "NFP.h"
#include "NFPRsc.h"
//#include "DateMonth_res.h"

#define CORNERBOX 	32

DateType s_currentDate;
DateType s_cycleStart;
DateType s_lastCycleCheckDate;
static Boolean cleanClose;
static Char **tempTextList;
static Char **tempMRatingList;
static Char **tempSensationList;
static Char **tempFluidColorList;
static UInt8 phase;

// FUNCTION: MainFormDoCommand
//
// DESCRIPTION: This routine preforms all menu commands for both Month View and Chart View.
//
// PARAMETERS:
//
// command
//     menu item id
Boolean MainFormDoCommand(UInt16 command)
{
    Boolean handled = false;
    FormType * frmP;
//    CycleRecord* cycle;
//    UInt16 cycleIndex;
//    Boolean dataExists;
    UInt16 alertChoice;
    

    switch (command)
    {
    case OptionsAboutNFP:

        // Clear the menu status from the display
        MenuEraseStatus(0);

        // Display the About Box.
        frmP = FrmInitForm (AboutForm);
        FrmDoDialog (frmP);                    
        FrmDeleteForm (frmP);

        handled = true;
        break;
        
    case OptionsPassword:

        // Clear the menu status from the display
        MenuEraseStatus(0);  
        FrmGotoForm(SetPasswordForm);
        handled = true;
        break;

    case OptionsPreferences:

        // Clear the menu status from the display
        MenuEraseStatus(0);  
        FrmGotoForm(PreferencesForm);
        handled = true;
        break;

    case OptionsSummary:
    	
        // Display the Summary Box.
        frmP = FrmInitForm (SummaryForm);
        SetSummaryForm (frmP);
        FrmDoDialog (frmP);                    
        FrmDeleteForm (frmP);
        handled = true;
        break;
        
    case OptionsAlarms:
    
    	// Clear the menu status from the display
        MenuEraseStatus(0);  
        FrmGotoForm(SetAlarmsForm);
        handled = true;
        break;
        
    case ViewsCycleChart:
    	MenuEraseStatus(0);    	 
    	FrmGotoForm(TempChartForm);
    	handled = true;
    	break;

    case ViewsMonthView:
    	MenuEraseStatus(0); 
    	FrmGotoForm(MonthViewForm);
    	handled = true;
    	break;
    	
    case ViewsToday:
    	MenuEraseStatus(0);
 		//set to today
		DateSecondsToDate (TimGetSeconds (), &s_currentDate);
    	FrmGotoForm(DataEntryForm);
    	handled = true;
    	break;
    	
    case CycleDelete:
    	
        // confirmation alert
        alertChoice = FrmAlert(CycleDeleteAlert);
        
        if(alertChoice == 0)
        {
        	ChartDeleteEntireCycle();
        	
        	//need to reopen Cycle Chart, because current cycle is now gone.
        	FrmGotoForm(TempChartForm);
        }
                
        handled = true;
        break;
        
	case CycleExport:
    	
        // confirmation alert
        alertChoice = ExportConfirm();
        
        if(alertChoice == 0)
        	ChartExportCycle();
        
        handled = true;
        break;	
    	
    }
    
    return handled;
}


static void SECT3 NoteFormInit(FormType* frmP)
{

	MemHandle textH;
	MemHandle oldtextH;
	Char* pSource;
	Char* pString;
	FieldType *fldP;
	NFPDBRecord* nilP=0; 
	UInt16 offset;
	
	//Draw in date
    pString = (Char *)FrmGetTitle(frmP);
    //this function may cause problems in Japanese OS 3.1.  Can fix it with Glue function
    DateToDOWDMFormat(s_currentDate.month,s_currentDate.day,
    	s_currentDate.year + 1904,(DateFormatType)PrefGetPreference(prefLongDateFormat),pString); //dfMDYLongWithComma
    FrmSetTitle(frmP,pString);	
	
	//Unset from DataEntryForm first (which is technically the active Form)
	GetObjectPtr(fldP,DataEntryNoteField);
	
	// Was the note string modified by the user.
	if (FldDirty (fldP))
		{
		// Release any free space in the note field.
		FldCompactText (fldP);

		// Mark the record dirty.	
		//DirtyRecord (CurrentRecord);
		}

	// Clear the handle value in the field, otherwise the handle
	// will be freed when the form is disposed of,  this call also unlocks
	// the handle that contains the note string.
	textH = FldGetTextHandle (fldP);
	FldSetTextHandle (fldP, 0);

	
	fldP = (FieldType*)FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, NoteNoteField));
	
	//I shouldn't need to get this because it should already be blank
	oldtextH = FldGetTextHandle(fldP);
		
	//in case handle is blank, don't try to move handle or will get error
	if(textH)
	{
		pSource = (Char*)MemHandleLock(textH);		
		if(s_isOldRecord)
		{
			offset = &(nilP->note) - (Char*)nilP;
			pSource += offset;
		}
		else
		{
			offset = 0;	
		}
		
		FldSetText (fldP, textH, offset, StrLen(pSource)+1);
		MemHandleUnlock(textH);
	}
	
	//this should never be needed because the old text handle should be blank
	if (oldtextH != NULL)
			MemHandleFree(oldtextH);			
}

// FUNCTION: DrawTop
//
// DESCRIPTION: Draws top corner squares. Shows cycle day and phase
//
// PARAMETERS: pointer to DataEntry form
//
// RETURNS:
static void SECT3 DrawTop(FormType* frmP)
{

   	RGBColorType	color;
    RectangleType rFrame;

	//this uses a slightly clumsy way of drawing the square in order to get the
	//corners I wanted, but it works.
	
   	if(colorEnabled)
	{
		WinPushDrawState();
		color = PhaseColor (phase);
		WinSetBackColor(WinRGBToIndex(&color));
	}
	//main window
//	rFrame.topLeft.x = 0;
//	rFrame.topLeft.y = 0;
//	rFrame.extent.x = CORNERBOX;
//	rFrame.extent.y = 12;
//	WinEraseRectangle(&rFrame,3);
//	//clean up right corners
//	rFrame.topLeft.x = CORNERBOX-2;
//	rFrame.extent.x = 2;
//	WinEraseRectangle(&rFrame,0);
//	//clean up bottom corner
//	rFrame.topLeft.x = 0;
//	rFrame.topLeft.y = 10;
//	rFrame.extent.x = 2;
//	rFrame.extent.y = 2;
//	WinEraseRectangle(&rFrame,0);
	//main window
	rFrame.topLeft.x = 156-CORNERBOX;
	rFrame.topLeft.y = 0;
	rFrame.extent.x = CORNERBOX-14;  //-14
	rFrame.extent.y = 12;
	WinEraseRectangle(&rFrame,0);
//	//clean up left corners
//	rFrame.extent.x = 2;
//	WinEraseRectangle(&rFrame,0);
//	//clean up bottom corner
//	rFrame.topLeft.x = 154;
//	rFrame.topLeft.y = 10;
//	rFrame.extent.x = 2;
//	rFrame.extent.y = 2;
//	WinEraseRectangle(&rFrame,0);

	FrmShowObject(frmP,FrmGetObjectIndex(frmP,DataEntryPhaseLabel));

	if(colorEnabled)
		WinPopDrawState();
		
//	WinDrawInvertedChars( ,1,156-12,0);
//
//	//draw circle around i
//	rFrame.topLeft.x = 156-12;
//	rFrame.topLeft.y = 0;
//	rFrame.extent.x = 10;  //-14
//	rFrame.extent.y = 10;
//	WinDrawRectangleFrame(roundFrame,&rFrame);	
	
//	//main window
//	rFrame.topLeft.x = 156-CORNERBOX;
//	rFrame.topLeft.y = 0;
//	rFrame.extent.x = CORNERBOX;
//	rFrame.extent.y = 12;
//	WinEraseRectangle(&rFrame,3);
//	//clean up left corners
//	rFrame.extent.x = 2;
//	WinEraseRectangle(&rFrame,0);
//	//clean up bottom corner
//	rFrame.topLeft.x = 154;
//	rFrame.topLeft.y = 10;
//	rFrame.extent.x = 2;
//	rFrame.extent.y = 2;
	//main window
	rFrame.topLeft.x = 0;
	rFrame.topLeft.y = 0;
	rFrame.extent.x = CORNERBOX;
	rFrame.extent.y = 12;
	WinEraseRectangle(&rFrame,3);
	//clean up right corners
	rFrame.topLeft.x = CORNERBOX-2;
	rFrame.extent.x = 2;
	WinEraseRectangle(&rFrame,0);
	//clean up bottom corner
	rFrame.topLeft.x = 0;
	rFrame.topLeft.y = 10;
	rFrame.extent.x = 2;
	rFrame.extent.y = 2;
	WinEraseRectangle(&rFrame,0);
	FrmShowObject(frmP,FrmGetObjectIndex(frmP,DataEntryCycleDayLabel));
	FrmShowObject(frmP,FrmGetObjectIndex(frmP,DataEntryDayLabel));
                      
	return;
}

static void SECT3 NoteFormClose(FormType* frmP)
{
	MemHandle textH;
	MemHandle oldtextH;
	Char* pSource;
	FieldType *fldP;
	NFPDBRecord* nilP=0;
	UInt16 offset;
	UInt16 objectIndex;

	//Clean up Note Form
	//Process note field
	fldP = (FieldType*)FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, NoteNoteField));
	
	// Was the note string modified by the user.
	if (FldDirty (fldP))
		{
		// Release any free space in the note field.
		FldCompactText (fldP);

		// Mark the record dirty.	
		//DirtyRecord (CurrentRecord);
		}

	// Clear the handle value in the field, otherwise the handle
	// will be freed when the form is disposed of,  this call also unlocks
	// the handle that contains the note string.
	textH = FldGetTextHandle (fldP);
	FldSetTextHandle (fldP, 0);
	
	//Set up DataEntry Form
	frmP = FrmGetActiveForm();
	objectIndex = FrmGetObjectIndex(frmP,DataEntryNoteField);
	FrmHideObject(frmP,objectIndex);
	
	GetObjectPtr(fldP,DataEntryNoteField);
	
	//I shouldn't need to get this because it should already be blank
	oldtextH = FldGetTextHandle(fldP);

	if(textH)
	{
		pSource = (Char*)MemHandleLock(textH);
		if(s_isOldRecord)
		{
			offset = &(nilP->note) - (Char*)nilP;
			pSource += offset;
		}
		else
		{
			offset = 0;	
		}
			
		FldSetText (fldP, textH, offset, StrLen(pSource)+1);
		
		MemHandleUnlock(textH);
	}
	else //be sure DataEntry is also set to NULL in case it was deleted as note
	{
		FldSetText (fldP, textH, 0, 0);
	}
	
	//this should never be needed because the old text handle should be blank
	if (oldtextH != NULL)
			MemHandleFree(oldtextH);
				
	//This, along with FrmHideObject above, causes the text to be redrawn
	FrmShowObject(frmP,objectIndex);
	
	//recolor phase indicator
	DrawTop(frmP);
}


// FUNCTION: DataEntryFormInit
//
// DESCRIPTION: This routine initializes the DataEntryForm form.
//
// PARAMETERS:
//
// frm
//     pointer to the DataEntryForm form.
static void SECT3 DataEntryFormInit(FormType * frmP)
{
    Char* pString;
    ListType* lstP;
    Boolean dataExists;
    Boolean beyondMaxIndex;
    Int16 labelLength;
    MemHandle tempH;
    Int16 topList;
    NumberFormatType numFormat;
    UInt16 objectIndex;
    ControlType *ctlP;
    RectangleType bounds;
	//Fill in current record information into form
		
	//Draw in date for title
    pString = (Char *)FrmGetTitle(frmP);
    DateToDOWDMFormat(s_currentDate.month,s_currentDate.day,
    	s_currentDate.year + 1904,(DateFormatType)PrefGetPreference(prefLongDateFormat),pString);//dfMDYLongWithComma
    FrmSetTitle(frmP,pString);	
    
   	numFormat = (NumberFormatType)PrefGetPreference(prefNumberFormat);

    //Put in correct temp list depending on temp units
    GetObjectPtr(lstP,DataEntryTemperatureList);
    if(g_prefs.tempUnits==1)
    {
   		pString = (Char*)MemHandleLock (DmGetResource ('tSTL', TemperaturesStringList));
		labelLength = numberOfFahrenheitTemps + 1;//plus 1 for ----;
	}
	else //presumable g_prefs.tempUnits==2
	{
		if(numFormat==nfPeriodComma || numFormat==nfSpaceComma || numFormat==nfApostropheComma)
	   		pString = (Char*)MemHandleLock (DmGetResource ('tSTL', CelsiusTemperaturesCommStringList));
		else
	   		pString = (Char*)MemHandleLock (DmGetResource ('tSTL', CelsiusTemperaturesStringList));

		labelLength = numberOfCelsiusTemps + 1;//plus 1 for ----;
	}
	
	pString += 3;//gets past header to packed string
	tempH = SysFormPointerArrayToStrings(pString,labelLength);//returns unlocked handle
	tempTextList = (Char**)MemHandleLock(tempH);
	pString -= 3;
	MemPtrUnlock(pString);
	
    LstSetListChoices(lstP,tempTextList,labelLength);
    LstSetHeight (lstP, 14); 
    
    //set top of list so that last selection is in the middle
	//note: this does not affect which item is selected
	topList = temp_prefs.lastTemp - 6;
	if (topList<0)
		topList = 0;
	GetObjectPtr(lstP,DataEntryTemperatureList);
	LstSetTopItem (lstP,topList);
	//note: if temp data exists, this will have to overwritten, do that in SetFormFromRecord
    
	//hide unused objects if Marquette model
	if(g_prefs.fluidIndicator == 1)
	{
		//hide Fluid Color		
		objectIndex = FrmGetObjectIndex(frmP,DataEntryFluidColorLabel);
		FrmHideObject(frmP,objectIndex);

		objectIndex = FrmGetObjectIndex(frmP,DataEntryFluidColorPopTrigger);
		FrmHideObject(frmP,objectIndex);	

		//change label text
		FrmCopyLabel(frmP,DataEntrySensationLabel,"Marquette:");
		
	    //change list in Sensation to MRating
		if(g_prefs.presets != 3)
		{
	    	GetObjectPtr(lstP,DataEntryFluidSensationList);
	 		pString = (Char*)MemHandleLock (DmGetResource ('tSTL', MRatingStringList));
			labelLength = 11 + 1;//plus 1 for ----;	
			pString += 3;//gets past header to packed string
			tempH = SysFormPointerArrayToStrings(pString,labelLength);//returns unlocked handle
			tempMRatingList = (Char**)MemHandleLock(tempH);
			pString -= 3;
			MemPtrUnlock(pString);
		    LstSetListChoices(lstP,tempMRatingList,labelLength);
			objectIndex = FrmGetObjectIndex(frmP,DataEntryFluidSensationList);
			bounds.topLeft.x=64;
			bounds.topLeft.y=33;
			bounds.extent.x=120;
			bounds.extent.y=77;
		    FrmSetObjectBounds(frmP,objectIndex,&bounds);
		    LstSetHeight (lstP, 12);
	    }
		else //g_prefs.presets == 3 //change Sensation for MM II
		{
		    //change list in Sensation to just the first three
	    	GetObjectPtr(lstP,DataEntryFluidSensationList);
			pString = (Char*)MemHandleLock (DmGetResource ('tSTL', MRatingStringList));
			labelLength = 3 + 1;//plus 1 for ----;  only set it to first three
			pString += 3;//gets past header to packed string
			tempH = SysFormPointerArrayToStrings(pString,labelLength);//returns unlocked handle
			tempMRatingList = (Char**)MemHandleLock(tempH);
			pString -= 3;
			MemPtrUnlock(pString);
		    LstSetListChoices(lstP,tempMRatingList,labelLength);
			objectIndex = FrmGetObjectIndex(frmP,DataEntryFluidSensationList);
		    LstSetHeight (lstP, labelLength);
	    }	    
	    
	    
	    //change position of PopTrigger and List
	    objectIndex = FrmGetObjectIndex(frmP,DataEntryFluidSensationPopTrigger);
		FrmSetObjectPosition (frmP,objectIndex,10,46);
		
	    objectIndex = FrmGetObjectIndex(frmP,DataEntryFluidSensationList);
		FrmSetObjectPosition (frmP,objectIndex,23,46);
	}
	else if(g_prefs.fluidIndicator == 2) //display Couple to Couple League
	{
		//Change list in Sensation
	    	GetObjectPtr(lstP,DataEntryFluidSensationList);
	 		pString = (Char*)MemHandleLock (DmGetResource ('tSTL', ctcSensationStringList));
			labelLength = 8 + 1;//plus 1 for ----;	
			pString += 3;//gets past header to packed string
			tempH = SysFormPointerArrayToStrings(pString,labelLength);//returns unlocked handle
			tempSensationList = (Char**)MemHandleLock(tempH);
			pString -= 3;
			MemPtrUnlock(pString);
		    LstSetListChoices(lstP,tempSensationList,labelLength);

		//Change list in Fluid Color
	    	GetObjectPtr(lstP,DataEntryFluidColorList);
	 		pString = (Char*)MemHandleLock (DmGetResource ('tSTL', ctcFluidColorStringList));
			labelLength = 4 + 1;//plus 1 for ----;	
			pString += 3;//gets past header to packed string
			tempH = SysFormPointerArrayToStrings(pString,labelLength);//returns unlocked handle
			tempFluidColorList = (Char**)MemHandleLock(tempH);
			pString -= 3;
			MemPtrUnlock(pString);
		    LstSetListChoices(lstP,tempFluidColorList ,labelLength);
	}

	//change from LH Test to Clearplan from Marquette model
	if(g_prefs.hormoneTest == 1)
	{
   		//change text in LHTest for use with Clearplan
		GetObjectPtr(ctlP,DataEntryLHTestNegativePushButton);
		CtlSetLabel (ctlP,"L");
		GetObjectPtr(ctlP,DataEntryLHTestPositivePushButton);
		CtlSetLabel (ctlP,"P");
		//move P over
		objectIndex = FrmGetObjectIndex(frmP,DataEntryLHTestPositivePushButton);
		FrmSetObjectPosition (frmP,objectIndex,85,81);
		objectIndex = FrmGetObjectIndex(frmP,DataEntryClearplanHPushButton);
		FrmShowObject(frmP,objectIndex);
		
		//change label text
		FrmCopyLabel(frmP,DataEntryLHTestLabel,"ClrPlan:");
	}
	
	//hide protected sex
	if(g_prefs.presets == 2 || g_prefs.presets == 3 || g_prefs.presets == 4)
	{
		objectIndex = FrmGetObjectIndex(frmP,DataEntrySexProtectedPushButton);
		FrmHideObject(frmP,objectIndex);
		
		//and change "Sex:" to "SI:" for sexual intercourse		
		objectIndex = FrmGetObjectIndex(frmP,DataEntrySexLabel);
		FrmHideObject(frmP,objectIndex);
		objectIndex = FrmGetObjectIndex(frmP,DataEntryAltSexLabel);
		FrmShowObject(frmP,objectIndex);
		
		//and hide cervix position
		objectIndex = FrmGetObjectIndex(frmP,DataEntryCervixPositionLabel);
		FrmHideObject(frmP,objectIndex);
		objectIndex = FrmGetObjectIndex(frmP,DataEntryCervixNPushButton);
		FrmHideObject(frmP,objectIndex);
		objectIndex = FrmGetObjectIndex(frmP,DataEntryCervix1PushButton);
		FrmHideObject(frmP,objectIndex);
		objectIndex = FrmGetObjectIndex(frmP,DataEntryCervix2PushButton);
		FrmHideObject(frmP,objectIndex);
		objectIndex = FrmGetObjectIndex(frmP,DataEntryCervix3PushButton);
		FrmHideObject(frmP,objectIndex);
	}

	//hide protected sex for Couple to Couple League Presets
	if(g_prefs.presets == 5|| g_prefs.presets == 6 || g_prefs.presets == 7)
	{
		objectIndex = FrmGetObjectIndex(frmP,DataEntrySexProtectedPushButton);
		FrmHideObject(frmP,objectIndex);
	}

	//hide temp for MM and MM II
	if(g_prefs.presets == 2 || g_prefs.presets == 3)
	{
		//hide Temp		
		objectIndex = FrmGetObjectIndex(frmP,DataEntryTemperatureLabel);
		FrmHideObject(frmP,objectIndex);

		objectIndex = FrmGetObjectIndex(frmP,DataEntryTemperaturePopTrigger);
		FrmHideObject(frmP,objectIndex);
		
		//hide Time		
		objectIndex = FrmGetObjectIndex(frmP,DataEntryTimeLabel);
		FrmHideObject(frmP,objectIndex);

		objectIndex = FrmGetObjectIndex(frmP,DataEntryTimePopTrigger);
		FrmHideObject(frmP,objectIndex);
	}
    

    //check to see if there is data for s_currentDate and assign currentRecordNumber
	//use previous record number so I'm not always searching from the beginning
	//    s_currentRecordNumber = 0; //moved this line to start up, continue to just use the old one for each call
    s_isOldRecord = FindRecordFromDate(s_currentDate, &s_currentRecordNumber, &dataExists, &beyondMaxIndex);
        
	SetFormFromRecord(s_currentRecordNumber, s_isOldRecord, dataExists, beyondMaxIndex);

	return;
}

// FUNCTION: DataEntryFormCleanUp
//
// DESCRIPTION:  Called when closing the DataEntry form.  At this point, cleans up
//				memory allocated for the temperature selector list
//
// PARAMETERS:
//
// RETURNED:
static void SECT3 DataEntryFormCleanUp()
{

	MemPtrFree (tempTextList);
	if(g_prefs.fluidIndicator == 1)
		MemPtrFree (tempMRatingList);
	if(g_prefs.fluidIndicator == 2)
	{
		MemPtrFree (tempSensationList);
		MemPtrFree (tempFluidColorList);
	}
	
	return;
}


// FUNCTION: DataEntryFormHandleEvent
//
// DESCRIPTION:
//
// This routine is the event handler for the "DataEntryForm" of this 
// application.
//
// PARAMETERS:
//
// eventP
//     a pointer to an EventType structure
//
// RETURNED:
//     true if the event was handled and should not be passed to
//     FrmHandleEvent

Boolean DataEntryFormHandleEvent(EventType * eventP)
{
    Boolean handled = false;
    FormType * frmP;
    FieldType *fldP;
//    Boolean added;
    UInt8 changeStatus;
    DateType today;
    UInt16 button;
    MemHandle oldTextH;
      
    switch (eventP->eType) 
        {
//        case menuEvent:
//            return MainFormDoCommand(eventP->data.menu.itemID);

        case frmOpenEvent:
            frmP = FrmGetActiveForm();
            DataEntryFormInit(frmP);
            cleanClose = false;
            FrmDrawForm(frmP);
			DrawTop(frmP);
            handled = true;
            break;
            
        case frmUpdateEvent:
            // To do any custom drawing here, first call 
            // FrmDrawForm(), then do your drawing, and 
            // then set handled to true.
			frmP = FrmGetActiveForm();
            FrmDrawForm(frmP);
			DrawTop(frmP);
            handled = true;
            break;

        case ctlSelectEvent:
        	switch (eventP->data.ctlSelect.controlID)
        	{
        		case DataEntryOKButton:
        			//manages data saving and deleting
        			changeStatus = OKDataFunction();
//					if(changeStatus != 2)
//	        			CalcPhase(changeStatus);
         			cleanClose = true;

        			//returns to main form
        	        GoToView();
        			handled = true;        		
        			break;
        			
        		case DataEntryClearButton:
					//confirm alert
					if(!FrmAlert(RecordDeleteAlert))
					{
		      			//if a database entry exists, deletes it
						GetObjectPtr(fldP,DataEntryNoteField);
						//hide note so I don't have to worry about redraw issues
						frmP = FrmGetActiveForm();
						FrmHideObject(frmP,FrmGetObjectIndex(frmP,DataEntryNoteField));
						oldTextH = FldGetTextHandle (fldP);
						FldSetTextHandle (fldP, 0);
						if(!s_isOldRecord && oldTextH)
							MemHandleFree(oldTextH);	      			
	       			    if(s_isOldRecord)
	       			    {
	       			    	DataDelete(s_currentRecordNumber);
							//only need to recalculate Phase if there was a record
	    	   			    CalcPhase( false );
	    	   			}
        	 			cleanClose = true;
        			
	        			//returns to main form
    	    	        GoToView();
    	    	    }			        	
        			handled = true;
        			break;
        			
        		case DataEntryCancelButton:
        			//ignores any data entry and returns to main form
        			
        			//although first be clear note association
        			//NB: Cancel does not undo changes to note, since the record is edited directly to save stack space
       				//Process note field
					GetObjectPtr(fldP,DataEntryNoteField);
					if (FldDirty (fldP))
					{
						FldCompactText (fldP);
						//hide note so I don't have to worry about redraw issues
						frmP = FrmGetActiveForm();
						FrmHideObject(frmP,FrmGetObjectIndex(frmP,DataEntryNoteField));
						// Mark the record dirty.	
						//DirtyRecord (CurrentRecord);
					}
					oldTextH = FldGetTextHandle (fldP); //need this so that modified text in a record that does not exist can be freed
					FldSetTextHandle (fldP, 0);
					if(!s_isOldRecord && oldTextH)  //need this so that modified text in a record that does not exist can be freed
							MemHandleFree(oldTextH);
        				
        			cleanClose = true;
        			
        	        GoToView();        			
        			handled = true;
        			break;
        			
        		case DataEntryFullNoteButton:
       		        frmP = FrmInitForm (NoteForm);
		        	NoteFormInit (frmP);
		    	    button = FrmDoDialog (frmP);
		    	    NoteFormClose (frmP);                    
			        FrmDeleteForm (frmP);
        			
        			handled = true;
        			break;
        	
        		default:
        			break;
			
			}
			break;
			
		case keyDownEvent:
        	//may need to add TxtGlueCharlsVirtual to restrict this
        	//to virtual characters only
        	switch (eventP->data.keyDown.chr)
        	{
        		case vchrPageUp:
        			//why do I adjust the date back one and then forward one?
        			//ah, so I can use DateTypeCmp, which may be faster than 
        			//converting to days and then comparing
        			DateAdjust(&s_currentDate,-1);
        			DateSecondsToDate(TimGetSeconds(), &today);	//get today	
        			if( DateTypeCmp(today,s_currentDate) >= 0 )
        			{
        				DateAdjust(&s_currentDate,+1);
        				changeStatus = OKDataFunction();
//						if(changeStatus != 2)
//		        			CalcPhase(changeStatus);
	        			DateAdjust(&s_currentDate,-1);
			       		cleanClose = true; 
	       				FrmGotoForm(DataEntryForm);
       				}
       				else
       				{
				        DateAdjust(&s_currentDate,+1);
				        FrmAlert(NoFutureAlert);      				
       				}       		
        		    handled = true;
        			break;        		
        		
        		case vchrPageDown:
        			DateAdjust(&s_currentDate,+1);
        			DateSecondsToDate(TimGetSeconds(), &today);	//get today	
        			if( DateTypeCmp(today,s_currentDate) >= 0 )
        			{
        				DateAdjust(&s_currentDate,-1);
        				changeStatus = OKDataFunction();
//        				if(changeStatus != 2)
//	        				CalcPhase(changeStatus);
        				DateAdjust(&s_currentDate,+1);
			       		cleanClose = true;
       					FrmGotoForm(DataEntryForm);
       				}
       				else
       				{
				        DateAdjust(&s_currentDate,-1);
				        FrmAlert(NoFutureAlert);     				
       				}              		
        		    handled = true;
        			break;

        		default:
        			break;
        	}
        	break;
        	       	
        case frmCloseEvent:
			if(!cleanClose)
			{
				changeStatus = OKDataFunction();
//				if(changeStatus != 2)
//					CalcPhase(changeStatus);
			}
			DataEntryFormCleanUp();	
        	break;

        default:
            break;
        }
    
    return handled;
}

//  FUNCTION: OKDataFunction
//
//  DESCRIPTION: this function is run when OK, or an equivalen, is selected in the Data Entry form.
//				It manages whether or not to delete the day, add a new record, or update an existing record.
//
//  PARAMETERS:
//
//  RETURNS: 1 if record added or modified, 0 if record deleted, 2 if no change; information needed to make CalcPhase run smoothly
UInt8 OKDataFunction( void )
{
    Boolean isBlank;
    FieldType *fldP;
    MemHandle oldTextH;
    UInt8 changeStatus;
    FormType * frmP;
    Err err;
    
	isBlank = IsDataEntryNull();
	
	//check to see if entry exists and delete it
	if(isBlank && s_isOldRecord)
	{
		GetObjectPtr(fldP,DataEntryNoteField);
		//hide note so I don't have to worry about redraw issues
		frmP = FrmGetActiveForm();
		FrmHideObject(frmP,FrmGetObjectIndex(frmP,DataEntryNoteField));
		oldTextH = FldGetTextHandle (fldP);
		FldSetTextHandle (fldP, 0);			
		DataDelete(s_currentRecordNumber);
		changeStatus = 0; //record was deleted
		CalcPhase(changeStatus);
	}

	//update or write to database
	else if(!isBlank && !s_isOldRecord)
	{
		NFPDataType nfpData;
		SaveFormToStructure(&nfpData);

		UInt16 tempIndex;
		err=DataNew(&nfpData,&tempIndex);
		
		if(err == errNone)
		{
			GetObjectPtr(fldP,DataEntryNoteField);
			//hide note so I don't have to worry about redraw issues
			frmP = FrmGetActiveForm();
			FrmHideObject(frmP,FrmGetObjectIndex(frmP,DataEntryNoteField));
			oldTextH = FldGetTextHandle (fldP);
	//		FldSetTextHandle (fldP, 0);	

			changeStatus = 1; //record was added
			CalcPhase(changeStatus);
		}
		else
		{
			FrmAlert(OutOfMemoryAlert);
			ForceQuit(); 
			changeStatus = 2; //to keep compiler happy	
		}
	}
	else if(!isBlank && s_isOldRecord)
	{
		//changeStatus will depend on if DataSaveOld finds any difference
		if(DataSaveOld(&s_currentRecordNumber))
		{
			changeStatus = 1; //record modified
			CalcPhase(changeStatus);
		}
		else
			changeStatus = 2; //no change
			//this is also a problem when note is dirty, but blank
	}
	else if(isBlank && !s_isOldRecord)
	{
		//don't need to hide note, because not calling CalcPhase
		changeStatus = 2; //no change, never was a record, still is no record
	}
	
	return changeStatus; //used to be !isBlank
}

//  FUNCTION: IsDataEntryNull
//
//  DESCRIPTION: checks to see if data entry form is blank
//
//  PARAMETERS:
//
//  RETURNS: true if there is no data in form, false if there is data that needs to be saved
Boolean IsDataEntryNull(void)
{
	Boolean isBlank = true;
	UInt16 id;
	UInt8 value;
	ControlType *ctlP;
	ListType *lstP;
	FieldType *fldP;
	FormType * frmP;
	UInt16 temp;

	frmP = FrmGetActiveForm();

	
	GetObjectPtr(lstP,DataEntryTemperatureList);
	value=LstGetSelection(lstP);
	if(value!=0) isBlank = false;

	GetObjectPtr(lstP,DataEntryFluidSensationList);
	value=LstGetSelection(lstP);
	if(!(value == 0 || (g_prefs.fluidIndicator==1 && value==7) ) )
		isBlank = false;

	GetObjectPtr(lstP,DataEntryFluidColorList);
	value=LstGetSelection(lstP);
	if(value!=0) isBlank = false;
	
	GetObjectPtr(lstP,DataEntryTimeList);
	value=LstGetSelection(lstP);
	if(value!=4) isBlank = false;

	//Cervix Group 3
	id=FrmGetObjectId (frmP, FrmGetControlGroupSelection(frmP,3));
	if(id!=DataEntryCervixNPushButton) isBlank = false;
	
	//LHTest Group 2
	temp = FrmGetControlGroupSelection(frmP,2);
	id=FrmGetObjectId (frmP, temp);
	if(id!=DataEntryLHTestNPushButton) isBlank = false;

	//Sex Group 1
	temp = FrmGetControlGroupSelection(frmP,1);;
	id=FrmGetObjectId (frmP, temp);
	if(id!=DataEntrySexNPushButton) isBlank = false;
	
	GetObjectPtr(ctlP,DataEntryOtherCheckbox);
	value=CtlGetValue(ctlP);
	if(value!=0) isBlank = false;
	
	GetObjectPtr(ctlP,DataEntryAlcoholCheckbox);
	value=CtlGetValue(ctlP);
	if(value!=0) isBlank = false;


	GetObjectPtr(fldP,DataEntryNoteField);
	value=FldGetTextLength(fldP);
	if(value!=0) isBlank = false;		

	return isBlank;
}

//  FUNCTION: SaveFormToStructure
//
//  DESCRIPTION: used to save new day records to a structure that is then
//				written to the database.  Note: this is not used for old records.
//				they are modified in place.
//
//  PARAMETERS:  pass it a pointer to the day data structure into which it fills the data
//
//  RETURNS: 
void SaveFormToStructure(NFPDataType* newStruct)
{
	UInt16 id;
	UInt8 value;
	Char* pString;
	ControlType *ctlP;
	ListType *lstP;
	FieldType *fldP;
	FormType * frmP;
	frmP = FrmGetActiveForm();
	
	newStruct->recDate=s_currentDate;
	
	GetObjectPtr(lstP,DataEntryTemperatureList);
	value=LstGetSelection(lstP);
	newStruct->basalTemp=value;
	if(value) //if not blank
		temp_prefs.lastTemp = value; //store last temp for use in auto scroll down

	GetObjectPtr(lstP,DataEntryFluidSensationList);
	value=LstGetSelection(lstP);
	if(g_prefs.fluidIndicator==0 || g_prefs.fluidIndicator==2)
	{		
		if(value==7)
			value=0;
		if(value==8)
			newStruct->fluidSensation=50;
		else
			newStruct->fluidSensation=value;
	}
	else //assume fluidIndicator==1; Marquette rating method
	{
		//relationship between classic double indicator for fluid, Marquette Rating, list numbers and stored value in recod
		//(classic, Marquette, stored, list)
		//Heavy Flow  ( , ,1,1)
		//Medium Flow ( , ,2,2)
		//Spotting    ( , ,3,3)
		//Dry         (4,1,4,4)
		//		  (4,2,12,5)
		//            (4,3,13,6)
		//Moist       (5,4,5,7)
		//            (5,5,15,8)
		//            (5,6,16,9)
		//Lubricated  (6,7,6,10)
		//            (6,8,18,11)
		//Begin
		// Postpartum ( , ,50,8) in classic list it is number 8
	
		switch(value)
		{
			case 5: //2: not dry
				newStruct->fluidSensation=12;
				break;

			case 6: //3: moist
				newStruct->fluidSensation=13;  
				break;
				
			case 7: //4:
				newStruct->fluidSensation=5; //to match Moist
				break;

			case 8: //5:
				newStruct->fluidSensation=15;
				break;
				
			case 9: //6:
				newStruct->fluidSensation=16;
				break;

			case 10: //7:
				newStruct->fluidSensation=6; //to match Lubricated
				break;

			case 11: //8:
				newStruct->fluidSensation=18;
				break;

			default: //list numbers 1-4 match up
				newStruct->fluidSensation=value;
				break;
		}
	
	}

	GetObjectPtr(lstP,DataEntryFluidColorList);
	value=LstGetSelection(lstP);
	newStruct->fluidColor=value;
	
	GetObjectPtr(lstP,DataEntryTimeList);
	value=LstGetSelection(lstP);
	newStruct->time=value;

	//Cervix Group 3
	id=FrmGetObjectId (frmP, FrmGetControlGroupSelection(frmP,3));
	value = id-DataEntryCervixNPushButton;
	newStruct->cervix=value;	
	
	//LHTest Group 2
	id=FrmGetObjectId (frmP, FrmGetControlGroupSelection(frmP,2));
	value=id-DataEntryLHTestNPushButton;
	newStruct->LHTest=value;
	//will store - = 1, + = 2; L = 1, H = 3, P = 2

	//Sex Group 1
	id=FrmGetObjectId (frmP, FrmGetControlGroupSelection(frmP,1));
	value=id-DataEntrySexNPushButton;
	newStruct->sex=value;
	
	GetObjectPtr(ctlP,DataEntryOtherCheckbox);
	value=CtlGetValue(ctlP);
	newStruct->other=value;
	
	GetObjectPtr(ctlP,DataEntryAlcoholCheckbox);
	value=CtlGetValue(ctlP);
	newStruct->alcohol=value;

	GetObjectPtr(fldP,DataEntryNoteField);
	pString=FldGetTextPtr(fldP);
	newStruct->note=pString;
	
	return;
}



void SetFormFromRecord(UInt16 index, Boolean isOldRecord, Boolean dataExists, Boolean beyondMaxIndex)
{
	UInt16 id;
	Int16 cycleDay;
	UInt16 offset;
	Err err;
	NFPDBRecord* item;
	ControlType *ctlP;
	ListType *lstP;
	MemHandle textH;
	MemHandle oldtextH;
	Char* pString;
	Char* pSource;
	FieldType *fldP;
	FormType * frmP;
	UInt16 objectIndex;
//	UInt8 phase = 0;
	Char *phaseLabels;
 	Char *label;
 	UInt8 value;
 	Int16 topList;
	frmP = FrmGetActiveForm();
//	RectangleType r;
	
	phase = 0;
	
	if(isOldRecord)
	{
		item = DataQueryData(index);
	
		GetObjectPtr(lstP,DataEntryTemperatureList);
		//in case data was entered as F, and then switched to C
		//make sure it doesn't try to select beyond the length of the list
		value = item->basalTemp;
		if(g_prefs.tempUnits==2 && value > numberOfCelsiusTemps) value = numberOfCelsiusTemps;
		LstSetSelection(lstP,value);
		GetObjectPtr(ctlP,DataEntryTemperaturePopTrigger);
		CtlSetLabel (ctlP,LstGetSelectionText(lstP,value));
		//do this to over-ride setting top to previous selected temp in Init
		topList = value - 1;
		if(topList < 0)
			topList = 0;	
		LstSetTopItem (lstP,topList); 
		
		GetObjectPtr(lstP,DataEntryFluidSensationList);
		GetObjectPtr(ctlP,DataEntryFluidSensationPopTrigger);	
		if(g_prefs.fluidIndicator==0 || g_prefs.fluidIndicator==2) //double indicator
		{
			switch(item->fluidSensation)
			{
			case 12:
			case 13:
			case 4:
				LstSetSelection(lstP,4);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,4));
				break;

			case 15:
			case 16:
			case 5:
				LstSetSelection(lstP,5);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,5));		
				break;

			case 18:
			case 6:
				LstSetSelection(lstP,6);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,6));
				break;

			case 50:
				LstSetSelection(lstP,8);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,8));
				break;			

			default:
				LstSetSelection(lstP,item->fluidSensation);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,item->fluidSensation));		
				break;
			}
		}
		else if(g_prefs.presets == 3) //MM II therefore have to map all non menstrual flows to ----
		{
			switch(item->fluidSensation)
			{
			case 12:
			case 13:
			case 4:
			case 15:
			case 16:
			case 5:
			case 18:
			case 6:
				LstSetSelection(lstP,0);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,0));
				break;

			default:
				LstSetSelection(lstP,item->fluidSensation);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,item->fluidSensation));		
				break;
			}		
		}
		else //assume fluidIndicator==1; Marquette rating method and not MM II
		{
			switch(item->fluidSensation)
			{
			case 12: //2: not dry
				LstSetSelection(lstP,5);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,5));
				break;

			case 13: //3:
				LstSetSelection(lstP,6);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,6));
				break;

			case 5: //4: moist
				LstSetSelection(lstP,7);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,7));
				break;
				
			case 15: //5:
				LstSetSelection(lstP,8);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,8));
				break;
				
			case 16: //6:
				LstSetSelection(lstP,9);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,9));
				break;

			case 6: //7:
				LstSetSelection(lstP,10);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,10));
				break;

			case 18: //8:
				LstSetSelection(lstP,11);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,11));
				break;

			default: //list numbers 1-4 match up
				LstSetSelection(lstP,item->fluidSensation);
				CtlSetLabel (ctlP,LstGetSelectionText(lstP,item->fluidSensation));
				break;
			}
		}

		GetObjectPtr(lstP,DataEntryFluidColorList);
		LstSetSelection(lstP,item->fluidColor);
		GetObjectPtr(ctlP,DataEntryFluidColorPopTrigger);
		CtlSetLabel (ctlP,LstGetSelectionText(lstP,item->fluidColor));	
	
	
		GetObjectPtr(lstP,DataEntryTimeList);
		LstSetSelection(lstP,item->time);
		GetObjectPtr(ctlP,DataEntryTimePopTrigger);
		CtlSetLabel (ctlP,LstGetSelectionText(lstP,item->time));
	
	   	//Cervix Group 3
		id = item->cervix + DataEntryCervixNPushButton;
		FrmSetControlGroupSelection(frmP,3,id);
	
		//LHTest Group 2
		id = item->LHTest + DataEntryLHTestNPushButton;
		if(g_prefs.hormoneTest == 0 && id == DataEntryClearplanHPushButton) //if LH test, map H(3) to -
			FrmSetControlGroupSelection(frmP,2,DataEntryLHTestNegativePushButton);
		else
			FrmSetControlGroupSelection(frmP,2,id);

		//Sex Group 1
		id = item->sex + DataEntrySexNPushButton;
		FrmSetControlGroupSelection(frmP,1,id);

	    GetObjectPtr(ctlP,DataEntryOtherCheckbox);
	    CtlSetValue(ctlP,item->other);
    
	    GetObjectPtr(ctlP,DataEntryAlcoholCheckbox);
	    CtlSetValue(ctlP,item->alcohol);
    
		GetObjectPtr(fldP,DataEntryNoteField);
	
	    //I shouldn't need to get this because it should already be blank
	    oldtextH = FldGetTextHandle(fldP);

		pSource = &(item->note);
		offset = pSource - (Char*)item;
		textH = LockedHandle();
			
		FldSetText (fldP, textH, offset, StrLen(pSource)+1);
	
		//this should never be needed because the old text handle should be blank
		if (oldtextH != NULL)
				MemHandleFree(oldtextH);

/*			//I shouldn't need to get this because it should already be blank
		    oldtextH = FldGetTextHandle(fldP);

			// make new chunk and lock it down
			pSource = &(item->note);
			textH = MemHandleNew (StrLen(pSource) + 1);
			pString = (Char*)MemHandleLock(textH);
	
			//copy string into new pointer
			if(item->note) StrCopy(pString,pSource);

			// unlock the chunk and set to field
			err = MemHandleUnlock(textH);
			FldSetTextHandle(fldP,textH);
	
			//this should never be needed because the old text handle should be blank
			if (oldtextH != NULL)
				MemHandleFree(oldtextH);
*/
	
		DataReleaseData();
	}
	
	else //not old record
	{
	    GetObjectPtr(ctlP,DataEntryCervixNPushButton);
	    CtlSetValue(ctlP,1);
	    GetObjectPtr(ctlP,DataEntrySexNPushButton);
	    CtlSetValue(ctlP,1);
	    GetObjectPtr(ctlP,DataEntryLHTestNPushButton);
	    CtlSetValue(ctlP,1);
	    
    	GetObjectPtr(lstP,DataEntryTimeList);
		LstSetSelection(lstP,4);
	}
	
	phase = DeterminePhase(s_currentDate, index, isOldRecord, dataExists, beyondMaxIndex, &cycleDay, true, NULL);
	
	//set phase on form
	objectIndex = FrmGetObjectIndex(frmP,DataEntryPhaseLabel);
	FrmHideObject(frmP,objectIndex);
	phaseLabels = (Char*)MemHandleLock (DmGetResource (strRsc, PhaseLabelsString));
	label = (Char*)MemPtrNew(StrLen(phaseLabels) + 1);
	switch(phase)
	{
		case 0:
			break;
		case 1:
			StrCopy(label,&phaseLabels[0]);
			break;
		case 2:
			StrCopy(label,&phaseLabels[1]);
			break;
		case 3:
			StrCopy(label,&phaseLabels[2]);
			break;
		case 4:
			StrCopy(label,&phaseLabels[3]);
			break;
		case 5:
			StrCopy(label,&phaseLabels[4]);
			break;
		case 6:
			StrCopy(label,&phaseLabels[5]);
			break;					
		default:
			break;
	}
	*(label+1)=NULL;
	MemPtrUnlock (phaseLabels);

	if(!phase)
		StrCopy(label,"");
	FrmCopyLabel(frmP,DataEntryPhaseLabel,label);
	err = MemPtrFree(label);

	//will show this object after I get color info
//	FrmShowObject(frmP,objectIndex);

	//set cycleDay on form
	objectIndex = FrmGetObjectIndex(frmP,DataEntryCycleDayLabel);
	FrmHideObject(frmP,objectIndex);
	pString = (Char*)MemPtrNew(maxStrIToALen);

	if(cycleDay != -1)
		StrIToA(pString,cycleDay);
	else
		StrCopy(pString,"");
		
	FrmCopyLabel(frmP,DataEntryCycleDayLabel,pString);
	err = MemPtrFree(pString);
	//will show after I draw box
//	FrmShowObject(frmP,objectIndex); 
	
	
	return;
}



//  FUNCTION: DateTypeCmp
//
//  DESCRIPTION: Compare two dates
//
//  PARAMETERS: 
//
//  RETURNS: 
//  CREATED: 1/20/95 
//  BY: Roger Flores
Int16 DateTypeCmp(DateType d1, DateType d2)
{
	Int16 result;
	
	result = d1.year - d2.year;
	if (result != 0)
		{
		if (DateToInt(d1) == 0xffff)
			return 1;
		if (DateToInt(d2) == 0xffff)
			return -1;
		return result;
		}
	
	result = d1.month - d2.month;
	if (result != 0)
		return result;
	
	result = d1.day - d2.day;

	return result;

}


//  FUNCTION: GoToView
//
//  DESCRIPTION: Go to last major form as stored in temporary preferences
//
//  PARAMETERS: void
//
//  RETURNS: void
void GoToView(void)
{
	if(temp_prefs.view == 1)
    	FrmGotoForm(MonthViewForm);
    else if(temp_prefs.view == 2)
       	FrmGotoForm(TempChartForm);
       	
	return;
}
