//    Copyright (C) 2003 Chad Brassil
//    This file is part of NFP.
//
//    NFP is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Foobar is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <PalmOS.h>
#include "NFP.h"
#include "NFPRsc.h"

void DB_WriteStringFromHandle(void *dest, UInt32 *off, MemHandle h, UInt32 len) SECT2;
void DB_WriteString(void *dest, UInt32 *off, Char const *str) SECT2;

/*
 * Maximum length of record supported by the Memo Pad application
 */
#define kMaxExport (4<<10)

/*
 * Database constants to access the built-in Memo Pad.
 */
#define kMemoType 'DATA'
#define kApplType 'appl'
#define kMemoCreator 'memo'


/*
 * Display a dialog to the user, offering a choice of separator character
 */

UInt16 ExportConfirm()
{
	FormPtr alert;
	ListType *lstP;
	UInt16  buttonHit;
	UInt16  separator;
	
	alert = FrmInitForm (ConfirmExportForm);

	lstP = (ListType *) FrmGetObjectPtr(alert, FrmGetObjectIndex (alert, ConfirmExportSeparatorList));
	LstSetSelection (lstP, g_prefs.exportSeparator);
	buttonHit = FrmDoDialog (alert);
	separator = LstGetSelection(lstP);

	FrmDeleteForm (alert);

	if (buttonHit != ConfirmExportOKButton)
		return 1; //export will be cancelled
	
	g_prefs.exportSeparator = separator;

	return 0;
}


static void SECT2 Export_Failure(void)
{
    FrmAlert(MemoDatabaseErrorAlert);
}


/*
 * LEN is the length of the string.  If you want the terminating NUL
 * included, then it must be included in the length.
 *
 * If the handle is NULL, then if the length is 1 a NUL is written and if the
 * length is zero nothing is written.  Anything else is invalid.
 */
void DB_WriteStringFromHandle(void *dest, UInt32 *off, MemHandle h, UInt32 len)
{
    if (h) {
	Char * p = (Char*)MemHandleLock(h);
	DmWrite(dest, *off, p, len);
	*off += len;
	MemHandleUnlock(h);
    } else {
        ErrNonFatalDisplayIf(len != 1 && len != 0,
                             "bogus length in " __FUNCTION__);
        if (len) {
            DmWrite(dest, *off, "", 1);
            (*off)++;
        }
    }
}


/*
 * Write a string into a database record, and update a position pointer.
 * The NUL is not included.
 */
void DB_WriteString(void *dest, UInt32 *off, Char const *str)
{
    UInt16 len;

    len = StrLen(str);
    DmWrite(dest, *off, str, len);
    *off += len;
}


/*
 * Write the exported form of an unpacked record into a pre-allocated
 * space in a memo record.  Returns the number of bytes written, which
 * will be the new length of the memo record.
 */
static UInt16 SECT2 Export_BuildText(	UInt16 cycleIndex, UInt16 itemStartIndex, UInt16 itemFinalIndex, void *memoRecord)
{
    UInt32	off = 0;
   	NFPDBRecord* item;
   	UInt16 itemIndex;
   	Char* pString;
	Char *pLabels;
	Char *pTempString;
	const Char *separator;
   	Err err;
	Int16 cycleDay;
	DateType curDate;
	UInt8 phase;
	UInt8 tempToGrab;
	Int16 fluidTrans;
	Int16 lengthOfpString = 25;
	Int16 tempAdjust;
	
	switch (g_prefs.exportSeparator)
	{
		case 1:
			separator = "\t";
			break;
		default:
			separator = ";";
	}

	//write memo title, for example "Cycle info: 02/20/03"
	DB_WriteString(memoRecord, &off, "Cycle info: ");
	item = DataQueryData(itemStartIndex);
	pString = (Char*)MemPtrNew(lengthOfpString);
	//pString = (Char*)MemPtrNew(maxStrIToALen);
	DateToAscii(item->recDate.month, item->recDate.day, item->recDate.year + 1904, (DateFormatType)PrefGetPreference(prefDateFormat), pString); //dfMDYWithSlashes
	DB_WriteString(memoRecord, &off, pString);
	DataReleaseData();

	
	//make maximum exported records 50 so as to stay within limits of Memo Pad
	if(itemFinalIndex-itemStartIndex > 50)
		itemFinalIndex = itemStartIndex + 50;
	
	for(itemIndex = itemStartIndex; itemIndex <= itemFinalIndex; itemIndex++)
	{
		item = DataQueryData(itemIndex);
	
		//write day of cycle
		curDate = item->recDate;
		DataReleaseData();
		phase = DeterminePhase(curDate, itemIndex, true, true, false, &cycleDay, true, NULL);
		StrIToA(pString,cycleDay);
		DB_WriteString(memoRecord, &off, "\n");
		DB_WriteString(memoRecord, &off, pString);
	
		//write date
		//always use DMY format to insure compatibility with Excel notebook
		DateToAscii(curDate.month, curDate.day, curDate.year + firstYear,dfDMYWithSlashes, pString);
		DB_WriteString(memoRecord, &off, separator);
		DB_WriteString(memoRecord, &off, pString);
	
		//write phase
		pLabels = (Char*)MemHandleLock (DmGetResource (strRsc, PhaseLabelsString));
		switch(phase)
		{
			case 0:
				break;
			case 1:
				StrCopy(pString,&pLabels[0]);
				break;
			case 2:
				StrCopy(pString,&pLabels[1]);
				break;
			case 3:
				StrCopy(pString,&pLabels[2]);
				break;
			case 4:
				StrCopy(pString,&pLabels[3]);
				break;
			case 5:
				StrCopy(pString,&pLabels[4]);
				break;
			case 6:
				StrCopy(pString,&pLabels[5]);
				break;					
			default:
				break;
		}
		*(pString+1)=NULL;
		MemPtrUnlock (pLabels);
		DB_WriteString(memoRecord, &off, separator);
		DB_WriteString(memoRecord, &off, pString);
	
		//write temperatures
		//first raw temp
		item = DataQueryData(itemIndex); //pick up record again.
		//grab appropriate temp labels; Always use "." for decimal to insure compatability with Excel spreadsheet
		pTempString = (Char*)MemPtrNew(5);
		tempToGrab = item->basalTemp;
		if(g_prefs.tempUnits==1)
			pTempString = SysStringByIndex(TemperaturesStringList,tempToGrab,pTempString,5);
		else //presumable g_prefs.tempUnits==2
		{
			if (tempToGrab > numberOfCelsiusTemps)
				tempToGrab = numberOfCelsiusTemps;
			pTempString = SysStringByIndex(CelsiusTemperaturesStringList,tempToGrab,pTempString,5);
		}
		DB_WriteString(memoRecord, &off, separator);
		DB_WriteString(memoRecord, &off, pTempString);
		
		//second time adjusted temp if different than raw temp
		tempToGrab = item->basalTemp;
		
		if(g_prefs.timeAdjustMode == 0 || g_prefs.timeAdjustMode == 1)
		{
			//time adjustments of 0.1 F or 0.05 C per half hour (earlier versions were 0.1 C per hour
			tempAdjust = item->time - 4; //so that entry of 4 is 0 adjustment
			if(g_prefs.timeAdjustMode == 1 /*&& g_prefs.tempUnits==1*/) //previousl on half rate in F, now in both
				//change time adjustments to 1/10 F per hour
				tempAdjust = tempAdjust/2;		
			//if(g_prefs.tempUnits==2) //if in C, then 1/10 C per hour
			//		tempAdjust = tempAdjust/2;
		}
		else
			tempAdjust = 0;
		
		if(tempAdjust)
		{
			tempToGrab = tempToGrab - tempAdjust;
			if(g_prefs.tempUnits==1)
				pTempString = SysStringByIndex(TemperaturesStringList,tempToGrab,pTempString,5);
			else //presumable g_prefs.tempUnits==2
			{
				if (tempToGrab > numberOfCelsiusTemps)
					tempToGrab = numberOfCelsiusTemps;
				pTempString = SysStringByIndex(CelsiusTemperaturesStringList,tempToGrab,pTempString,5);
			}
		}
		DB_WriteString(memoRecord, &off, separator);
		if(tempAdjust)
			DB_WriteString(memoRecord, &off, pTempString);
		
		err = MemPtrFree(pTempString);
		

	
		//write time
		DB_WriteString(memoRecord, &off, separator);	
		switch(item->time)
		{
			case 0:
				DB_WriteString(memoRecord, &off, "-2");
				break;
			case 1:
				DB_WriteString(memoRecord, &off, "-1.5");
				break;
			case 2:
				DB_WriteString(memoRecord, &off, "-1");
				break;
			case 3:
				DB_WriteString(memoRecord, &off, "-0.5");
				break;
			case 4:
				DB_WriteString(memoRecord, &off, "0");				
				break;
			case 5:
				DB_WriteString(memoRecord, &off, "0.5");				
				break;
			case 6:
				DB_WriteString(memoRecord, &off, "1");				
				break;
			case 7:
				DB_WriteString(memoRecord, &off, "1.5");				
				break;
			case 8:
				DB_WriteString(memoRecord, &off, "2.0");				
				break;		
			default:
				break;
		}
		
		//write fluid sensation
		if(g_prefs.fluidIndicator==0)//classic
		{
			switch(item->fluidSensation)
			{
			case 0:
				fluidTrans = item->fluidSensation;
				break;
			case 1:
			case 2:
			case 3:
				fluidTrans = item->fluidSensation - 4;
				break;
			case 12:
			case 13:
			case 4:
				fluidTrans = 1;
				break;
			case 16:
			case 15:
			case 5:
				fluidTrans = 2;
				break;
			case 18:
			case 6:
				fluidTrans = 3;
				break;
			default:
				fluidTrans = 0;  //to satisfy prc-tools warning
				break;						
			}
		}
		else //assume g_prefs.fluidIndicator==1 //Marquette
		{
			switch(item->fluidSensation)
			{
			case 0:
				fluidTrans = item->fluidSensation;
				break;
			case 1:
			case 2:
			case 3:
				fluidTrans = item->fluidSensation - 4;
				break;
			case 4:
				fluidTrans = 11; //add ten to all of Marquette ratings so they can be distinguished from classic
				break;
			case 12:
				fluidTrans = 12;
				break;	
			case 13:
				fluidTrans = 13;
				break;	
			case 5:
				fluidTrans = 14;
				break;			
			case 15:
				fluidTrans = 15;
				break;
			case 16:
				fluidTrans = 16;
				break;
			case 6:
				fluidTrans = 17;
				break;
			case 18:
				fluidTrans = 18;
				break;				
			default:
				fluidTrans = 0;  //to satisfy prc-tools warning
				break;						
			}
		}
		StrIToA(pString,fluidTrans);
		DB_WriteString(memoRecord, &off, separator);
		DB_WriteString(memoRecord, &off, pString);
		
		//write fluid color
		StrIToA(pString,item->fluidColor);
		DB_WriteString(memoRecord, &off, separator);
		DB_WriteString(memoRecord, &off, pString);
		
		//write cervix	
		StrIToA(pString,item->cervix);
		DB_WriteString(memoRecord, &off, separator);
		DB_WriteString(memoRecord, &off, pString);	
		
		//write LHTest
		DB_WriteString(memoRecord, &off, separator);	
		if(g_prefs.hormoneTest==0) //LH Test
		{
			switch(item->LHTest)
			{
			case 0:
				DB_WriteString(memoRecord, &off, "0");
				break;
			
			case 3:
			case 1:
				DB_WriteString(memoRecord, &off, "-");
				break;
			
			case 2:
				DB_WriteString(memoRecord, &off, "+");
				break;
			
			default:
				break;
			}
		}
		else //assume g_prefs.hormoneTest==1; Clearplan
		{
			switch(item->LHTest)
			{
			case 0:
				DB_WriteString(memoRecord, &off, "0");
				break;
			
			case 1:
				DB_WriteString(memoRecord, &off, "L");
				break;
			
			case 2:
				DB_WriteString(memoRecord, &off, "P");
				break;
			
			case 3:
				DB_WriteString(memoRecord, &off, "H");
				break;
			
			default:
				break;
			}
		}
		
		//write sex	
		DB_WriteString(memoRecord, &off, separator);
		switch(item->sex)
		{
			case 0:
				DB_WriteString(memoRecord, &off, "N");
				break;
			
			case 1:
				DB_WriteString(memoRecord, &off, "Y");
				break;
			
			case 2:
				DB_WriteString(memoRecord, &off, "P");
				break;
			
			default:
				break;
		
		}
		
		//write other
		StrIToA(pString,item->other);
		DB_WriteString(memoRecord, &off, separator);
		DB_WriteString(memoRecord, &off, pString);
		
		//write alcohol
		StrIToA(pString,item->alcohol);
		DB_WriteString(memoRecord, &off, separator);
		DB_WriteString(memoRecord, &off, pString);
		
		//write note
		pTempString = &(item->note);
		StrNCopy(pString,pTempString,lengthOfpString);
		*(pString + lengthOfpString - 1)=NULL;
		DB_WriteString(memoRecord, &off, separator);
		DB_WriteString(memoRecord, &off, pString);				

		DataReleaseData();	
	}
	
	err = MemPtrFree(pString);	

    DmWrite(memoRecord, off, "", 1); /* write nul */
    off++;
    return (UInt16) off;
}


/*
 * Create a new memo to hold an exported record, and return a locked pointer
 * thereto.
 */
static void* /*SECT2*/ Export_CreateMemo(DmOpenRef *dbp, UInt16 *pidx, LocalID *memoDbID,
			       UInt16 *memoDbCard)
{    
    UInt32	recLen;
    Err		err;
    MemHandle	newHandle;
    void *     recPtr;

    if (!(*dbp = DmOpenDatabaseByTypeCreator(kMemoType, kMemoCreator, dmModeReadWrite)))
	goto failOut;

    err = DmOpenDatabaseInfo(*dbp, memoDbID, NULL, NULL, memoDbCard, NULL);
    if (err)
	goto failClose;

    recLen = kMaxExport;
    *pidx = dmMaxRecordIndex;
    newHandle = DmNewRecord(*dbp, pidx, recLen);
    if (!newHandle) 
	goto failClose;

    recPtr = MemHandleLock(newHandle);
    if (!recPtr) 
	goto failDelete;
	
    return recPtr;
    

 failDelete:
    DmDeleteRecord(*dbp, *pidx);    
    
 failClose:
    DmCloseDatabase(*dbp);

 failOut:
    return NULL;
}


/*
 * Finish off the memo record: resize, unlock, and commit.
 */
static Int16 SECT2 Export_Finish(DmOpenRef dbp, Int16 idx, Int16 size, void *recPtr)
{
    Boolean	dirty = true;
    
    MemPtrUnlock(recPtr);
    DmReleaseRecord(dbp, idx, dirty);

    if (!(DmResizeRecord(dbp, idx, size))) {
         /* XXX: Perhaps we should do this before releasing the
          * record?  I don't think it matters. */
	return 0;
    }
    
    DmCloseDatabase(dbp);
    return 1;
}


/* Export the cycle to a MemoPad record. */
Boolean ExportKey(	UInt16 cycleIndex, UInt16 itemStartIndex, UInt16 itemFinalIndex)
{
    void	*memoRecord;
    UInt16	idx;
    Int16	size;
    DmOpenRef	dbp;
    LocalID	memoDbID;
    UInt16	memoDbCard;
    Boolean status = false;

    if (!(memoRecord = Export_CreateMemo(&dbp, &idx, &memoDbID, &memoDbCard)))
		Export_Failure();
    else if (!(size = Export_BuildText(cycleIndex, itemStartIndex, itemFinalIndex, memoRecord)))
		Export_Failure();
    else if (!Export_Finish(dbp, idx, size, memoRecord))
		Export_Failure();
	else
		status = true;
    
    return status;
}
