//    Copyright (C) 2003 Chad Brassil
//    This file is part of NFP.
//
//    NFP is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Foobar is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Foobar; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include <PalmOS.h>

// Handspring SDK headers
//#include <HSExt.h>
#include <SysEvtMgr.h>

#include "NFP.h"
#include "NFPRsc.h"

static void SECT1 SearchDraw (DateType date,Char* desc, Int16 x, Int16 y, Int16 width)
{
	UInt16 len;
	UInt16 maxNoteWidth;
	const Char* lineFeedP;
	Char dateStr [dateStringLength];
	DateFormatType dateFormat;
	Coord drawX;
	
	maxNoteWidth = width - 42;
	
	lineFeedP = StrChr(desc, chrLineFeed);
	if (lineFeedP != NULL)
	{
		len = lineFeedP - desc;
	}
	else
	{
		len = StrLen(desc);
	}

	WinDrawTruncChars(desc, len, x, y, maxNoteWidth);
	
	// Get date format from the system preferences.
	dateFormat = (DateFormatType)PrefGetPreference(prefDateFormat);
	DateToAscii(date.month, date.day, date.year + firstYear, dateFormat, dateStr);
	len = StrLen (dateStr);
	drawX = x + (width - FntCharsWidth (dateStr, len));		
	WinDrawChars (dateStr, len, drawX, y);
	
	return;
}


void Search (FindParamsPtr findParams)
{
	Err err;
	UInt16 pos;
	Char* desc;
	//Char* note;
	Char* header;
	UInt16 recordNum;
	MemHandle recordH;
	MemHandle headerH;
	RectangleType r;
	Boolean done;
	Boolean match;
	LocalID dbID;
	DmOpenRef dbP;
	NFPDBRecord *item;
	//ToDoDBRecordPtr toDoRec;
	UInt16 cardNo = 0;
	DmSearchStateType	searchState;
	UInt32 longPos;
	UInt16 matchLength;

	// Find the application's data file.
	err = DmGetNextDatabaseByTypeCreator (true, &searchState, RECORD_DB_TYPE,
					appFileCreator, true, &cardNo, &dbID);
	if (err)
		{
		findParams->more = false;
		return;
		}

	// Open the ToDo database.
	dbP = DmOpenDatabase(cardNo, dbID, findParams->dbAccesMode);
	if (!dbP) 
		{
		findParams->more = false;
		return;
		}

	// Display the heading line.
	headerH = DmGetResource (strRsc, FindNFPHeaderString);
	header = (Char*)MemHandleLock (headerH);
	done = FindDrawHeader (findParams, header);
	MemHandleUnlock(headerH);
	if (done)
		goto Exit;

	// Search the note fields for the "find" string.
	recordNum = findParams->recordNum;
	while (true)
		{
		// Because applications can take a long time to finish a find when
		// the result may be on the screen or for other reasons, users like
		// to be able to stop the find.  Stop the find if an event is pending.
		// This stops if the user does something with the device.  Because
		// this call slows winDown the search we perform it every so many 
		// records instead of every record.  The response time should still
		// be Int16 without introducing much extra work to the search.
		
		// Note that in the implementation below, if the next 16th record is  
		// secret the check doesn't happen.  Generally, this shouldn't be a 
		// problem if most of the records are secret then the search 
		// won't take long.
		if ((recordNum & 0x000f) == 0 &&			// every 16th record
			EvtSysEventAvail(true))
			{
			// Stop the search process.
			findParams->more = true;
			break;
			}

		//recordH = DmQueryNextInCategory (dbP, &recordNum, dmAllCategories);
		recordH = DmQueryRecord (dbP, recordNum);

		// Have we run out of records?
		if (! recordH)
			{
			findParams->more = false;			
			break;
			}

		item = (NFPDBRecord*)MemHandleLock (recordH);
		 
		// Search the description field,  if a match is not found search the
		// note field.
		desc = &item->note;
		match = TxtFindString (desc, findParams->strToFind, &longPos, &matchLength);
		pos = longPos;
		
		if (match)
			{
			// Add the match to the find paramter block,  if there is no room to
			// display the match the following function will return true.
			done = FindSaveMatch (findParams, recordNum, pos, 0, matchLength, cardNo, dbID);
			if (done)
				{
				MemHandleUnlock (recordH);
				break;
				}

			// Get the bounds of the region where we will draw the results.
			FindGetLineBounds (findParams, &r);
			
			// Display the description.
			SearchDraw (item->recDate, desc, r.topLeft.x+1, r.topLeft.y, r.extent.x-2);

			findParams->lineNumber++;
			}

		MemHandleUnlock (recordH);
		recordNum++;
		}

Exit:
	DmCloseDatabase (dbP);	
}


void GoToItem (GoToParamsPtr goToParams, Boolean launchingApp)
{
	UInt16 formID;
	UInt16 recordNum;
//	UInt16 attr;
//	Int32 dateL;
//	Int32 todayL;
//	UInt32 uniqueID;
//	MemHandle recordH;
	EventType event;
//	DateTimeType today;
	//ToDoDBRecordPtr toDoRec;
//	NFPDBRecord *item;

	recordNum = goToParams->recordNum;
	
//	if (!DmQueryRecord(s_dbNFP, recordNum))
//	{
		// Record isn't accessible. This can happen when receiving a beam while in an
		// empty record. This prevents a fatal alert, but doesn't fix the off-by-one
		// error. (See DOLATER in sysAppLaunchCmdExgReceiveData case.)
//		if (!SeekRecord(&recordNum, 0, dmSeekBackward))
//			if (!SeekRecord(&recordNum, 0, dmSeekForward))
//			{
//				FrmAlert(secGotoInvalidRecordAlert);
//				FrmGotoForm(ListView);
//				return;
//			}
//	}
	
	// If the application is already running, close all the open forms.  If
	// the current record is blank, then it will be deleted, so we'll get 
	// the unique id of the record and use it to find the record index 
	// after all the forms are closed.
	if (! launchingApp)
		DataFindClose(&recordNum);
	//I don't think I need this, because a partially edited record will remain
	//open until I call FrmGotoForm in the GotoEventHandler, and
	//by then I will have converted it into currentDate
	
	//actually I do need to have the above lines because
	//when I close any currently opened DataEntryForm it uses currentDate
	//therefore I can't change currentDate until all forms have been closed

	formID = DataEntryForm;

	// Send an event to goto a form and select the matching text.
	MemSet (&event, sizeof(EventType), 0);

	//need this so that it knows to goto DataEntryForm for GotoEvent
//	event.eType = frmLoadEvent;
//	event.data.frmLoad.formID = formID;
//	EvtAddEventToQueue (&event);
 
	event.eType = frmGotoEvent;
	event.data.frmGoto.formID = formID;
	event.data.frmGoto.recordNum = recordNum;
//	event.data.frmGoto.matchPos = goToParams->matchPos;
//	event.data.frmGoto.matchLen = goToParams->matchCustom;
//	event.data.frmGoto.matchFieldNum = goToParams->matchFieldNum;
	EvtAddEventToQueue (&event);

}

//from find, goto	
Boolean GotoEventHandler(EventType * eventP)
{
    NFPDBRecord* item;
    
	//find date for recordNum
	item = DataQueryData(eventP->data.frmGoto.recordNum);		
		
	//set currentDate
	s_currentDate = item->recDate;
	DataReleaseData();
			
	FrmGotoForm(DataEntryForm);
	
	//The code below is from the ToDo example; it selects the next
	//ItemSelected = true;
	//CurrentRecord = event->data.frmGoto.recordNum;
	//NoteViewInit (frm);

	//fld = GetObjectPtr (NoteField);
	//pos = event->data.frmGoto.matchPos;
	//FldSetScrollPosition (fld, pos);
	//FldSetSelection (fld, pos, pos + event->data.frmGoto.matchLen);
	//NoteViewDrawTitleAndForm (frm);
	//NoteViewUpdateScrollBar ();
	//FrmSetFocus (frm, FrmGetObjectIndex (frm, NoteField));

	return true;
}
